`timescale 1ns/1ns

module FFT_radix4 (
	input clk, clrn, start,
	input [3:0] iteration,
	input [15:0] n_2,
	input [31:0] datain,
	output [31:0] dataout,
	output overflow,
	input w_mem_cs,
	input w_mem_wr,
	input [15:0] w_mem_addr,
//	input [17:0] w_mem_din,
//	output [17:0] w_mem_dout);
	input [31:0] w_mem_din,
	output [31:0] w_mem_dout);
	
parameter dly = 1;

reg [3:0] counter;

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		counter	<= # dly 3'b0;
	end else if (start || |counter) begin
		counter	<= # dly counter + 1'b1;
	end
end

wire fft4_dav;

assign fft4_dav	= counter[0]&&(~counter[3]); //when counter[3] we are sending out the data and the Xs must stay stable

reg [31:0] datain_old;

always @(posedge clk) begin
	datain_old	<= # dly datain;
end

wire [33:0] X0_r, X1_r, X2_r, X3_r;
wire [33:0] X0_i, X1_i, X2_i, X3_i;

fft4 fft4_i(
	.clk(clk), 
	.dav(fft4_dav),
	.index(counter[2:1]),
	.x_r(datain_old), 
	.x_i(datain),
	.X0_r(X0_r), 
	.X0_i(X0_i),
	.X1_r(X1_r), 
	.X1_i(X1_i),
	.X2_r(X2_r), 
	.X2_i(X2_i),
	.X3_r(X3_r), 
	.X3_i(X3_i));

reg [35:0] mult_x_r, mult_x_i;	
wire mult_r_i;

assign mult_r_i	= ~counter[0];

always @(*) begin
	case (counter[2:1]) 
		2'b00: begin
			mult_x_r	= {X1_r, 2'b00};
			mult_x_i	= {X1_i, 2'b00};
		end
		2'b01: begin
			mult_x_r	= {X2_r, 2'b00};
			mult_x_i	= {X2_i, 2'b00};
		end
		default: begin
			mult_x_r	= {X3_r, 2'b00};
			mult_x_i	= {X3_i, 2'b00};
		end
	endcase
end

wire [35:0] mult_w_r, mult_w_i;

wire [72:0] half_mult_out;
	
half_compl_mult_36x36 half_compl_mult_i(
	.clk(clk),
	.x_r(mult_x_r), 
	.x_i(mult_x_i),
	.w_r(mult_w_r), 
	.w_i(mult_w_i),
	.r_i(mult_r_i),
	.result(half_mult_out));

reg overexteem;

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		overexteem	<= # dly 1'b0;
	end else if (counter[3]) begin
		case (counter[2:0])
			3'b000:	overexteem	<= # dly (X0_r[1:0]==2'b10) ? ~overexteem : overexteem;
			3'b001: overexteem	<= # dly (X0_r[1:0]==2'b10) ? ~overexteem : overexteem;
			default: overexteem	<= # dly (half_mult_out[37] && ~|half_mult_out[36:0]) ? ~overexteem : overexteem;
		endcase
	end
end

reg [35:0] actual_value;
reg [35:0] rounded_out;

always @(*) begin
	case (counter[2:0])
		3'b000:	actual_value	= {X0_r[33],X0_r[33], X0_r[33], X0_r[33], X0_r[33:2]}; 
		3'b001: actual_value	= {X0_i[33], X0_i[33], X0_i[33], X0_i[33], X0_i[33:2]};
		default: actual_value	= {half_mult_out[72], half_mult_out[72:38]};
	endcase
end

			
always @(*) begin
	case (counter[2:0])
		3'b000:		rounded_out	= (X0_r[1] && (X0_r[0] || overexteem)) ? actual_value+1'b1 : actual_value;
		3'b001:		rounded_out	= (X0_i[1] && (X0_i[0] || overexteem)) ? actual_value+1'b1 : actual_value;
		default:	rounded_out	= (half_mult_out[37] && (|half_mult_out[36:0] || overexteem)) ? actual_value+1'b1 : actual_value;
	endcase
end

wire en_overflow;

assign en_overflow	= counter[3];
assign overflow		= en_overflow && (|rounded_out[35:31] && ~&rounded_out[35:31]);

assign dataout	= overflow	? (rounded_out[35] ? 32'h8000_0000 : 32'h7fff_ffff) : rounded_out[31:0];

wire w_gen_start;

assign w_gen_start	= counter==4'h4;

wire w_mem_wr_int;

assign w_mem_wr_int	= w_mem_wr && w_mem_cs;

//w_generator	w_generator_i(
w_generator_34	w_generator_i(
	.clk(clk), 
	.start(w_gen_start), 
	.n_2(n_2), 
	.iteration(iteration),
//	.w_r(mult_w_r[35:16]), 
//	.w_i(mult_w_i[35:16]),
	.w_r(mult_w_r[35:2]), 
	.w_i(mult_w_i[35:2]),
	.w_mem_wr(w_mem_wr_int),
	.w_mem_addr(w_mem_addr),
	.w_mem_din(w_mem_din),
	.w_mem_dout(w_mem_dout)); 

//assign mult_w_r[15:0]	= 16'b0;
//assign mult_w_i[15:0]	= 16'h0;
assign mult_w_r[1:0]	= 2'b0;
assign mult_w_i[1:0]	= 2'h0;
					
endmodule