`timescale 1ns/1ns

module stx_vme_interface(
	input clk, clrn,
	
	input [4:0] vme_ga,
	input vme_gap,
	input vme_as, 
	input vme_ds1,
	input vme_ds2,
	input [5:0] vme_am,
	input vme_wr,
	output reg vme_dtack,
	input vme_lword,
	input [31:1] vme_addr,
	inout [31:0] vme_data,
	input 	vme_iackinn,
	input 	vme_iackn,
	output 	reg vme_iack_outn,
	output 	reg [6:1] vme_irqn,
	
	input 	irq_vector_enable,
	input 	[2:0] intlev_reg,
	output	reg irq_vector_n,
		
	output reg stx_245_oe_n,
	output reg stx_vme_accn,
	
	output reg wr_reg,
	output reg rd_reg,
	output reg [31:2] vme_addr_out,
	output reg [31:0] vme_data_out,
	input dav,
	input [31:0] data_in,
	
	input new_interrupt,
	input int_in_fifo,
	input rora_roak);
	
parameter dly = 1;

reg oe_vme_data;	

reg [1:0] ds_shr;
reg [1:0] as_shr;
wire as_int, pre_as_int, ds_int;

wire [4:0] base_addr;
wire gap_error;
wire selected;

reg dav_reg;
reg [31:0] data_in_reg;

wire valid_am;

reg clear_int;
reg [3:0] int_counter;

reg pre_wr_reg;
	
reg [2:0] state;

parameter s_idle	= 3'h0;
parameter s_read	= 3'h1;
parameter s_write	= 3'h2; 
parameter s_ack_int	= 3'h3;

assign gap_error	= ^{vme_ga, ~vme_gap}; 
assign base_addr	= gap_error	? 5'h1f : ~vme_ga;

assign valid_am	= (vme_am==6'h0F) || (vme_am==6'h0D) || (vme_am==6'h0B) || (vme_am==6'h09);

assign selected		= vme_iackn && ~as_int &&  ~ds_int && ~vme_lword && valid_am && (vme_addr_out[31:24]=={3'b0, base_addr});


always @(posedge clk) dav_reg	<= # dly dav;
always @(posedge clk) if (dav) data_in_reg	<= # dly data_in;

assign vme_data	= oe_vme_data	? data_in_reg : 32'hz;

assign pre_as_int	= as_shr[0];
assign as_int		= as_shr[1];
assign ds_int		= ds_shr[1];

always @(posedge clk) begin
	as_shr	<= # dly {as_shr[0], vme_as};
	ds_shr	<= # dly {ds_shr[0], vme_ds1 || vme_ds2};
end		


always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		int_counter	<= # dly 4'h0;
	end else begin
		if (new_interrupt && ~clear_int && ~&int_counter) begin
			int_counter	<= # dly int_counter + 1'b1;
		end else if (clear_int && ~new_interrupt && |int_counter) begin
			int_counter	<= # dly int_counter - 1'b1;
		end
	end
end			

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		vme_irqn	<= # dly 6'h3f;
	end else begin
		vme_irqn	<= # dly 6'h3f;
		if (irq_vector_enable && ((|int_counter && ~rora_roak) || (int_in_fifo && rora_roak))) begin
			vme_irqn[intlev_reg]	<= # dly 1'b0;
	 	end
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		state			<= # dly s_idle;
		oe_vme_data		<= # dly 1'b0;
		stx_245_oe_n	<= # dly 1'b1;
		stx_vme_accn	<= # dly 1'b1;
		vme_dtack		<= # dly 1'b1;
		wr_reg			<= # dly 1'b0;
		rd_reg			<= # dly 1'b0;
		vme_addr_out	<= # dly 30'h0;
		vme_data_out	<= # dly 32'h0;	
		pre_wr_reg		<= # dly 1'b0;
		vme_iack_outn	<= # dly 1'b1;
		irq_vector_n	<= # dly 1'b1;	
		clear_int		<= # dly 1'b0;		
	end else case (state)
		s_idle: begin
			pre_wr_reg		<= # dly 1'b0;
			state			<= # dly s_idle;
			oe_vme_data		<= # dly 1'b0;
			stx_245_oe_n	<= # dly 1'b1;
			stx_vme_accn	<= # dly 1'b1;
			vme_dtack		<= # dly 1'b1;
			wr_reg			<= # dly 1'b0;
			rd_reg			<= # dly 1'b0;
			vme_iack_outn	<= # dly 1'b1;
			clear_int		<= # dly 1'b0;
			irq_vector_n	<= # dly 1'b1;						
			if (as_int && ~pre_as_int) begin
				vme_addr_out	<= # dly vme_addr[31:2];	
			end
			if (selected) begin
				stx_vme_accn	<= # dly 1'b0;
				stx_245_oe_n	<= # dly 1'b0;
				if (vme_wr) begin
					state	<= # dly s_read;
					rd_reg	<= # dly 1'b1;
				end else begin
					state	<= # dly s_write;
				end
			end else if (~vme_iackinn) begin
				if (~as_int && vme_addr[3:1]==intlev_reg && |vme_irqn && ~vme_ds1 && vme_wr) begin
					state			<= # dly s_ack_int;
					irq_vector_n	<= # dly 1'b0;	
					stx_245_oe_n	<= # dly 1'b0;				
				end else if (~as_int) begin
					vme_iack_outn	<= # dly 1'b0;					
				end		
			end			
		end
		s_read: begin
			oe_vme_data		<= # dly 1'b1;
			rd_reg			<= # dly 1'b0;
			if (dav_reg) begin
				vme_dtack	<= # dly 1'b0;
			end
			if (~selected) begin
				state			<= # dly s_idle;
				stx_vme_accn	<= # dly 1'b1;
				stx_245_oe_n	<= # dly 1'b1;
				oe_vme_data		<= # dly 1'b0;
				vme_addr_out	<= # dly vme_addr_out + 1'b1;					
			end 
		end
		s_write: begin
			wr_reg	<= # dly pre_wr_reg;
			if (vme_dtack && ~pre_wr_reg) begin
				pre_wr_reg		<= # dly 1'b1; 
			end else begin
				pre_wr_reg		<= # dly 1'b0;
			end
			if (pre_wr_reg) begin
				vme_data_out	<= # dly vme_data;			
				vme_dtack		<= # dly 1'b0;
			end
			if (~selected) begin
				state			<= # dly s_idle;
				stx_vme_accn	<= # dly 1'b1;
				stx_245_oe_n	<= # dly 1'b1;
				vme_addr_out	<= # dly vme_addr_out + 1'b1;					
			end			
		end
		s_ack_int: begin
			if (as_int && vme_iackinn) begin
				state			<= # dly s_idle;
				clear_int		<= # dly 1'b1;
				irq_vector_n	<= # dly 1'b1;	
				vme_dtack		<= # dly 1'b1;					
			end else begin
				vme_dtack		<= # dly 1'b0;					
			end
		end
		default: begin
			state	<= # dly s_idle;
		end
	endcase
end
	
endmodule
	
	
	