`timescale 1ns/1ns

module codec_control(	input clk, clrn,
	input cs,
	input wr,
	input [17:0] addr, 
	input [31:0] data,
	
	input lrck, sclk, sdout, 
	output sdin,
	output spi_clk, spi_cs, spi_din,
	output reg rst,
	
	output reg [23:0] out_l, out_r,
	output adc_dav_l, adc_dav_r,
	
	output reg pre_mclk,
	input chirp_start_r, chirp_start_l);
	
parameter dly = 1;

parameter addr_freq_synth_right	= 2'd0;
parameter addr_freq_synth_left	= 2'd1;
parameter addr_conf_reg			= 2'd2;	

parameter addr_command_reg	= 16'h1;
parameter addr_mclk_div		= 16'h2;
parameter addr_f0_reg_r		= 16'h3;
parameter addr_f1_reg_r		= 16'h4;
parameter addr_f2_reg_r		= 16'h5;
parameter addr_df_reg_r		= 16'h6;
parameter addr_f0_reg_l		= 16'h7;
parameter addr_f1_reg_l		= 16'h8;
parameter addr_f2_reg_l		= 16'h9;
parameter addr_df_reg_l		= 16'ha;
parameter addr_spi_reg		= 16'h10;

parameter command_codec_rst_off		= 8'h0;
parameter command_codec_rst_on		= 8'h1;
parameter command_set_loopback_mode	= 8'h2;
parameter command_set_normal_mode	= 8'h3;
parameter command_enable_chirp_r	= 8'h4;
parameter command_disable_chirp_r	= 8'h5;
parameter command_enable_chirp_l	= 8'h6;
parameter command_disable_chirp_l	= 8'h7;


reg [31:0] phase_adv_left, f0_reg_l;
wire wr_left_mem;
wire [18:0] dout_left;
wire wr_left_ph_adv;

reg [31:0] phase_adv_right, f0_reg_r;
wire wr_right_mem;
wire [18:0] dout_right;
wire wr_right_ph_adv;

wire lword_req;		
reg lword_req_ff, load_lword;	

wire rword_req;		
reg rword_req_ff, load_rword;	

reg [23:0] new_word;

wire l_dav, r_dav;
reg l_dav_req, r_dav_req, adc_dav_l, adc_dav_r, adc_dav_l_int, adc_dav_r_int;
wire [23:0] adc_out_l, adc_out_r;

wire wr_spi;

reg [15:0] mclk_low_lenght, mclk_high_lenght, mclk_counter;

wire wr_df_reg_r, wr_f1_reg_r, wr_f2_reg_r;
reg [31:0] df_reg_r, f1_reg_r, f2_reg_r;

wire wr_df_reg_l, wr_f1_reg_l, wr_f2_reg_l;
reg [31:0] df_reg_l, f1_reg_l, f2_reg_l;

wire set_loopback, set_normal;
reg loopback_mode;

reg [7:0] loopback_counter;
wire lrck_int, sclk_int, sdout_int, sdin_int;

reg chirping_r, chirping_l;
reg enable_chirp_r, enable_chirp_l;


//LOOPBACK CONTROL

assign set_loopback		= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_command_reg && data[31:24]==command_set_loopback_mode;
assign set_normal		= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_command_reg && data[31:24]==command_set_normal_mode;

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		loopback_mode	<= # dly 1'b0;
	end else if (set_loopback) begin
		loopback_mode	<= # dly 1'b1;
	end else if (set_normal) begin
		loopback_mode	<= # dly 1'b0;
	end
end	

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		loopback_counter	<= # dly 8'b0;
	end else begin
		loopback_counter	<= # dly loopback_counter + 1'b1;
	end
end

assign lrck_int		= loopback_mode	? loopback_counter[7]	: lrck;
assign sclk_int		= loopback_mode	? loopback_counter[1]	: sclk;
assign sdout_int	= loopback_mode	? sdin_int				: sdout;
assign sdin			= sdin_int;

//RIGHT FREQUENCY SYNTHESIZER
		
assign wr_right_mem			= cs && wr && addr[17:16]==addr_freq_synth_right;
assign wr_right_ph_adv		= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_f0_reg_r;
assign wr_df_reg_r	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_df_reg_r;
assign wr_f1_reg_r	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_f1_reg_r;
assign wr_f2_reg_r	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_f2_reg_r;

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		enable_chirp_r	<= # dly 1'b0;
	end else if (cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_command_reg && data[31:24]==command_enable_chirp_r) begin
		enable_chirp_r	<= # dly 1'b1;
	end else if (cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_command_reg && data[31:24]==command_disable_chirp_r) begin
		enable_chirp_r	<= # dly 1'b0;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		df_reg_r	<= # dly 16'b0;
	end else if (wr_df_reg_r) begin
		df_reg_r	<= # dly data[15:0];
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		chirping_r	<= # dly 1'b0;
	end else if (enable_chirp_r && chirp_start_r) begin
		chirping_r	<= # dly 1'b1;
	end else if (phase_adv_right>=f2_reg_r) begin
		chirping_r	<= # dly 1'b0;
	end
end	

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		f1_reg_r	<= # dly 32'b0;
	end else if (wr_f1_reg_r) begin
		f1_reg_r	<= # dly data;
	end 
end
		
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		f2_reg_r	<= # dly 32'b0;
	end else if (wr_f2_reg_r) begin
		f2_reg_r	<= # dly data;
	end 
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		f0_reg_r	<= # dly 32'h0800_0000;
	end else if (wr_right_ph_adv) begin
		f0_reg_r	<= # dly data;
	end
end
		
always @(posedge clk) begin
	if (enable_chirp_r && chirp_start_r && ~chirping_r) begin
		phase_adv_right	<= # dly f1_reg_r;
	end else if (chirping_r) begin
		phase_adv_right <= # dly phase_adv_right + df_reg_r[15:0];
	end	else begin
		phase_adv_right	<= # dly f0_reg_r;
	end			
end
	
freq_synthesizer freq_synthesizer_right_i(  
	.clk(clk), 
	.clrn(clrn),
	.phase_adv(phase_adv_right),
	.wr(wr_right_mem),
	.addr(addr[15:0]),
	.data(data),
	.dout(dout_right));


//LEFT FREQUENCY SYNTHESIZER

assign wr_left_mem			= cs && wr && addr[17:16]==addr_freq_synth_left;
assign wr_left_ph_adv		= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_f0_reg_l;
assign wr_df_reg_l	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_df_reg_l;
assign wr_f1_reg_l	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_f1_reg_l;
assign wr_f2_reg_l	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_f2_reg_l;

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		enable_chirp_l	<= # dly 1'b0;
	end else if (cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_command_reg && data[31:24]==command_enable_chirp_l) begin
		enable_chirp_l	<= # dly 1'b1;
	end else if (cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_command_reg && data[31:24]==command_disable_chirp_l) begin
		enable_chirp_l	<= # dly 1'b0;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		df_reg_l	<= # dly 16'b0;
	end else if (wr_df_reg_l) begin
		df_reg_l	<= # dly data[15:0];
	end 
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		chirping_l	<= # dly 1'b0;
	end else if (enable_chirp_l && chirp_start_l) begin
		chirping_l	<= # dly 1'b1;
	end else if (phase_adv_left>=f2_reg_l) begin
		chirping_l	<= # dly 1'b0;
	end
end	


always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		f1_reg_l	<= # dly 32'b0;
	end else if (wr_f1_reg_l) begin
		f1_reg_l	<= # dly data;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		f2_reg_l	<= # dly 32'b0;
	end else if (wr_f2_reg_l) begin
		f2_reg_l	<= # dly data;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		f0_reg_l	<= # dly 32'h0800_0000;
	end else if (wr_left_ph_adv) begin
		f0_reg_l	<= # dly data;
	end
end
		
always @(posedge clk) begin
	if (enable_chirp_l && chirp_start_l && ~chirping_l) begin
		phase_adv_left	<= # dly f1_reg_l;
	end else if (chirping_l) begin
		phase_adv_left <= # dly phase_adv_left + df_reg_l[15:0];
	end	else begin
		phase_adv_left	<= # dly f0_reg_l;
	end			
end
	
freq_synthesizer freq_synthesizer_left_i(  
	.clk(clk), 
	.clrn(clrn),
	.phase_adv(phase_adv_left),
	.wr(wr_left_mem),
	.addr(addr[15:0]),
	.data(data),
	.dout(dout_left));

//DAC INTERFACE 

always @(posedge lword_req or posedge load_lword) begin
	if (load_lword) begin
		lword_req_ff	<= # dly 1'b0;
	end else begin
		lword_req_ff	<= # dly 1'b1;
	end
end

always @(posedge clk) begin
	load_lword	<= # dly lword_req_ff;
end

always @(posedge rword_req or posedge load_rword) begin
	if (load_rword) begin
		rword_req_ff	<= # dly 1'b0;
	end else begin
		rword_req_ff	<= # dly 1'b1;
	end
end

always @(posedge clk) begin
	load_rword	<= # dly rword_req_ff;
end

always @(posedge clk) begin
	if (load_lword) begin
		new_word	<= # dly {dout_left, 5'h0};
	end else if (load_rword) begin
		new_word	<= # dly {dout_right, 5'h0};
	end
end		
			
cs4272_dacs cs4272_dacs_i(
	.clrn(clrn), 
	.lrck(lrck_int), 
	.sclk(sclk_int), 
	.sdin(sdin_int),
	.lword_req(lword_req), 
	.rword_req(rword_req),
	.word(new_word));
	
//ADC INTERFACE

always @(posedge l_dav or posedge adc_dav_l_int) begin
	if (adc_dav_l_int) begin
		l_dav_req	<= # dly 1'b0;
	end else begin	
		l_dav_req	<= # dly 1'b1;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		adc_dav_l_int	<= # dly 1'b0;
	end else begin
		adc_dav_l_int	<= # dly l_dav_req;
	end
end

always @(posedge r_dav or posedge adc_dav_r_int) begin
	if (adc_dav_r_int) begin
		r_dav_req	<= # dly 1'b0;
	end else begin	
		r_dav_req	<= # dly 1'b1;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		adc_dav_r_int	<= # dly 1'b0;
	end else begin
		adc_dav_r_int	<= # dly r_dav_req;
	end
end

always @(posedge clk) begin
	adc_dav_l	<= # dly adc_dav_l_int;
	adc_dav_r	<= # dly adc_dav_r_int;	
	if (adc_dav_l_int) begin
		out_l	<= # dly adc_out_l;
	end
	if (adc_dav_r_int) begin
		out_r	<= # dly adc_out_r;
	end
end

cs4272_adcs cs4272_adcs_i( 
	.clrn(clrn), 
	.lrck(lrck_int), 
	.sclk(sclk_int), 
	.sdout(sdout_int),
	.l_dav(l_dav), 
	.r_dav(r_dav),
	.l_word(adc_out_l), 
	.r_word(adc_out_r));	
		
//SPI INTERFACE

assign wr_spi	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_spi_reg;

cs4272_spi_interface spi_interface_i(
	.clk(clk),
	.clrn(rst),
	.start(wr_spi),
	.addr(data[11:8]),
	.data_in(data[7:0]),
	.done(),
	.cs(spi_cs),
	.sclk(spi_clk),
	.sdi(spi_din));

//PRE MCLK
	
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		mclk_low_lenght		<= # dly 16'd63;
		mclk_high_lenght	<= # dly 16'd63;
	end else if (cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_mclk_div) begin
		mclk_low_lenght		<= # dly data[15:0];
		mclk_high_lenght	<= # dly data[31:16];
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		mclk_counter	<= # dly 16'h0;
		pre_mclk		<= # dly 1'b0;
	end else begin
		mclk_counter	<= # dly mclk_counter + 1'b1;
		if (pre_mclk && mclk_counter==mclk_high_lenght) begin
			pre_mclk		<= # dly 1'b0;
			mclk_counter	<= # dly 16'h0;
		end else if (~pre_mclk && mclk_counter==mclk_low_lenght) begin
			pre_mclk		<= # dly 1'b1;
			mclk_counter	<= # dly 16'h0;
		end else if (cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_mclk_div) begin
			mclk_counter	<= # dly 16'h0;
		end			
	end
end

//RST GENERATION


always@(posedge clk or negedge clrn) begin
	if (~clrn) begin
		rst	<= # dly 1'b0;
	end else begin
		if (cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_command_reg && data[31:24]==command_codec_rst_off) begin
			rst	<= # dly 1'b1;
		end else if (cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_command_reg && data[31:24]==command_codec_rst_on) begin
			rst	<= # dly 1'b0;
		end
	end
end		

endmodule