`timescale 1ns/1ns

module cordic_demodulator
	(input	clk,
	input 	clrn,
	
	input	[39:0] phase_pll,
	input	[41:0] ampl_pll,
	input	[41:0] ampl_i,
	input	[41:0] ampl_q,
	input 	start,
	
	output	reg done_pll,
	output	reg [35:0] cos_out_pll,
	output	reg [35:0] sin_out_pll,
	
	output	reg done_iq,
	output	reg [35:0] ampl_out_i,
	output	reg [39:0] phase_out_i,
	output	reg [35:0] ampl_out_q,
	output	reg [39:0] phase_out_q);

//parameter ampl_pll	= 42'h136_e9db_5080; //for maximum amplitude out on 36 bits
	
parameter atan_0	= 38'h2000000000;   
parameter atan_1	= 38'h12e4051d9e;   
parameter atan_2	= 38'h09fb385b5f;   
parameter atan_3	= 38'h051111d41e;
parameter atan_4	= 38'h028b0d430e;   
parameter atan_5	= 38'h0145d7e159;   
parameter atan_6	= 38'h00a2f61e5c;   
parameter atan_7	= 38'h00517c5512;
parameter atan_8	= 38'h0028be5347;   
parameter atan_9	= 38'h00145f2ebb;   
parameter atan_10	= 38'h000a2f9801;   
parameter atan_11	= 38'h000517cc15;
parameter atan_12	= 38'h00028be60d;  
parameter atan_13	= 38'h000145f307;   
parameter atan_14	= 38'h0000a2f983;   
parameter atan_15	= 38'h0000517cc2;
parameter atan_16	= 38'h000028be61;   
parameter atan_17	= 38'h0000145f30;   
parameter atan_18	= 38'h00000a2f98;   
parameter atan_19	= 38'h00000517cc;
parameter atan_20	= 38'h0000028be6;   
parameter atan_21	= 38'h00000145f3;   
parameter atan_22	= 38'h000000a2fa;   
parameter atan_23	= 38'h000000517d;
parameter atan_24	= 38'h00000028be;   
parameter atan_25	= 38'h000000145f;   
parameter atan_26	= 38'h0000000a30;   
parameter atan_27	= 38'h0000000518;
parameter atan_28	= 38'h000000028c;   
parameter atan_29	= 38'h0000000146;   
parameter atan_30	= 38'h00000000a3;   
parameter atan_31	= 38'h0000000051;
parameter atan_32	= 38'h0000000029;   
parameter atan_33	= 38'h0000000014;   
parameter atan_34	= 38'h000000000a;   
parameter atan_35	= 38'h0000000005;		

wire [38:0] atan [35:0];

assign atan[ 0]	= {1'b0, atan_0 };
assign atan[ 1]	= {1'b0, atan_1 };
assign atan[ 2]	= {1'b0, atan_2 };
assign atan[ 3]	= {1'b0, atan_3 };
assign atan[ 4]	= {1'b0, atan_4 };
assign atan[ 5]	= {1'b0, atan_5 };
assign atan[ 6]	= {1'b0, atan_6 };
assign atan[ 7]	= {1'b0, atan_7 };
assign atan[ 8]	= {1'b0, atan_8 };
assign atan[ 9]	= {1'b0, atan_9 };
assign atan[10]	= {1'b0, atan_10};   
assign atan[11]	= {1'b0, atan_11};
assign atan[12]	= {1'b0, atan_12};  
assign atan[13]	= {1'b0, atan_13};   
assign atan[14]	= {1'b0, atan_14};   
assign atan[15]	= {1'b0, atan_15};
assign atan[16]	= {1'b0, atan_16};   
assign atan[17]	= {1'b0, atan_17};   
assign atan[18]	= {1'b0, atan_18};   
assign atan[19]	= {1'b0, atan_19};
assign atan[20]	= {1'b0, atan_20};   
assign atan[21]	= {1'b0, atan_21};   
assign atan[22]	= {1'b0, atan_22};   
assign atan[23]	= {1'b0, atan_23};
assign atan[24]	= {1'b0, atan_24};   
assign atan[25]	= {1'b0, atan_25};   
assign atan[26]	= {1'b0, atan_26};   
assign atan[27]	= {1'b0, atan_27};
assign atan[28]	= {1'b0, atan_28};   
assign atan[29]	= {1'b0, atan_29};   
assign atan[30]	= {1'b0, atan_30};   
assign atan[31]	= {1'b0, atan_31};
assign atan[32]	= {1'b0, atan_32};   
assign atan[33]	= {1'b0, atan_33};   
assign atan[34]	= {1'b0, atan_34};   
assign atan[35]	= {1'b0, atan_35};


parameter dly=1;


reg [41:0] pre_sin_pll; //output precision + log2(output precision)
reg [41:0] pre_cos_pll;
reg [38:0] phase_acc_pll;

reg [41:0] pre_sin_i; //output precision + log2(output precision)
reg [41:0] pre_cos_i;

reg [41:0] pre_sin_q; //output precision + log2(output precision)
reg [41:0] pre_cos_q;


reg	[5:0] counter_pll;

reg [35:0] cos_out_i, sin_out_i, sin_out_q, cos_out_q;



//***********************************************
// SIN AND COS CALCULATION AND MIXER
//***********************************************

always @(posedge clk or negedge clrn) begin
	if (~clrn || counter_pll==6'd36) begin 
		counter_pll	<= # dly 6'd0;
	end else if (start || |counter_pll) begin
		counter_pll	<= # dly counter_pll + 1'b1;
	end   
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phase_acc_pll	<= # dly 39'd0;
	end else case (counter_pll)
		6'd0: 	phase_acc_pll	<= # dly phase_pll[38] ? phase_pll[38:0] + atan[0] : phase_pll[38:0] - atan[0];		
		6'd1: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[1] : phase_acc_pll - atan[1];	
		6'd2: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[2] : phase_acc_pll - atan[2];
		6'd3: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[3] : phase_acc_pll - atan[3];
		6'd4: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[4] : phase_acc_pll - atan[4];
		6'd5: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[5] : phase_acc_pll - atan[5];
		6'd6: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[6] : phase_acc_pll - atan[6];
		6'd7: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[7] : phase_acc_pll - atan[7];
		6'd8: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[8] : phase_acc_pll - atan[8];
		6'd9: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[9] : phase_acc_pll - atan[9];
		6'd10: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[10] : phase_acc_pll - atan[10];
		6'd11: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[11] : phase_acc_pll - atan[11];
		6'd12: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[12] : phase_acc_pll - atan[12];
		6'd13: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[13] : phase_acc_pll - atan[13];
		6'd14: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[14] : phase_acc_pll - atan[14];
		6'd15: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[15] : phase_acc_pll - atan[15];
		6'd16: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[16] : phase_acc_pll - atan[16];
		6'd17: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[17] : phase_acc_pll - atan[17];
		6'd18: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[18] : phase_acc_pll - atan[18];
		6'd19: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[19] : phase_acc_pll - atan[19];
		6'd20: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[20] : phase_acc_pll - atan[20];
		6'd21:	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[21] : phase_acc_pll - atan[21];
		6'd22: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[22] : phase_acc_pll - atan[22];
		6'd23:	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[23] : phase_acc_pll - atan[23];
		6'd24: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[24] : phase_acc_pll - atan[24];
		6'd25: 	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[25] : phase_acc_pll - atan[25];
		6'd26:	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[26] : phase_acc_pll - atan[26];
		6'd27:	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[27] : phase_acc_pll - atan[27];
		6'd28:	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[28] : phase_acc_pll - atan[28];
		6'd29:	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[29] : phase_acc_pll - atan[29];
		6'd30:	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[30] : phase_acc_pll - atan[30];
		6'd31:	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[31] : phase_acc_pll - atan[31];
		6'd32:	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[32] : phase_acc_pll - atan[32];
		6'd33:	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[33] : phase_acc_pll - atan[33];
		6'd34:	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[34] : phase_acc_pll - atan[34];
		6'd35:	phase_acc_pll	<= # dly phase_acc_pll[38] ? phase_acc_pll + atan[35] : phase_acc_pll - atan[35];
	endcase
end


always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		pre_cos_pll	<= # dly 42'd0;
		pre_sin_pll	<= # dly 42'd0;
	end else case (counter_pll)
		6'd0: begin
			pre_cos_pll	<= # dly ^phase_pll[39:38] ? -ampl_pll : ampl_pll;
			pre_sin_pll	<= # dly phase_pll[39] ? -ampl_pll : ampl_pll;		
		end
		6'd1: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{1{pre_sin_pll[41]}}, pre_sin_pll[41:1]} : pre_cos_pll - {{1{pre_sin_pll[41]}}, pre_sin_pll[41:1]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{1{pre_cos_pll[41]}}, pre_cos_pll[41:1]} : pre_sin_pll + {{1{pre_cos_pll[41]}}, pre_cos_pll[41:1]};
		end	
		6'd2: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{2{pre_sin_pll[41]}}, pre_sin_pll[41:2]} : pre_cos_pll - {{2{pre_sin_pll[41]}}, pre_sin_pll[41:2]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{2{pre_cos_pll[41]}}, pre_cos_pll[41:2]} : pre_sin_pll + {{2{pre_cos_pll[41]}}, pre_cos_pll[41:2]};
		end	
		6'd3: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{3{pre_sin_pll[41]}}, pre_sin_pll[41:3]} : pre_cos_pll - {{3{pre_sin_pll[41]}}, pre_sin_pll[41:3]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{3{pre_cos_pll[41]}}, pre_cos_pll[41:3]} : pre_sin_pll + {{3{pre_cos_pll[41]}}, pre_cos_pll[41:3]};
		end		
		6'd4: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{4{pre_sin_pll[41]}}, pre_sin_pll[41:4]} : pre_cos_pll - {{4{pre_sin_pll[41]}}, pre_sin_pll[41:4]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{4{pre_cos_pll[41]}}, pre_cos_pll[41:4]} : pre_sin_pll + {{4{pre_cos_pll[41]}}, pre_cos_pll[41:4]};
		end		
		6'd5: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{5{pre_sin_pll[41]}}, pre_sin_pll[41:5]} : pre_cos_pll - {{5{pre_sin_pll[41]}}, pre_sin_pll[41:5]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{5{pre_cos_pll[41]}}, pre_cos_pll[41:5]} : pre_sin_pll + {{5{pre_cos_pll[41]}}, pre_cos_pll[41:5]};
		end		
		6'd6: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{6{pre_sin_pll[41]}}, pre_sin_pll[41:6]} : pre_cos_pll - {{6{pre_sin_pll[41]}}, pre_sin_pll[41:6]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{6{pre_cos_pll[41]}}, pre_cos_pll[41:6]} : pre_sin_pll + {{6{pre_cos_pll[41]}}, pre_cos_pll[41:6]};
		end		
		6'd7: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{7{pre_sin_pll[41]}}, pre_sin_pll[41:7]} : pre_cos_pll - {{7{pre_sin_pll[41]}}, pre_sin_pll[41:7]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{7{pre_cos_pll[41]}}, pre_cos_pll[41:7]} : pre_sin_pll + {{7{pre_cos_pll[41]}}, pre_cos_pll[41:7]};
		end		
		6'd8: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{8{pre_sin_pll[41]}}, pre_sin_pll[41:8]} : pre_cos_pll - {{8{pre_sin_pll[41]}}, pre_sin_pll[41:8]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{8{pre_cos_pll[41]}}, pre_cos_pll[41:8]} : pre_sin_pll + {{8{pre_cos_pll[41]}}, pre_cos_pll[41:8]};
		end		
		6'd9: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{9{pre_sin_pll[41]}}, pre_sin_pll[41:9]} : pre_cos_pll - {{9{pre_sin_pll[41]}}, pre_sin_pll[41:9]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{9{pre_cos_pll[41]}}, pre_cos_pll[41:9]} : pre_sin_pll + {{9{pre_cos_pll[41]}}, pre_cos_pll[41:9]};
		end		
		6'd10: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{10{pre_sin_pll[41]}}, pre_sin_pll[41:10]} : pre_cos_pll - {{10{pre_sin_pll[41]}}, pre_sin_pll[41:10]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{10{pre_cos_pll[41]}}, pre_cos_pll[41:10]} : pre_sin_pll + {{10{pre_cos_pll[41]}}, pre_cos_pll[41:10]};
		end		
		6'd11: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{11{pre_sin_pll[41]}}, pre_sin_pll[41:11]} : pre_cos_pll - {{11{pre_sin_pll[41]}}, pre_sin_pll[41:11]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{11{pre_cos_pll[41]}}, pre_cos_pll[41:11]} : pre_sin_pll + {{11{pre_cos_pll[41]}}, pre_cos_pll[41:11]};
		end		
		6'd12: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{12{pre_sin_pll[41]}}, pre_sin_pll[41:12]} : pre_cos_pll - {{12{pre_sin_pll[41]}}, pre_sin_pll[41:12]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{12{pre_cos_pll[41]}}, pre_cos_pll[41:12]} : pre_sin_pll + {{12{pre_cos_pll[41]}}, pre_cos_pll[41:12]};
		end		
		6'd13: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{13{pre_sin_pll[41]}}, pre_sin_pll[41:13]} : pre_cos_pll - {{13{pre_sin_pll[41]}}, pre_sin_pll[41:13]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{13{pre_cos_pll[41]}}, pre_cos_pll[41:13]} : pre_sin_pll + {{13{pre_cos_pll[41]}}, pre_cos_pll[41:13]};
		end		
		6'd14: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{14{pre_sin_pll[41]}}, pre_sin_pll[41:14]} : pre_cos_pll - {{14{pre_sin_pll[41]}}, pre_sin_pll[41:14]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{14{pre_cos_pll[41]}}, pre_cos_pll[41:14]} : pre_sin_pll + {{14{pre_cos_pll[41]}}, pre_cos_pll[41:14]};
		end		
		6'd15: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{15{pre_sin_pll[41]}}, pre_sin_pll[41:15]} : pre_cos_pll - {{15{pre_sin_pll[41]}}, pre_sin_pll[41:15]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{15{pre_cos_pll[41]}}, pre_cos_pll[41:15]} : pre_sin_pll + {{15{pre_cos_pll[41]}}, pre_cos_pll[41:15]};
		end		
		6'd16: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{16{pre_sin_pll[41]}}, pre_sin_pll[41:16]} : pre_cos_pll - {{16{pre_sin_pll[41]}}, pre_sin_pll[41:16]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{16{pre_cos_pll[41]}}, pre_cos_pll[41:16]} : pre_sin_pll + {{16{pre_cos_pll[41]}}, pre_cos_pll[41:16]};
		end		
		6'd17: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{17{pre_sin_pll[41]}}, pre_sin_pll[41:17]} : pre_cos_pll - {{17{pre_sin_pll[41]}}, pre_sin_pll[41:17]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{17{pre_cos_pll[41]}}, pre_cos_pll[41:17]} : pre_sin_pll + {{17{pre_cos_pll[41]}}, pre_cos_pll[41:17]};
		end
		6'd18: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{18{pre_sin_pll[41]}}, pre_sin_pll[41:18]} : pre_cos_pll - {{18{pre_sin_pll[41]}}, pre_sin_pll[41:18]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{18{pre_cos_pll[41]}}, pre_cos_pll[41:18]} : pre_sin_pll + {{18{pre_cos_pll[41]}}, pre_cos_pll[41:18]};
		end		
		6'd19: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{19{pre_sin_pll[41]}}, pre_sin_pll[41:19]} : pre_cos_pll - {{19{pre_sin_pll[41]}}, pre_sin_pll[41:19]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{19{pre_cos_pll[41]}}, pre_cos_pll[41:19]} : pre_sin_pll + {{19{pre_cos_pll[41]}}, pre_cos_pll[41:19]};
		end		
		6'd20: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{20{pre_sin_pll[41]}}, pre_sin_pll[41:20]} : pre_cos_pll - {{20{pre_sin_pll[41]}}, pre_sin_pll[41:20]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{20{pre_cos_pll[41]}}, pre_cos_pll[41:20]} : pre_sin_pll + {{20{pre_cos_pll[41]}}, pre_cos_pll[41:20]};
		end		
		6'd21: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{21{pre_sin_pll[41]}}, pre_sin_pll[41:21]} : pre_cos_pll - {{21{pre_sin_pll[41]}}, pre_sin_pll[41:21]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{21{pre_cos_pll[41]}}, pre_cos_pll[41:21]} : pre_sin_pll + {{21{pre_cos_pll[41]}}, pre_cos_pll[41:21]};
		end		
		6'd22: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{22{pre_sin_pll[41]}}, pre_sin_pll[41:22]} : pre_cos_pll - {{22{pre_sin_pll[41]}}, pre_sin_pll[41:22]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{22{pre_cos_pll[41]}}, pre_cos_pll[41:22]} : pre_sin_pll + {{22{pre_cos_pll[41]}}, pre_cos_pll[41:22]};
		end	
		6'd23: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{23{pre_sin_pll[41]}}, pre_sin_pll[41:23]} : pre_cos_pll - {{23{pre_sin_pll[41]}}, pre_sin_pll[41:23]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{23{pre_cos_pll[41]}}, pre_cos_pll[41:23]} : pre_sin_pll + {{23{pre_cos_pll[41]}}, pre_cos_pll[41:23]};
		end	
		6'd24: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{24{pre_sin_pll[41]}}, pre_sin_pll[41:24]} : pre_cos_pll - {{24{pre_sin_pll[41]}}, pre_sin_pll[41:24]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{24{pre_cos_pll[41]}}, pre_cos_pll[41:24]} : pre_sin_pll + {{24{pre_cos_pll[41]}}, pre_cos_pll[41:24]};
		end	
		6'd25: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{25{pre_sin_pll[41]}}, pre_sin_pll[41:25]} : pre_cos_pll - {{25{pre_sin_pll[41]}}, pre_sin_pll[41:25]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{25{pre_cos_pll[41]}}, pre_cos_pll[41:25]} : pre_sin_pll + {{25{pre_cos_pll[41]}}, pre_cos_pll[41:25]};
		end		
		6'd26: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{26{pre_sin_pll[41]}}, pre_sin_pll[41:26]} : pre_cos_pll - {{26{pre_sin_pll[41]}}, pre_sin_pll[41:26]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{26{pre_cos_pll[41]}}, pre_cos_pll[41:26]} : pre_sin_pll + {{26{pre_cos_pll[41]}}, pre_cos_pll[41:26]};
		end		
		6'd27: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{27{pre_sin_pll[41]}}, pre_sin_pll[41:27]} : pre_cos_pll - {{27{pre_sin_pll[41]}}, pre_sin_pll[41:27]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{27{pre_cos_pll[41]}}, pre_cos_pll[41:27]} : pre_sin_pll + {{27{pre_cos_pll[41]}}, pre_cos_pll[41:27]};
		end		
		6'd28: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{28{pre_sin_pll[41]}}, pre_sin_pll[41:28]} : pre_cos_pll - {{28{pre_sin_pll[41]}}, pre_sin_pll[41:28]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{28{pre_cos_pll[41]}}, pre_cos_pll[41:28]} : pre_sin_pll + {{28{pre_cos_pll[41]}}, pre_cos_pll[41:28]};
		end		
		6'd29: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{29{pre_sin_pll[41]}}, pre_sin_pll[41:29]} : pre_cos_pll - {{29{pre_sin_pll[41]}}, pre_sin_pll[41:29]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{29{pre_cos_pll[41]}}, pre_cos_pll[41:29]} : pre_sin_pll + {{29{pre_cos_pll[41]}}, pre_cos_pll[41:29]};
		end		
		6'd30: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{30{pre_sin_pll[41]}}, pre_sin_pll[41:30]} : pre_cos_pll - {{30{pre_sin_pll[41]}}, pre_sin_pll[41:30]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{30{pre_cos_pll[41]}}, pre_cos_pll[41:30]} : pre_sin_pll + {{30{pre_cos_pll[41]}}, pre_cos_pll[41:30]};
		end		
		6'd31: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{31{pre_sin_pll[41]}}, pre_sin_pll[41:31]} : pre_cos_pll - {{31{pre_sin_pll[41]}}, pre_sin_pll[41:31]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{31{pre_cos_pll[41]}}, pre_cos_pll[41:31]} : pre_sin_pll + {{31{pre_cos_pll[41]}}, pre_cos_pll[41:31]};
		end		
		6'd32: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{32{pre_sin_pll[41]}}, pre_sin_pll[41:32]} : pre_cos_pll - {{32{pre_sin_pll[41]}}, pre_sin_pll[41:32]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{32{pre_cos_pll[41]}}, pre_cos_pll[41:32]} : pre_sin_pll + {{32{pre_cos_pll[41]}}, pre_cos_pll[41:32]};
		end		
		6'd33: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{33{pre_sin_pll[41]}}, pre_sin_pll[41:33]} : pre_cos_pll - {{33{pre_sin_pll[41]}}, pre_sin_pll[41:33]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{33{pre_cos_pll[41]}}, pre_cos_pll[41:33]} : pre_sin_pll + {{33{pre_cos_pll[41]}}, pre_cos_pll[41:33]};
		end		
		6'd34: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{34{pre_sin_pll[41]}}, pre_sin_pll[41:34]} : pre_cos_pll - {{34{pre_sin_pll[41]}}, pre_sin_pll[41:34]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{34{pre_cos_pll[41]}}, pre_cos_pll[41:34]} : pre_sin_pll + {{34{pre_cos_pll[41]}}, pre_cos_pll[41:34]};
		end		
		6'd35: begin
			pre_cos_pll	<= # dly phase_acc_pll[38] ? pre_cos_pll + {{35{pre_sin_pll[41]}}, pre_sin_pll[41:35]} : pre_cos_pll - {{35{pre_sin_pll[41]}}, pre_sin_pll[41:35]};
			pre_sin_pll	<= # dly phase_acc_pll[38] ? pre_sin_pll - {{35{pre_cos_pll[41]}}, pre_cos_pll[41:35]} : pre_sin_pll + {{35{pre_cos_pll[41]}}, pre_cos_pll[41:35]};
		end
	endcase
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		pre_cos_i	<= # dly 42'd0;
		pre_sin_i	<= # dly 42'd0;
	end else case (counter_pll)
		6'd0: begin
			pre_cos_i	<= # dly ^phase_pll[39:38] ? -ampl_i : ampl_i;
			pre_sin_i	<= # dly phase_pll[39] ? -ampl_i : ampl_i;		
		end
		6'd1: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{1{pre_sin_i[41]}}, pre_sin_i[41:1]} : pre_cos_i - {{1{pre_sin_i[41]}}, pre_sin_i[41:1]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{1{pre_cos_i[41]}}, pre_cos_i[41:1]} : pre_sin_i + {{1{pre_cos_i[41]}}, pre_cos_i[41:1]};
		end	
		6'd2: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{2{pre_sin_i[41]}}, pre_sin_i[41:2]} : pre_cos_i - {{2{pre_sin_i[41]}}, pre_sin_i[41:2]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{2{pre_cos_i[41]}}, pre_cos_i[41:2]} : pre_sin_i + {{2{pre_cos_i[41]}}, pre_cos_i[41:2]};
		end	
		6'd3: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{3{pre_sin_i[41]}}, pre_sin_i[41:3]} : pre_cos_i - {{3{pre_sin_i[41]}}, pre_sin_i[41:3]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{3{pre_cos_i[41]}}, pre_cos_i[41:3]} : pre_sin_i + {{3{pre_cos_i[41]}}, pre_cos_i[41:3]};
		end		
		6'd4: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{4{pre_sin_i[41]}}, pre_sin_i[41:4]} : pre_cos_i - {{4{pre_sin_i[41]}}, pre_sin_i[41:4]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{4{pre_cos_i[41]}}, pre_cos_i[41:4]} : pre_sin_i + {{4{pre_cos_i[41]}}, pre_cos_i[41:4]};
		end		
		6'd5: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{5{pre_sin_i[41]}}, pre_sin_i[41:5]} : pre_cos_i - {{5{pre_sin_i[41]}}, pre_sin_i[41:5]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{5{pre_cos_i[41]}}, pre_cos_i[41:5]} : pre_sin_i + {{5{pre_cos_i[41]}}, pre_cos_i[41:5]};
		end		
		6'd6: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{6{pre_sin_i[41]}}, pre_sin_i[41:6]} : pre_cos_i - {{6{pre_sin_i[41]}}, pre_sin_i[41:6]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{6{pre_cos_i[41]}}, pre_cos_i[41:6]} : pre_sin_i + {{6{pre_cos_i[41]}}, pre_cos_i[41:6]};
		end		
		6'd7: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{7{pre_sin_i[41]}}, pre_sin_i[41:7]} : pre_cos_i - {{7{pre_sin_i[41]}}, pre_sin_i[41:7]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{7{pre_cos_i[41]}}, pre_cos_i[41:7]} : pre_sin_i + {{7{pre_cos_i[41]}}, pre_cos_i[41:7]};
		end		
		6'd8: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{8{pre_sin_i[41]}}, pre_sin_i[41:8]} : pre_cos_i - {{8{pre_sin_i[41]}}, pre_sin_i[41:8]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{8{pre_cos_i[41]}}, pre_cos_i[41:8]} : pre_sin_i + {{8{pre_cos_i[41]}}, pre_cos_i[41:8]};
		end		
		6'd9: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{9{pre_sin_i[41]}}, pre_sin_i[41:9]} : pre_cos_i - {{9{pre_sin_i[41]}}, pre_sin_i[41:9]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{9{pre_cos_i[41]}}, pre_cos_i[41:9]} : pre_sin_i + {{9{pre_cos_i[41]}}, pre_cos_i[41:9]};
		end		
		6'd10: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{10{pre_sin_i[41]}}, pre_sin_i[41:10]} : pre_cos_i - {{10{pre_sin_i[41]}}, pre_sin_i[41:10]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{10{pre_cos_i[41]}}, pre_cos_i[41:10]} : pre_sin_i + {{10{pre_cos_i[41]}}, pre_cos_i[41:10]};
		end		
		6'd11: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{11{pre_sin_i[41]}}, pre_sin_i[41:11]} : pre_cos_i - {{11{pre_sin_i[41]}}, pre_sin_i[41:11]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{11{pre_cos_i[41]}}, pre_cos_i[41:11]} : pre_sin_i + {{11{pre_cos_i[41]}}, pre_cos_i[41:11]};
		end		
		6'd12: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{12{pre_sin_i[41]}}, pre_sin_i[41:12]} : pre_cos_i - {{12{pre_sin_i[41]}}, pre_sin_i[41:12]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{12{pre_cos_i[41]}}, pre_cos_i[41:12]} : pre_sin_i + {{12{pre_cos_i[41]}}, pre_cos_i[41:12]};
		end		
		6'd13: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{13{pre_sin_i[41]}}, pre_sin_i[41:13]} : pre_cos_i - {{13{pre_sin_i[41]}}, pre_sin_i[41:13]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{13{pre_cos_i[41]}}, pre_cos_i[41:13]} : pre_sin_i + {{13{pre_cos_i[41]}}, pre_cos_i[41:13]};
		end		
		6'd14: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{14{pre_sin_i[41]}}, pre_sin_i[41:14]} : pre_cos_i - {{14{pre_sin_i[41]}}, pre_sin_i[41:14]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{14{pre_cos_i[41]}}, pre_cos_i[41:14]} : pre_sin_i + {{14{pre_cos_i[41]}}, pre_cos_i[41:14]};
		end		
		6'd15: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{15{pre_sin_i[41]}}, pre_sin_i[41:15]} : pre_cos_i - {{15{pre_sin_i[41]}}, pre_sin_i[41:15]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{15{pre_cos_i[41]}}, pre_cos_i[41:15]} : pre_sin_i + {{15{pre_cos_i[41]}}, pre_cos_i[41:15]};
		end		
		6'd16: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{16{pre_sin_i[41]}}, pre_sin_i[41:16]} : pre_cos_i - {{16{pre_sin_i[41]}}, pre_sin_i[41:16]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{16{pre_cos_i[41]}}, pre_cos_i[41:16]} : pre_sin_i + {{16{pre_cos_i[41]}}, pre_cos_i[41:16]};
		end		
		6'd17: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{17{pre_sin_i[41]}}, pre_sin_i[41:17]} : pre_cos_i - {{17{pre_sin_i[41]}}, pre_sin_i[41:17]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{17{pre_cos_i[41]}}, pre_cos_i[41:17]} : pre_sin_i + {{17{pre_cos_i[41]}}, pre_cos_i[41:17]};
		end
		6'd18: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{18{pre_sin_i[41]}}, pre_sin_i[41:18]} : pre_cos_i - {{18{pre_sin_i[41]}}, pre_sin_i[41:18]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{18{pre_cos_i[41]}}, pre_cos_i[41:18]} : pre_sin_i + {{18{pre_cos_i[41]}}, pre_cos_i[41:18]};
		end		
		6'd19: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{19{pre_sin_i[41]}}, pre_sin_i[41:19]} : pre_cos_i - {{19{pre_sin_i[41]}}, pre_sin_i[41:19]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{19{pre_cos_i[41]}}, pre_cos_i[41:19]} : pre_sin_i + {{19{pre_cos_i[41]}}, pre_cos_i[41:19]};
		end		
		6'd20: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{20{pre_sin_i[41]}}, pre_sin_i[41:20]} : pre_cos_i - {{20{pre_sin_i[41]}}, pre_sin_i[41:20]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{20{pre_cos_i[41]}}, pre_cos_i[41:20]} : pre_sin_i + {{20{pre_cos_i[41]}}, pre_cos_i[41:20]};
		end		
		6'd21: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{21{pre_sin_i[41]}}, pre_sin_i[41:21]} : pre_cos_i - {{21{pre_sin_i[41]}}, pre_sin_i[41:21]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{21{pre_cos_i[41]}}, pre_cos_i[41:21]} : pre_sin_i + {{21{pre_cos_i[41]}}, pre_cos_i[41:21]};
		end		
		6'd22: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{22{pre_sin_i[41]}}, pre_sin_i[41:22]} : pre_cos_i - {{22{pre_sin_i[41]}}, pre_sin_i[41:22]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{22{pre_cos_i[41]}}, pre_cos_i[41:22]} : pre_sin_i + {{22{pre_cos_i[41]}}, pre_cos_i[41:22]};
		end	
		6'd23: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{23{pre_sin_i[41]}}, pre_sin_i[41:23]} : pre_cos_i - {{23{pre_sin_i[41]}}, pre_sin_i[41:23]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{23{pre_cos_i[41]}}, pre_cos_i[41:23]} : pre_sin_i + {{23{pre_cos_i[41]}}, pre_cos_i[41:23]};
		end	
		6'd24: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{24{pre_sin_i[41]}}, pre_sin_i[41:24]} : pre_cos_i - {{24{pre_sin_i[41]}}, pre_sin_i[41:24]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{24{pre_cos_i[41]}}, pre_cos_i[41:24]} : pre_sin_i + {{24{pre_cos_i[41]}}, pre_cos_i[41:24]};
		end	
		6'd25: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{25{pre_sin_i[41]}}, pre_sin_i[41:25]} : pre_cos_i - {{25{pre_sin_i[41]}}, pre_sin_i[41:25]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{25{pre_cos_i[41]}}, pre_cos_i[41:25]} : pre_sin_i + {{25{pre_cos_i[41]}}, pre_cos_i[41:25]};
		end		
		6'd26: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{26{pre_sin_i[41]}}, pre_sin_i[41:26]} : pre_cos_i - {{26{pre_sin_i[41]}}, pre_sin_i[41:26]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{26{pre_cos_i[41]}}, pre_cos_i[41:26]} : pre_sin_i + {{26{pre_cos_i[41]}}, pre_cos_i[41:26]};
		end		
		6'd27: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{27{pre_sin_i[41]}}, pre_sin_i[41:27]} : pre_cos_i - {{27{pre_sin_i[41]}}, pre_sin_i[41:27]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{27{pre_cos_i[41]}}, pre_cos_i[41:27]} : pre_sin_i + {{27{pre_cos_i[41]}}, pre_cos_i[41:27]};
		end		
		6'd28: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{28{pre_sin_i[41]}}, pre_sin_i[41:28]} : pre_cos_i - {{28{pre_sin_i[41]}}, pre_sin_i[41:28]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{28{pre_cos_i[41]}}, pre_cos_i[41:28]} : pre_sin_i + {{28{pre_cos_i[41]}}, pre_cos_i[41:28]};
		end		
		6'd29: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{29{pre_sin_i[41]}}, pre_sin_i[41:29]} : pre_cos_i - {{29{pre_sin_i[41]}}, pre_sin_i[41:29]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{29{pre_cos_i[41]}}, pre_cos_i[41:29]} : pre_sin_i + {{29{pre_cos_i[41]}}, pre_cos_i[41:29]};
		end		
		6'd30: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{30{pre_sin_i[41]}}, pre_sin_i[41:30]} : pre_cos_i - {{30{pre_sin_i[41]}}, pre_sin_i[41:30]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{30{pre_cos_i[41]}}, pre_cos_i[41:30]} : pre_sin_i + {{30{pre_cos_i[41]}}, pre_cos_i[41:30]};
		end		
		6'd31: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{31{pre_sin_i[41]}}, pre_sin_i[41:31]} : pre_cos_i - {{31{pre_sin_i[41]}}, pre_sin_i[41:31]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{31{pre_cos_i[41]}}, pre_cos_i[41:31]} : pre_sin_i + {{31{pre_cos_i[41]}}, pre_cos_i[41:31]};
		end		
		6'd32: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{32{pre_sin_i[41]}}, pre_sin_i[41:32]} : pre_cos_i - {{32{pre_sin_i[41]}}, pre_sin_i[41:32]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{32{pre_cos_i[41]}}, pre_cos_i[41:32]} : pre_sin_i + {{32{pre_cos_i[41]}}, pre_cos_i[41:32]};
		end		
		6'd33: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{33{pre_sin_i[41]}}, pre_sin_i[41:33]} : pre_cos_i - {{33{pre_sin_i[41]}}, pre_sin_i[41:33]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{33{pre_cos_i[41]}}, pre_cos_i[41:33]} : pre_sin_i + {{33{pre_cos_i[41]}}, pre_cos_i[41:33]};
		end		
		6'd34: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{34{pre_sin_i[41]}}, pre_sin_i[41:34]} : pre_cos_i - {{34{pre_sin_i[41]}}, pre_sin_i[41:34]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{34{pre_cos_i[41]}}, pre_cos_i[41:34]} : pre_sin_i + {{34{pre_cos_i[41]}}, pre_cos_i[41:34]};
		end		
		6'd35: begin
			pre_cos_i	<= # dly phase_acc_pll[38] ? pre_cos_i + {{35{pre_sin_i[41]}}, pre_sin_i[41:35]} : pre_cos_i - {{35{pre_sin_i[41]}}, pre_sin_i[41:35]};
			pre_sin_i	<= # dly phase_acc_pll[38] ? pre_sin_i - {{35{pre_cos_i[41]}}, pre_cos_i[41:35]} : pre_sin_i + {{35{pre_cos_i[41]}}, pre_cos_i[41:35]};
		end
	endcase
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		pre_cos_q	<= # dly 42'd0;
		pre_sin_q	<= # dly 42'd0;
	end else case (counter_pll)
		6'd0: begin
			pre_cos_q	<= # dly ^phase_pll[39:38] ? -ampl_q : ampl_q;
			pre_sin_q	<= # dly phase_pll[39] ? -ampl_q : ampl_q;		
		end
		6'd1: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{1{pre_sin_q[41]}}, pre_sin_q[41:1]} : pre_cos_q - {{1{pre_sin_q[41]}}, pre_sin_q[41:1]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{1{pre_cos_q[41]}}, pre_cos_q[41:1]} : pre_sin_q + {{1{pre_cos_q[41]}}, pre_cos_q[41:1]};
		end	
		6'd2: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{2{pre_sin_q[41]}}, pre_sin_q[41:2]} : pre_cos_q - {{2{pre_sin_q[41]}}, pre_sin_q[41:2]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{2{pre_cos_q[41]}}, pre_cos_q[41:2]} : pre_sin_q + {{2{pre_cos_q[41]}}, pre_cos_q[41:2]};
		end	
		6'd3: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{3{pre_sin_q[41]}}, pre_sin_q[41:3]} : pre_cos_q - {{3{pre_sin_q[41]}}, pre_sin_q[41:3]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{3{pre_cos_q[41]}}, pre_cos_q[41:3]} : pre_sin_q + {{3{pre_cos_q[41]}}, pre_cos_q[41:3]};
		end		
		6'd4: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{4{pre_sin_q[41]}}, pre_sin_q[41:4]} : pre_cos_q - {{4{pre_sin_q[41]}}, pre_sin_q[41:4]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{4{pre_cos_q[41]}}, pre_cos_q[41:4]} : pre_sin_q + {{4{pre_cos_q[41]}}, pre_cos_q[41:4]};
		end		
		6'd5: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{5{pre_sin_q[41]}}, pre_sin_q[41:5]} : pre_cos_q - {{5{pre_sin_q[41]}}, pre_sin_q[41:5]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{5{pre_cos_q[41]}}, pre_cos_q[41:5]} : pre_sin_q + {{5{pre_cos_q[41]}}, pre_cos_q[41:5]};
		end		
		6'd6: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{6{pre_sin_q[41]}}, pre_sin_q[41:6]} : pre_cos_q - {{6{pre_sin_q[41]}}, pre_sin_q[41:6]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{6{pre_cos_q[41]}}, pre_cos_q[41:6]} : pre_sin_q + {{6{pre_cos_q[41]}}, pre_cos_q[41:6]};
		end		
		6'd7: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{7{pre_sin_q[41]}}, pre_sin_q[41:7]} : pre_cos_q - {{7{pre_sin_q[41]}}, pre_sin_q[41:7]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{7{pre_cos_q[41]}}, pre_cos_q[41:7]} : pre_sin_q + {{7{pre_cos_q[41]}}, pre_cos_q[41:7]};
		end		
		6'd8: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{8{pre_sin_q[41]}}, pre_sin_q[41:8]} : pre_cos_q - {{8{pre_sin_q[41]}}, pre_sin_q[41:8]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{8{pre_cos_q[41]}}, pre_cos_q[41:8]} : pre_sin_q + {{8{pre_cos_q[41]}}, pre_cos_q[41:8]};
		end		
		6'd9: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{9{pre_sin_q[41]}}, pre_sin_q[41:9]} : pre_cos_q - {{9{pre_sin_q[41]}}, pre_sin_q[41:9]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{9{pre_cos_q[41]}}, pre_cos_q[41:9]} : pre_sin_q + {{9{pre_cos_q[41]}}, pre_cos_q[41:9]};
		end		
		6'd10: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{10{pre_sin_q[41]}}, pre_sin_q[41:10]} : pre_cos_q - {{10{pre_sin_q[41]}}, pre_sin_q[41:10]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{10{pre_cos_q[41]}}, pre_cos_q[41:10]} : pre_sin_q + {{10{pre_cos_q[41]}}, pre_cos_q[41:10]};
		end		
		6'd11: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{11{pre_sin_q[41]}}, pre_sin_q[41:11]} : pre_cos_q - {{11{pre_sin_q[41]}}, pre_sin_q[41:11]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{11{pre_cos_q[41]}}, pre_cos_q[41:11]} : pre_sin_q + {{11{pre_cos_q[41]}}, pre_cos_q[41:11]};
		end		
		6'd12: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{12{pre_sin_q[41]}}, pre_sin_q[41:12]} : pre_cos_q - {{12{pre_sin_q[41]}}, pre_sin_q[41:12]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{12{pre_cos_q[41]}}, pre_cos_q[41:12]} : pre_sin_q + {{12{pre_cos_q[41]}}, pre_cos_q[41:12]};
		end		
		6'd13: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{13{pre_sin_q[41]}}, pre_sin_q[41:13]} : pre_cos_q - {{13{pre_sin_q[41]}}, pre_sin_q[41:13]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{13{pre_cos_q[41]}}, pre_cos_q[41:13]} : pre_sin_q + {{13{pre_cos_q[41]}}, pre_cos_q[41:13]};
		end		
		6'd14: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{14{pre_sin_q[41]}}, pre_sin_q[41:14]} : pre_cos_q - {{14{pre_sin_q[41]}}, pre_sin_q[41:14]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{14{pre_cos_q[41]}}, pre_cos_q[41:14]} : pre_sin_q + {{14{pre_cos_q[41]}}, pre_cos_q[41:14]};
		end		
		6'd15: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{15{pre_sin_q[41]}}, pre_sin_q[41:15]} : pre_cos_q - {{15{pre_sin_q[41]}}, pre_sin_q[41:15]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{15{pre_cos_q[41]}}, pre_cos_q[41:15]} : pre_sin_q + {{15{pre_cos_q[41]}}, pre_cos_q[41:15]};
		end		
		6'd16: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{16{pre_sin_q[41]}}, pre_sin_q[41:16]} : pre_cos_q - {{16{pre_sin_q[41]}}, pre_sin_q[41:16]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{16{pre_cos_q[41]}}, pre_cos_q[41:16]} : pre_sin_q + {{16{pre_cos_q[41]}}, pre_cos_q[41:16]};
		end		
		6'd17: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{17{pre_sin_q[41]}}, pre_sin_q[41:17]} : pre_cos_q - {{17{pre_sin_q[41]}}, pre_sin_q[41:17]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{17{pre_cos_q[41]}}, pre_cos_q[41:17]} : pre_sin_q + {{17{pre_cos_q[41]}}, pre_cos_q[41:17]};
		end
		6'd18: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{18{pre_sin_q[41]}}, pre_sin_q[41:18]} : pre_cos_q - {{18{pre_sin_q[41]}}, pre_sin_q[41:18]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{18{pre_cos_q[41]}}, pre_cos_q[41:18]} : pre_sin_q + {{18{pre_cos_q[41]}}, pre_cos_q[41:18]};
		end		
		6'd19: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{19{pre_sin_q[41]}}, pre_sin_q[41:19]} : pre_cos_q - {{19{pre_sin_q[41]}}, pre_sin_q[41:19]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{19{pre_cos_q[41]}}, pre_cos_q[41:19]} : pre_sin_q + {{19{pre_cos_q[41]}}, pre_cos_q[41:19]};
		end		
		6'd20: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{20{pre_sin_q[41]}}, pre_sin_q[41:20]} : pre_cos_q - {{20{pre_sin_q[41]}}, pre_sin_q[41:20]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{20{pre_cos_q[41]}}, pre_cos_q[41:20]} : pre_sin_q + {{20{pre_cos_q[41]}}, pre_cos_q[41:20]};
		end		
		6'd21: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{21{pre_sin_q[41]}}, pre_sin_q[41:21]} : pre_cos_q - {{21{pre_sin_q[41]}}, pre_sin_q[41:21]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{21{pre_cos_q[41]}}, pre_cos_q[41:21]} : pre_sin_q + {{21{pre_cos_q[41]}}, pre_cos_q[41:21]};
		end		
		6'd22: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{22{pre_sin_q[41]}}, pre_sin_q[41:22]} : pre_cos_q - {{22{pre_sin_q[41]}}, pre_sin_q[41:22]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{22{pre_cos_q[41]}}, pre_cos_q[41:22]} : pre_sin_q + {{22{pre_cos_q[41]}}, pre_cos_q[41:22]};
		end	
		6'd23: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{23{pre_sin_q[41]}}, pre_sin_q[41:23]} : pre_cos_q - {{23{pre_sin_q[41]}}, pre_sin_q[41:23]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{23{pre_cos_q[41]}}, pre_cos_q[41:23]} : pre_sin_q + {{23{pre_cos_q[41]}}, pre_cos_q[41:23]};
		end	
		6'd24: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{24{pre_sin_q[41]}}, pre_sin_q[41:24]} : pre_cos_q - {{24{pre_sin_q[41]}}, pre_sin_q[41:24]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{24{pre_cos_q[41]}}, pre_cos_q[41:24]} : pre_sin_q + {{24{pre_cos_q[41]}}, pre_cos_q[41:24]};
		end	
		6'd25: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{25{pre_sin_q[41]}}, pre_sin_q[41:25]} : pre_cos_q - {{25{pre_sin_q[41]}}, pre_sin_q[41:25]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{25{pre_cos_q[41]}}, pre_cos_q[41:25]} : pre_sin_q + {{25{pre_cos_q[41]}}, pre_cos_q[41:25]};
		end		
		6'd26: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{26{pre_sin_q[41]}}, pre_sin_q[41:26]} : pre_cos_q - {{26{pre_sin_q[41]}}, pre_sin_q[41:26]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{26{pre_cos_q[41]}}, pre_cos_q[41:26]} : pre_sin_q + {{26{pre_cos_q[41]}}, pre_cos_q[41:26]};
		end		
		6'd27: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{27{pre_sin_q[41]}}, pre_sin_q[41:27]} : pre_cos_q - {{27{pre_sin_q[41]}}, pre_sin_q[41:27]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{27{pre_cos_q[41]}}, pre_cos_q[41:27]} : pre_sin_q + {{27{pre_cos_q[41]}}, pre_cos_q[41:27]};
		end		
		6'd28: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{28{pre_sin_q[41]}}, pre_sin_q[41:28]} : pre_cos_q - {{28{pre_sin_q[41]}}, pre_sin_q[41:28]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{28{pre_cos_q[41]}}, pre_cos_q[41:28]} : pre_sin_q + {{28{pre_cos_q[41]}}, pre_cos_q[41:28]};
		end		
		6'd29: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{29{pre_sin_q[41]}}, pre_sin_q[41:29]} : pre_cos_q - {{29{pre_sin_q[41]}}, pre_sin_q[41:29]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{29{pre_cos_q[41]}}, pre_cos_q[41:29]} : pre_sin_q + {{29{pre_cos_q[41]}}, pre_cos_q[41:29]};
		end		
		6'd30: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{30{pre_sin_q[41]}}, pre_sin_q[41:30]} : pre_cos_q - {{30{pre_sin_q[41]}}, pre_sin_q[41:30]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{30{pre_cos_q[41]}}, pre_cos_q[41:30]} : pre_sin_q + {{30{pre_cos_q[41]}}, pre_cos_q[41:30]};
		end		
		6'd31: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{31{pre_sin_q[41]}}, pre_sin_q[41:31]} : pre_cos_q - {{31{pre_sin_q[41]}}, pre_sin_q[41:31]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{31{pre_cos_q[41]}}, pre_cos_q[41:31]} : pre_sin_q + {{31{pre_cos_q[41]}}, pre_cos_q[41:31]};
		end		
		6'd32: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{32{pre_sin_q[41]}}, pre_sin_q[41:32]} : pre_cos_q - {{32{pre_sin_q[41]}}, pre_sin_q[41:32]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{32{pre_cos_q[41]}}, pre_cos_q[41:32]} : pre_sin_q + {{32{pre_cos_q[41]}}, pre_cos_q[41:32]};
		end		
		6'd33: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{33{pre_sin_q[41]}}, pre_sin_q[41:33]} : pre_cos_q - {{33{pre_sin_q[41]}}, pre_sin_q[41:33]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{33{pre_cos_q[41]}}, pre_cos_q[41:33]} : pre_sin_q + {{33{pre_cos_q[41]}}, pre_cos_q[41:33]};
		end		
		6'd34: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{34{pre_sin_q[41]}}, pre_sin_q[41:34]} : pre_cos_q - {{34{pre_sin_q[41]}}, pre_sin_q[41:34]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{34{pre_cos_q[41]}}, pre_cos_q[41:34]} : pre_sin_q + {{34{pre_cos_q[41]}}, pre_cos_q[41:34]};
		end		
		6'd35: begin
			pre_cos_q	<= # dly phase_acc_pll[38] ? pre_cos_q + {{35{pre_sin_q[41]}}, pre_sin_q[41:35]} : pre_cos_q - {{35{pre_sin_q[41]}}, pre_sin_q[41:35]};
			pre_sin_q	<= # dly phase_acc_pll[38] ? pre_sin_q - {{35{pre_cos_q[41]}}, pre_cos_q[41:35]} : pre_sin_q + {{35{pre_cos_q[41]}}, pre_cos_q[41:35]};
		end
	endcase
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin 	
		done_pll	<= # dly 1'b0;
		cos_out_pll	<= # dly 36'h0;
		sin_out_pll	<= # dly 36'h0;
		cos_out_i	<= # dly 36'h0;
		sin_out_i	<= # dly 36'h0;
		cos_out_q	<= # dly 36'h0;
		sin_out_q	<= # dly 36'h0;
	end else begin
		done_pll	<= # dly counter_pll==6'd36;	
		if (counter_pll==6'd36) begin 	
			cos_out_pll	<= # dly pre_cos_pll[41:6];
			sin_out_pll	<= # dly pre_sin_pll[41:6];
			cos_out_i	<= # dly cos_out_i - {{10{cos_out_i[35]}}, cos_out_i[35:10]} + {{10{pre_cos_i[41]}}, pre_cos_i[41:16]}; //LP IIR : H(z)=(1-a)/(1-a*z^-1) ; a=1-1/1024; 
			sin_out_i	<= # dly sin_out_i - {{10{sin_out_i[35]}}, sin_out_i[35:10]} + {{10{pre_sin_i[41]}}, pre_sin_i[41:16]}; //LP IIR : H(z)=(1-a)/(1-a*z^-1) ; a=1-1/1024; 
//			cos_out_i	<= # dly cos_out_i - {{4{cos_out_i[35]}}, cos_out_i[35:10], 6'b0} + {{4{pre_cos_i[41]}}, pre_cos_i[41:16], 6'b0}; //LP IIR : H(z)=(1-a)/(1-a*z^-1) ; a=1-1/1024; 
//			sin_out_i	<= # dly sin_out_i - {{4{sin_out_i[35]}}, sin_out_i[35:10], 6'b0} + {{4{pre_sin_i[41]}}, pre_sin_i[41:16], 6'b0}; //LP IIR : H(z)=(1-a)/(1-a*z^-1) ; a=1-1/1024; 
			cos_out_q	<= # dly cos_out_q - {{10{cos_out_q[35]}}, cos_out_q[35:10]} + {{10{pre_cos_q[41]}}, pre_cos_q[41:16]}; //LP IIR : H(z)=(1-a)/(1-a*z^-1) ; a=1-1/1024; 
			sin_out_q	<= # dly sin_out_q - {{10{sin_out_q[35]}}, sin_out_q[35:10]} + {{10{pre_sin_q[41]}}, pre_sin_q[41:16]}; //LP IIR : H(z)=(1-a)/(1-a*z^-1) ; a=1-1/1024; 
		end 	 
	end
end						


//***********************************************
// FROM RECTANGULAR TO POLAR
//***********************************************

reg	[5:0] counter_iq;
reg [39:0] phase_acc_i, phase_acc_q;
reg [41:0] ex_sin_i, ex_cos_i, ex_sin_q, ex_cos_q;

always @(posedge clk or negedge clrn) begin
	if (~clrn || counter_iq==6'd36) begin 
		counter_iq	<= # dly 6'd0;
	end else if (done_pll || |counter_iq) begin
		counter_iq	<= # dly counter_iq + 1'b1;
	end   
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phase_acc_i	<= # dly 40'd0;
	end else case (counter_iq)
		6'd0: begin
			if (cos_out_i[35]) begin
				phase_acc_i	<= # dly ~sin_out_i[35] ? 40'h80_0000_0000 - {1'b0, atan[0]} : 40'h80_0000_0000 + {1'b0, atan[0]};			
			end else begin	
				phase_acc_i	<= # dly sin_out_i[35] ? -{1'b0, atan[0]} : {1'b0, atan[0]};
			end
		end	
		6'd1: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[1] } : phase_acc_i + {1'b0, atan[1] };	
		6'd2: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[2] } : phase_acc_i + {1'b0, atan[2] };
		6'd3: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[3] } : phase_acc_i + {1'b0, atan[3] };
		6'd4: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[4] } : phase_acc_i + {1'b0, atan[4] };
		6'd5: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[5] } : phase_acc_i + {1'b0, atan[5] };
		6'd6: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[6] } : phase_acc_i + {1'b0, atan[6] };
		6'd7: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[7] } : phase_acc_i + {1'b0, atan[7] };
		6'd8: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[8] } : phase_acc_i + {1'b0, atan[8] };
		6'd9: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[9] } : phase_acc_i + {1'b0, atan[9] };
		6'd10: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[10]} : phase_acc_i + {1'b0, atan[10]};
		6'd11: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[11]} : phase_acc_i + {1'b0, atan[11]};
		6'd12: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[12]} : phase_acc_i + {1'b0, atan[12]};
		6'd13: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[13]} : phase_acc_i + {1'b0, atan[13]};
		6'd14: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[14]} : phase_acc_i + {1'b0, atan[14]};
		6'd15: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[15]} : phase_acc_i + {1'b0, atan[15]};
		6'd16: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[16]} : phase_acc_i + {1'b0, atan[16]};
		6'd17: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[17]} : phase_acc_i + {1'b0, atan[17]};
		6'd18: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[18]} : phase_acc_i + {1'b0, atan[18]};
		6'd19: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[19]} : phase_acc_i + {1'b0, atan[19]};
		6'd20: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[20]} : phase_acc_i + {1'b0, atan[20]};
		6'd21:	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[21]} : phase_acc_i + {1'b0, atan[21]};
		6'd22: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[22]} : phase_acc_i + {1'b0, atan[22]};
		6'd23:	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[23]} : phase_acc_i + {1'b0, atan[23]};
		6'd24: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[24]} : phase_acc_i + {1'b0, atan[24]};
		6'd25: 	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[25]} : phase_acc_i + {1'b0, atan[25]};
		6'd26:	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[26]} : phase_acc_i + {1'b0, atan[26]};
		6'd27:	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[27]} : phase_acc_i + {1'b0, atan[27]};
		6'd28:	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[28]} : phase_acc_i + {1'b0, atan[28]};
		6'd29:	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[29]} : phase_acc_i + {1'b0, atan[29]};
		6'd30:	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[30]} : phase_acc_i + {1'b0, atan[30]};
		6'd31:	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[31]} : phase_acc_i + {1'b0, atan[31]};
		6'd32:	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[32]} : phase_acc_i + {1'b0, atan[32]};
		6'd33:	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[33]} : phase_acc_i + {1'b0, atan[33]};
		6'd34:	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[34]} : phase_acc_i + {1'b0, atan[34]};
		6'd35:	phase_acc_i	<= # dly ex_sin_i[41] ? phase_acc_i - {1'b0, atan[35]} : phase_acc_i + {1'b0, atan[35]};
	endcase
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		ex_cos_i		<= # dly 42'd0;
		ex_sin_i		<= # dly 42'd0;
	end else case (counter_iq)
		6'd0: begin
			if (cos_out_i[35]) begin   
				ex_cos_i	<= # dly ~sin_out_i[35] ? -{ cos_out_i, 6'b0} + {sin_out_i, 6'b0} : -{cos_out_i, 6'b0} - {sin_out_i, 6'b0};
				ex_sin_i	<= # dly ~sin_out_i[35] ? -{sin_out_i, 6'b0} - {cos_out_i, 6'b0} : -{sin_out_i, 6'b0} + {cos_out_i, 6'b0};
			end else begin
				ex_cos_i	<= # dly sin_out_i[35] ? {cos_out_i, 6'b0} - {sin_out_i, 6'b0} : {cos_out_i, 6'b0} + {sin_out_i, 6'b0};
				ex_sin_i	<= # dly sin_out_i[35] ? {sin_out_i, 6'b0} + {cos_out_i, 6'b0} : {sin_out_i, 6'b0} - {cos_out_i, 6'b0};
			end		
		end			
		6'd1: begin
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{1{ex_sin_i[41]}}, ex_sin_i[41:1]} : ex_cos_i + {{1{ex_sin_i[41]}}, ex_sin_i[41:1]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{1{ex_cos_i[41]}}, ex_cos_i[41:1]} : ex_sin_i - {{1{ex_cos_i[41]}}, ex_cos_i[41:1]};
		end	        	                                                                                                                      
		6'd2: begin 	                                                                                                                      
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{2{ex_sin_i[41]}}, ex_sin_i[41:2]} : ex_cos_i + {{2{ex_sin_i[41]}}, ex_sin_i[41:2]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{2{ex_cos_i[41]}}, ex_cos_i[41:2]} : ex_sin_i - {{2{ex_cos_i[41]}}, ex_cos_i[41:2]};
		end	        	                                                                                                                      
		6'd3: begin 	                                                                                                                      
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{3{ex_sin_i[41]}}, ex_sin_i[41:3]} : ex_cos_i + {{3{ex_sin_i[41]}}, ex_sin_i[41:3]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{3{ex_cos_i[41]}}, ex_cos_i[41:3]} : ex_sin_i - {{3{ex_cos_i[41]}}, ex_cos_i[41:3]};
		end		    	                                                                                                                      
		6'd4: begin 	                                                                                                                      
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{4{ex_sin_i[41]}}, ex_sin_i[41:4]} : ex_cos_i + {{4{ex_sin_i[41]}}, ex_sin_i[41:4]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{4{ex_cos_i[41]}}, ex_cos_i[41:4]} : ex_sin_i - {{4{ex_cos_i[41]}}, ex_cos_i[41:4]};
		end		    	                                                                                                                      
		6'd5: begin 	                                                                                                                      
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{5{ex_sin_i[41]}}, ex_sin_i[41:5]} : ex_cos_i + {{5{ex_sin_i[41]}}, ex_sin_i[41:5]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{5{ex_cos_i[41]}}, ex_cos_i[41:5]} : ex_sin_i - {{5{ex_cos_i[41]}}, ex_cos_i[41:5]};
		end		    	                                                                                                                     
		6'd6: begin 	                                                                                                                      
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{6{ex_sin_i[41]}}, ex_sin_i[41:6]} : ex_cos_i + {{6{ex_sin_i[41]}}, ex_sin_i[41:6]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{6{ex_cos_i[41]}}, ex_cos_i[41:6]} : ex_sin_i - {{6{ex_cos_i[41]}}, ex_cos_i[41:6]};
		end		    	                                                                                                                      
		6'd7: begin 	                                                                                                                      
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{7{ex_sin_i[41]}}, ex_sin_i[41:7]} : ex_cos_i + {{7{ex_sin_i[41]}}, ex_sin_i[41:7]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{7{ex_cos_i[41]}}, ex_cos_i[41:7]} : ex_sin_i - {{7{ex_cos_i[41]}}, ex_cos_i[41:7]};
		end		    	                                                                                                                      
		6'd8: begin 	                                                                                                                      
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{8{ex_sin_i[41]}}, ex_sin_i[41:8]} : ex_cos_i + {{8{ex_sin_i[41]}}, ex_sin_i[41:8]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{8{ex_cos_i[41]}}, ex_cos_i[41:8]} : ex_sin_i - {{8{ex_cos_i[41]}}, ex_cos_i[41:8]};
		end		    	                                                                                                                      
		6'd9: begin 	                                                                                                                      
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{9{ex_sin_i[41]}}, ex_sin_i[41:9]} : ex_cos_i + {{9{ex_sin_i[41]}}, ex_sin_i[41:9]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{9{ex_cos_i[41]}}, ex_cos_i[41:9]} : ex_sin_i - {{9{ex_cos_i[41]}}, ex_cos_i[41:9]};
		end		    	
		6'd10: begin	
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{10{ex_sin_i[41]}}, ex_sin_i[41:10]} :  ex_cos_i + {{10{ex_sin_i[41]}}, ex_sin_i[41:10]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{10{ex_cos_i[41]}}, ex_cos_i[41:10]} :  ex_sin_i - {{10{ex_cos_i[41]}}, ex_cos_i[41:10]};
		end		    	                                                                                                                           
		6'd11: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{11{ex_sin_i[41]}}, ex_sin_i[41:11]} : ex_cos_i + {{11{ex_sin_i[41]}}, ex_sin_i[41:11]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{11{ex_cos_i[41]}}, ex_cos_i[41:11]} : ex_sin_i - {{11{ex_cos_i[41]}}, ex_cos_i[41:11]};
		end		    	                                                                                                                           
		6'd12: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{12{ex_sin_i[41]}}, ex_sin_i[41:12]} : ex_cos_i + {{12{ex_sin_i[41]}}, ex_sin_i[41:12]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{12{ex_cos_i[41]}}, ex_cos_i[41:12]} : ex_sin_i - {{12{ex_cos_i[41]}}, ex_cos_i[41:12]};
		end		    	                                                                                                                           
		6'd13: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{13{ex_sin_i[41]}}, ex_sin_i[41:13]} : ex_cos_i + {{13{ex_sin_i[41]}}, ex_sin_i[41:13]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{13{ex_cos_i[41]}}, ex_cos_i[41:13]} : ex_sin_i - {{13{ex_cos_i[41]}}, ex_cos_i[41:13]};
		end		    	                                                                                                                           
		6'd14: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{14{ex_sin_i[41]}}, ex_sin_i[41:14]} : ex_cos_i + {{14{ex_sin_i[41]}}, ex_sin_i[41:14]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{14{ex_cos_i[41]}}, ex_cos_i[41:14]} : ex_sin_i - {{14{ex_cos_i[41]}}, ex_cos_i[41:14]};
		end		    	                                                                                                                           
		6'd15: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{15{ex_sin_i[41]}}, ex_sin_i[41:15]} : ex_cos_i + {{15{ex_sin_i[41]}}, ex_sin_i[41:15]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{15{ex_cos_i[41]}}, ex_cos_i[41:15]} : ex_sin_i - {{15{ex_cos_i[41]}}, ex_cos_i[41:15]};
		end		    	                                                                                                                           
		6'd16: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{16{ex_sin_i[41]}}, ex_sin_i[41:16]} : ex_cos_i + {{16{ex_sin_i[41]}}, ex_sin_i[41:16]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{16{ex_cos_i[41]}}, ex_cos_i[41:16]} : ex_sin_i - {{16{ex_cos_i[41]}}, ex_cos_i[41:16]};
		end		    	                                                                                                                           
		6'd17: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{17{ex_sin_i[41]}}, ex_sin_i[41:17]} : ex_cos_i + {{17{ex_sin_i[41]}}, ex_sin_i[41:17]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{17{ex_cos_i[41]}}, ex_cos_i[41:17]} : ex_sin_i - {{17{ex_cos_i[41]}}, ex_cos_i[41:17]};
		end         	                                                                                                                           
		6'd18: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{18{ex_sin_i[41]}}, ex_sin_i[41:18]} : ex_cos_i + {{18{ex_sin_i[41]}}, ex_sin_i[41:18]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{18{ex_cos_i[41]}}, ex_cos_i[41:18]} : ex_sin_i - {{18{ex_cos_i[41]}}, ex_cos_i[41:18]};
		end		    	                                                                                                                        
		6'd19: begin	                                                                                                                        
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{19{ex_sin_i[41]}}, ex_sin_i[41:19]} : ex_cos_i + {{19{ex_sin_i[41]}}, ex_sin_i[41:19]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{19{ex_cos_i[41]}}, ex_cos_i[41:19]} : ex_sin_i - {{19{ex_cos_i[41]}}, ex_cos_i[41:19]};
		end		    	                                                                                                                           
		6'd20: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{20{ex_sin_i[41]}}, ex_sin_i[41:20]} : ex_cos_i + {{20{ex_sin_i[41]}}, ex_sin_i[41:20]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{20{ex_cos_i[41]}}, ex_cos_i[41:20]} : ex_sin_i - {{20{ex_cos_i[41]}}, ex_cos_i[41:20]};
		end		    	                                                                                                                           
		6'd21: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{21{ex_sin_i[41]}}, ex_sin_i[41:21]} : ex_cos_i + {{21{ex_sin_i[41]}}, ex_sin_i[41:21]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{21{ex_cos_i[41]}}, ex_cos_i[41:21]} : ex_sin_i - {{21{ex_cos_i[41]}}, ex_cos_i[41:21]};
		end		    	                                                                                                                           
		6'd22: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{22{ex_sin_i[41]}}, ex_sin_i[41:22]} : ex_cos_i + {{22{ex_sin_i[41]}}, ex_sin_i[41:22]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{22{ex_cos_i[41]}}, ex_cos_i[41:22]} : ex_sin_i - {{22{ex_cos_i[41]}}, ex_cos_i[41:22]};
		end	        	                                                                                                                           
		6'd23: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{23{ex_sin_i[41]}}, ex_sin_i[41:23]} : ex_cos_i + {{23{ex_sin_i[41]}}, ex_sin_i[41:23]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{23{ex_cos_i[41]}}, ex_cos_i[41:23]} : ex_sin_i - {{23{ex_cos_i[41]}}, ex_cos_i[41:23]};
		end	        	                                                                                                                           
		6'd24: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{24{ex_sin_i[41]}}, ex_sin_i[41:24]} : ex_cos_i + {{24{ex_sin_i[41]}}, ex_sin_i[41:24]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{24{ex_cos_i[41]}}, ex_cos_i[41:24]} : ex_sin_i - {{24{ex_cos_i[41]}}, ex_cos_i[41:24]};
		end	        	                                                                                                                           
		6'd25: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{25{ex_sin_i[41]}}, ex_sin_i[41:25]} : ex_cos_i + {{25{ex_sin_i[41]}}, ex_sin_i[41:25]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{25{ex_cos_i[41]}}, ex_cos_i[41:25]} : ex_sin_i - {{25{ex_cos_i[41]}}, ex_cos_i[41:25]};
		end		    	                                                                                                                           
		6'd26: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{26{ex_sin_i[41]}}, ex_sin_i[41:26]} : ex_cos_i + {{26{ex_sin_i[41]}}, ex_sin_i[41:26]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{26{ex_cos_i[41]}}, ex_cos_i[41:26]} : ex_sin_i - {{26{ex_cos_i[41]}}, ex_cos_i[41:26]};
		end		    	                                                                                                                           
		6'd27: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{27{ex_sin_i[41]}}, ex_sin_i[41:27]} : ex_cos_i + {{27{ex_sin_i[41]}}, ex_sin_i[41:27]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{27{ex_cos_i[41]}}, ex_cos_i[41:27]} : ex_sin_i - {{27{ex_cos_i[41]}}, ex_cos_i[41:27]};
		end		    	                                                                                                                        
		6'd28: begin	                                                                                                                        
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{28{ex_sin_i[41]}}, ex_sin_i[41:28]} : ex_cos_i + {{28{ex_sin_i[41]}}, ex_sin_i[41:28]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{28{ex_cos_i[41]}}, ex_cos_i[41:28]} : ex_sin_i - {{28{ex_cos_i[41]}}, ex_cos_i[41:28]};
		end		    	                                                                                                                           
		6'd29: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{29{ex_sin_i[41]}}, ex_sin_i[41:29]} : ex_cos_i + {{29{ex_sin_i[41]}}, ex_sin_i[41:29]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{29{ex_cos_i[41]}}, ex_cos_i[41:29]} : ex_sin_i - {{29{ex_cos_i[41]}}, ex_cos_i[41:29]};
		end		    	                                                                                                                           
		6'd30: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{30{ex_sin_i[41]}}, ex_sin_i[41:30]} : ex_cos_i + {{30{ex_sin_i[41]}}, ex_sin_i[41:30]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{30{ex_cos_i[41]}}, ex_cos_i[41:30]} : ex_sin_i - {{30{ex_cos_i[41]}}, ex_cos_i[41:30]};
		end		    	                                                                                                                           
		6'd31: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{31{ex_sin_i[41]}}, ex_sin_i[41:31]} : ex_cos_i + {{31{ex_sin_i[41]}}, ex_sin_i[41:31]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{31{ex_cos_i[41]}}, ex_cos_i[41:31]} : ex_sin_i - {{31{ex_cos_i[41]}}, ex_cos_i[41:31]};
		end		    	                                                                                                                           
		6'd32: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{32{ex_sin_i[41]}}, ex_sin_i[41:32]} : ex_cos_i + {{32{ex_sin_i[41]}}, ex_sin_i[41:32]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{32{ex_cos_i[41]}}, ex_cos_i[41:32]} : ex_sin_i - {{32{ex_cos_i[41]}}, ex_cos_i[41:32]};
		end		    	                                                                                                                           
		6'd33: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{33{ex_sin_i[41]}}, ex_sin_i[41:33]} : ex_cos_i + {{33{ex_sin_i[41]}}, ex_sin_i[41:33]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{33{ex_cos_i[41]}}, ex_cos_i[41:33]} : ex_sin_i - {{33{ex_cos_i[41]}}, ex_cos_i[41:33]};
		end		    	                                                                                                                           
		6'd34: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{34{ex_sin_i[41]}}, ex_sin_i[41:34]} : ex_cos_i + {{34{ex_sin_i[41]}}, ex_sin_i[41:34]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{34{ex_cos_i[41]}}, ex_cos_i[41:34]} : ex_sin_i - {{34{ex_cos_i[41]}}, ex_cos_i[41:34]};
		end		    	                                                                                                                           
		6'd35: begin	                                                                                                                           
			ex_cos_i	<= # dly ex_sin_i[41] ? ex_cos_i - {{35{ex_sin_i[41]}}, ex_sin_i[41:35]} : ex_cos_i + {{35{ex_sin_i[41]}}, ex_sin_i[41:35]};
			ex_sin_i	<= # dly ex_sin_i[41] ? ex_sin_i + {{35{ex_cos_i[41]}}, ex_cos_i[41:35]} : ex_sin_i - {{35{ex_cos_i[41]}}, ex_cos_i[41:35]};
		end                                                                                        
	endcase                                                                                        
end                                                                                               
 
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phase_acc_q	<= # dly 40'd0;
	end else case (counter_iq)
		6'd0: begin
			if (cos_out_q[35]) begin
				phase_acc_q	<= # dly ~sin_out_q[35] ? 40'h80_0000_0000 - {1'b0, atan[0]} : 40'h80_0000_0000 + {1'b0, atan[0]};			
			end else begin	
				phase_acc_q	<= # dly sin_out_q[35] ? -{1'b0, atan[0]} : {1'b0, atan[0]};
			end
		end	
		6'd1: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[1] } : phase_acc_q + {1'b0, atan[1] };	
		6'd2: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[2] } : phase_acc_q + {1'b0, atan[2] };
		6'd3: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[3] } : phase_acc_q + {1'b0, atan[3] };
		6'd4: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[4] } : phase_acc_q + {1'b0, atan[4] };
		6'd5: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[5] } : phase_acc_q + {1'b0, atan[5] };
		6'd6: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[6] } : phase_acc_q + {1'b0, atan[6] };
		6'd7: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[7] } : phase_acc_q + {1'b0, atan[7] };
		6'd8: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[8] } : phase_acc_q + {1'b0, atan[8] };
		6'd9: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[9] } : phase_acc_q + {1'b0, atan[9] };
		6'd10: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[10]} : phase_acc_q + {1'b0, atan[10]};
		6'd11: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[11]} : phase_acc_q + {1'b0, atan[11]};
		6'd12: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[12]} : phase_acc_q + {1'b0, atan[12]};
		6'd13: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[13]} : phase_acc_q + {1'b0, atan[13]};
		6'd14: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[14]} : phase_acc_q + {1'b0, atan[14]};
		6'd15: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[15]} : phase_acc_q + {1'b0, atan[15]};
		6'd16: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[16]} : phase_acc_q + {1'b0, atan[16]};
		6'd17: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[17]} : phase_acc_q + {1'b0, atan[17]};
		6'd18: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[18]} : phase_acc_q + {1'b0, atan[18]};
		6'd19: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[19]} : phase_acc_q + {1'b0, atan[19]};
		6'd20: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[20]} : phase_acc_q + {1'b0, atan[20]};
		6'd21:	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[21]} : phase_acc_q + {1'b0, atan[21]};
		6'd22: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[22]} : phase_acc_q + {1'b0, atan[22]};
		6'd23:	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[23]} : phase_acc_q + {1'b0, atan[23]};
		6'd24: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[24]} : phase_acc_q + {1'b0, atan[24]};
		6'd25: 	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[25]} : phase_acc_q + {1'b0, atan[25]};
		6'd26:	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[26]} : phase_acc_q + {1'b0, atan[26]};
		6'd27:	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[27]} : phase_acc_q + {1'b0, atan[27]};
		6'd28:	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[28]} : phase_acc_q + {1'b0, atan[28]};
		6'd29:	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[29]} : phase_acc_q + {1'b0, atan[29]};
		6'd30:	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[30]} : phase_acc_q + {1'b0, atan[30]};
		6'd31:	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[31]} : phase_acc_q + {1'b0, atan[31]};
		6'd32:	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[32]} : phase_acc_q + {1'b0, atan[32]};
		6'd33:	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[33]} : phase_acc_q + {1'b0, atan[33]};
		6'd34:	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[34]} : phase_acc_q + {1'b0, atan[34]};
		6'd35:	phase_acc_q	<= # dly ex_sin_q[41] ? phase_acc_q - {1'b0, atan[35]} : phase_acc_q + {1'b0, atan[35]};
	endcase
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		ex_cos_q		<= # dly 42'd0;
		ex_sin_q		<= # dly 42'd0;
	end else case (counter_iq)
		6'd0: begin
			if (cos_out_q[35]) begin   
				ex_cos_q	<= # dly ~sin_out_q[35] ? -{cos_out_q, 6'b0} + {sin_out_q, 6'b0} : -{cos_out_q, 6'b0} - {sin_out_q, 6'b0};
				ex_sin_q	<= # dly ~sin_out_q[35] ? -{sin_out_q, 6'b0} - {cos_out_q, 6'b0} : -{sin_out_q, 6'b0} + {cos_out_q, 6'b0};
			end else begin
				ex_cos_q	<= # dly sin_out_q[35] ? {cos_out_q, 6'b0} - {sin_out_q, 6'b0} : {cos_out_q, 6'b0} + {sin_out_q, 6'b0};
				ex_sin_q	<= # dly sin_out_q[35] ? {sin_out_q, 6'b0} + {cos_out_q, 6'b0} : {sin_out_q, 6'b0} - {cos_out_q, 6'b0};
			end		
		end			
		6'd1: begin
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{1{ex_sin_q[41]}}, ex_sin_q[41:1]} : ex_cos_q + {{1{ex_sin_q[41]}}, ex_sin_q[41:1]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{1{ex_cos_q[41]}}, ex_cos_q[41:1]} : ex_sin_q - {{1{ex_cos_q[41]}}, ex_cos_q[41:1]};
		end	        	                                                                                                                      
		6'd2: begin 	                                                                                                                      
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{2{ex_sin_q[41]}}, ex_sin_q[41:2]} : ex_cos_q + {{2{ex_sin_q[41]}}, ex_sin_q[41:2]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{2{ex_cos_q[41]}}, ex_cos_q[41:2]} : ex_sin_q - {{2{ex_cos_q[41]}}, ex_cos_q[41:2]};
		end	        	                                                                                                                      
		6'd3: begin 	                                                                                                                      
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{3{ex_sin_q[41]}}, ex_sin_q[41:3]} : ex_cos_q + {{3{ex_sin_q[41]}}, ex_sin_q[41:3]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{3{ex_cos_q[41]}}, ex_cos_q[41:3]} : ex_sin_q - {{3{ex_cos_q[41]}}, ex_cos_q[41:3]};
		end		    	                                                                                                                      
		6'd4: begin 	                                                                                                                      
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{4{ex_sin_q[41]}}, ex_sin_q[41:4]} : ex_cos_q + {{4{ex_sin_q[41]}}, ex_sin_q[41:4]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{4{ex_cos_q[41]}}, ex_cos_q[41:4]} : ex_sin_q - {{4{ex_cos_q[41]}}, ex_cos_q[41:4]};
		end		    	                                                                                                                      
		6'd5: begin 	                                                                                                                      
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{5{ex_sin_q[41]}}, ex_sin_q[41:5]} : ex_cos_q + {{5{ex_sin_q[41]}}, ex_sin_q[41:5]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{5{ex_cos_q[41]}}, ex_cos_q[41:5]} : ex_sin_q - {{5{ex_cos_q[41]}}, ex_cos_q[41:5]};
		end		    	                                                                                                                     
		6'd6: begin 	                                                                                                                      
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{6{ex_sin_q[41]}}, ex_sin_q[41:6]} : ex_cos_q + {{6{ex_sin_q[41]}}, ex_sin_q[41:6]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{6{ex_cos_q[41]}}, ex_cos_q[41:6]} : ex_sin_q - {{6{ex_cos_q[41]}}, ex_cos_q[41:6]};
		end		    	                                                                                                                      
		6'd7: begin 	                                                                                                                      
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{7{ex_sin_q[41]}}, ex_sin_q[41:7]} : ex_cos_q + {{7{ex_sin_q[41]}}, ex_sin_q[41:7]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{7{ex_cos_q[41]}}, ex_cos_q[41:7]} : ex_sin_q - {{7{ex_cos_q[41]}}, ex_cos_q[41:7]};
		end		    	                                                                                                                      
		6'd8: begin 	                                                                                                                      
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{8{ex_sin_q[41]}}, ex_sin_q[41:8]} : ex_cos_q + {{8{ex_sin_q[41]}}, ex_sin_q[41:8]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{8{ex_cos_q[41]}}, ex_cos_q[41:8]} : ex_sin_q - {{8{ex_cos_q[41]}}, ex_cos_q[41:8]};
		end		    	                                                                                                                      
		6'd9: begin 	                                                                                                                      
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{9{ex_sin_q[41]}}, ex_sin_q[41:9]} : ex_cos_q + {{9{ex_sin_q[41]}}, ex_sin_q[41:9]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{9{ex_cos_q[41]}}, ex_cos_q[41:9]} : ex_sin_q - {{9{ex_cos_q[41]}}, ex_cos_q[41:9]};
		end		    	
		6'd10: begin	
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{10{ex_sin_q[41]}}, ex_sin_q[41:10]} :  ex_cos_q + {{10{ex_sin_q[41]}}, ex_sin_q[41:10]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{10{ex_cos_q[41]}}, ex_cos_q[41:10]} :  ex_sin_q - {{10{ex_cos_q[41]}}, ex_cos_q[41:10]};
		end		    	                                                                                                                           
		6'd11: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{11{ex_sin_q[41]}}, ex_sin_q[41:11]} : ex_cos_q + {{11{ex_sin_q[41]}}, ex_sin_q[41:11]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{11{ex_cos_q[41]}}, ex_cos_q[41:11]} : ex_sin_q - {{11{ex_cos_q[41]}}, ex_cos_q[41:11]};
		end		    	                                                                                                                           
		6'd12: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{12{ex_sin_q[41]}}, ex_sin_q[41:12]} : ex_cos_q + {{12{ex_sin_q[41]}}, ex_sin_q[41:12]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{12{ex_cos_q[41]}}, ex_cos_q[41:12]} : ex_sin_q - {{12{ex_cos_q[41]}}, ex_cos_q[41:12]};
		end		    	                                                                                                                           
		6'd13: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{13{ex_sin_q[41]}}, ex_sin_q[41:13]} : ex_cos_q + {{13{ex_sin_q[41]}}, ex_sin_q[41:13]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{13{ex_cos_q[41]}}, ex_cos_q[41:13]} : ex_sin_q - {{13{ex_cos_q[41]}}, ex_cos_q[41:13]};
		end		    	                                                                                                                           
		6'd14: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{14{ex_sin_q[41]}}, ex_sin_q[41:14]} : ex_cos_q + {{14{ex_sin_q[41]}}, ex_sin_q[41:14]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{14{ex_cos_q[41]}}, ex_cos_q[41:14]} : ex_sin_q - {{14{ex_cos_q[41]}}, ex_cos_q[41:14]};
		end		    	                                                                                                                           
		6'd15: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{15{ex_sin_q[41]}}, ex_sin_q[41:15]} : ex_cos_q + {{15{ex_sin_q[41]}}, ex_sin_q[41:15]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{15{ex_cos_q[41]}}, ex_cos_q[41:15]} : ex_sin_q - {{15{ex_cos_q[41]}}, ex_cos_q[41:15]};
		end		    	                                                                                                                           
		6'd16: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{16{ex_sin_q[41]}}, ex_sin_q[41:16]} : ex_cos_q + {{16{ex_sin_q[41]}}, ex_sin_q[41:16]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{16{ex_cos_q[41]}}, ex_cos_q[41:16]} : ex_sin_q - {{16{ex_cos_q[41]}}, ex_cos_q[41:16]};
		end		    	                                                                                                                           
		6'd17: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{17{ex_sin_q[41]}}, ex_sin_q[41:17]} : ex_cos_q + {{17{ex_sin_q[41]}}, ex_sin_q[41:17]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{17{ex_cos_q[41]}}, ex_cos_q[41:17]} : ex_sin_q - {{17{ex_cos_q[41]}}, ex_cos_q[41:17]};
		end         	                                                                                                                           
		6'd18: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{18{ex_sin_q[41]}}, ex_sin_q[41:18]} : ex_cos_q + {{18{ex_sin_q[41]}}, ex_sin_q[41:18]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{18{ex_cos_q[41]}}, ex_cos_q[41:18]} : ex_sin_q - {{18{ex_cos_q[41]}}, ex_cos_q[41:18]};
		end		    	                                                                                                                        
		6'd19: begin	                                                                                                                        
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{19{ex_sin_q[41]}}, ex_sin_q[41:19]} : ex_cos_q + {{19{ex_sin_q[41]}}, ex_sin_q[41:19]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{19{ex_cos_q[41]}}, ex_cos_q[41:19]} : ex_sin_q - {{19{ex_cos_q[41]}}, ex_cos_q[41:19]};
		end		    	                                                                                                                           
		6'd20: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{20{ex_sin_q[41]}}, ex_sin_q[41:20]} : ex_cos_q + {{20{ex_sin_q[41]}}, ex_sin_q[41:20]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{20{ex_cos_q[41]}}, ex_cos_q[41:20]} : ex_sin_q - {{20{ex_cos_q[41]}}, ex_cos_q[41:20]};
		end		    	                                                                                                                           
		6'd21: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{21{ex_sin_q[41]}}, ex_sin_q[41:21]} : ex_cos_q + {{21{ex_sin_q[41]}}, ex_sin_q[41:21]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{21{ex_cos_q[41]}}, ex_cos_q[41:21]} : ex_sin_q - {{21{ex_cos_q[41]}}, ex_cos_q[41:21]};
		end		    	                                                                                                                           
		6'd22: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{22{ex_sin_q[41]}}, ex_sin_q[41:22]} : ex_cos_q + {{22{ex_sin_q[41]}}, ex_sin_q[41:22]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{22{ex_cos_q[41]}}, ex_cos_q[41:22]} : ex_sin_q - {{22{ex_cos_q[41]}}, ex_cos_q[41:22]};
		end	        	                                                                                                                           
		6'd23: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{23{ex_sin_q[41]}}, ex_sin_q[41:23]} : ex_cos_q + {{23{ex_sin_q[41]}}, ex_sin_q[41:23]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{23{ex_cos_q[41]}}, ex_cos_q[41:23]} : ex_sin_q - {{23{ex_cos_q[41]}}, ex_cos_q[41:23]};
		end	        	                                                                                                                           
		6'd24: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{24{ex_sin_q[41]}}, ex_sin_q[41:24]} : ex_cos_q + {{24{ex_sin_q[41]}}, ex_sin_q[41:24]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{24{ex_cos_q[41]}}, ex_cos_q[41:24]} : ex_sin_q - {{24{ex_cos_q[41]}}, ex_cos_q[41:24]};
		end	        	                                                                                                                           
		6'd25: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{25{ex_sin_q[41]}}, ex_sin_q[41:25]} : ex_cos_q + {{25{ex_sin_q[41]}}, ex_sin_q[41:25]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{25{ex_cos_q[41]}}, ex_cos_q[41:25]} : ex_sin_q - {{25{ex_cos_q[41]}}, ex_cos_q[41:25]};
		end		    	                                                                                                                           
		6'd26: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{26{ex_sin_q[41]}}, ex_sin_q[41:26]} : ex_cos_q + {{26{ex_sin_q[41]}}, ex_sin_q[41:26]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{26{ex_cos_q[41]}}, ex_cos_q[41:26]} : ex_sin_q - {{26{ex_cos_q[41]}}, ex_cos_q[41:26]};
		end		    	                                                                                                                           
		6'd27: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{27{ex_sin_q[41]}}, ex_sin_q[41:27]} : ex_cos_q + {{27{ex_sin_q[41]}}, ex_sin_q[41:27]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{27{ex_cos_q[41]}}, ex_cos_q[41:27]} : ex_sin_q - {{27{ex_cos_q[41]}}, ex_cos_q[41:27]};
		end		    	                                                                                                                        
		6'd28: begin	                                                                                                                        
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{28{ex_sin_q[41]}}, ex_sin_q[41:28]} : ex_cos_q + {{28{ex_sin_q[41]}}, ex_sin_q[41:28]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{28{ex_cos_q[41]}}, ex_cos_q[41:28]} : ex_sin_q - {{28{ex_cos_q[41]}}, ex_cos_q[41:28]};
		end		    	                                                                                                                           
		6'd29: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{29{ex_sin_q[41]}}, ex_sin_q[41:29]} : ex_cos_q + {{29{ex_sin_q[41]}}, ex_sin_q[41:29]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{29{ex_cos_q[41]}}, ex_cos_q[41:29]} : ex_sin_q - {{29{ex_cos_q[41]}}, ex_cos_q[41:29]};
		end		    	                                                                                                                           
		6'd30: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{30{ex_sin_q[41]}}, ex_sin_q[41:30]} : ex_cos_q + {{30{ex_sin_q[41]}}, ex_sin_q[41:30]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{30{ex_cos_q[41]}}, ex_cos_q[41:30]} : ex_sin_q - {{30{ex_cos_q[41]}}, ex_cos_q[41:30]};
		end		    	                                                                                                                           
		6'd31: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{31{ex_sin_q[41]}}, ex_sin_q[41:31]} : ex_cos_q + {{31{ex_sin_q[41]}}, ex_sin_q[41:31]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{31{ex_cos_q[41]}}, ex_cos_q[41:31]} : ex_sin_q - {{31{ex_cos_q[41]}}, ex_cos_q[41:31]};
		end		    	                                                                                                                           
		6'd32: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{32{ex_sin_q[41]}}, ex_sin_q[41:32]} : ex_cos_q + {{32{ex_sin_q[41]}}, ex_sin_q[41:32]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{32{ex_cos_q[41]}}, ex_cos_q[41:32]} : ex_sin_q - {{32{ex_cos_q[41]}}, ex_cos_q[41:32]};
		end		    	                                                                                                                           
		6'd33: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{33{ex_sin_q[41]}}, ex_sin_q[41:33]} : ex_cos_q + {{33{ex_sin_q[41]}}, ex_sin_q[41:33]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{33{ex_cos_q[41]}}, ex_cos_q[41:33]} : ex_sin_q - {{33{ex_cos_q[41]}}, ex_cos_q[41:33]};
		end		    	                                                                                                                           
		6'd34: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{34{ex_sin_q[41]}}, ex_sin_q[41:34]} : ex_cos_q + {{34{ex_sin_q[41]}}, ex_sin_q[41:34]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{34{ex_cos_q[41]}}, ex_cos_q[41:34]} : ex_sin_q - {{34{ex_cos_q[41]}}, ex_cos_q[41:34]};
		end		    	                                                                                                                           
		6'd35: begin	                                                                                                                           
			ex_cos_q	<= # dly ex_sin_q[41] ? ex_cos_q - {{35{ex_sin_q[41]}}, ex_sin_q[41:35]} : ex_cos_q + {{35{ex_sin_q[41]}}, ex_sin_q[41:35]};
			ex_sin_q	<= # dly ex_sin_q[41] ? ex_sin_q + {{35{ex_cos_q[41]}}, ex_cos_q[41:35]} : ex_sin_q - {{35{ex_cos_q[41]}}, ex_cos_q[41:35]};
		end                                                                                        
	endcase                                                                                        
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin 	
		done_iq		<= # dly 1'b0;
		ampl_out_i	<= # dly 36'h0;
		phase_out_i	<= # dly 40'h0;
		ampl_out_q	<= # dly 36'h0;
		phase_out_q	<= # dly 40'h0;
	end else begin
		done_iq	<= # dly counter_iq==6'd36;		 
		if (counter_iq==6'd36) begin 	
			ampl_out_i	<= # dly ex_cos_i[41:6];
			phase_out_i	<= # dly phase_acc_i;
			ampl_out_q	<= # dly ex_cos_i[41:6];
			phase_out_q	<= # dly phase_acc_q;
		end
	end
end						

                                                                                                
endmodule