`timescale 1 ns/ 1 ns

module cordic_windower (
	input	clk, 
	input	clrn,
	
	input	vme_cs,
	input	vme_wr,
	input	vme_rd,
	input	[5:2] vme_addr,
	input	[31:0] vme_datain,
	output	vme_dout_dav,
	output	[31:0] vme_dout,
	
	input	en_windowing,
	input	[4:0] acq_lenght,
	
	input	dav_in_l,
	input	[31:0] din_l,
	input	dav_in_r,
	input	[31:0] din_r,
	output	reg dav_out_l,
	output	reg [31:0] dout_l,
	output	reg dav_out_r,
	output	reg [31:0] dout_r);

parameter dly = 1;

reg 	[17:0]  phase_inc;
reg 	[18:0] phase;
reg 	waiting_first_dav;
reg 	win_calc_start;
wire	win_calc_done;
wire	[35:0] win_calc_out;
reg		[35:0] mult_coef, mult_data;
reg	    pre_dav_out_r, pre_dav_out_l;
wire	[71:0] mult_out;
reg		first_sample_received;

always @(posedge clk or negedge clrn) begin
	if (~clrn || ~en_windowing) begin
		waiting_first_dav	<= # dly 1'b1;
	end else if (dav_in_l||dav_in_r) begin
		waiting_first_dav	<= # dly ~waiting_first_dav;  
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn || ~en_windowing) begin
		first_sample_received	<= # dly 1'b0;
	end else if (~waiting_first_dav) begin
		first_sample_received	<= # dly 1'b1;  
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn || ~en_windowing) begin
		mult_coef	<= # dly 36'h0;		
	end else if (waiting_first_dav && first_sample_received && win_calc_done) begin	
		mult_coef	<= # dly win_calc_out[35:0];
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		mult_data	<= # dly 36'h0;		
	end else if (dav_in_l) begin	
		mult_data	<= # dly {din_l, 4'h0};
	end else begin
		mult_data	<= # dly {din_r, 4'h0};	
	end
end

assign mult_out	= $signed(mult_data)*$signed(mult_coef);

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		pre_dav_out_r	<= # dly 1'b0;
		pre_dav_out_l	<= # dly 1'b0;
	end else begin
		pre_dav_out_r	<= # dly dav_in_r;
		pre_dav_out_l	<= # dly dav_in_l;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		dav_out_r	<= # dly 1'b0;
		dav_out_l	<= # dly 1'b0;
	end else begin
		dav_out_r	<= # dly pre_dav_out_r;
		dav_out_l	<= # dly pre_dav_out_l;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		dout_l	<= # dly 32'h0;
	end else if (pre_dav_out_l) begin
		if (en_windowing) begin
			dout_l	<= # dly ^mult_out[71:70] ? 32'h7fff_ffff : mult_out[70:39];	//the only ovf case would be (-2^35)^2=2^70 (72 bits in 2 compl), all the rest is representable on 71 bits
		end else begin
			dout_l	<= # dly din_l;
		end
	end
end
		
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		dout_r	<= # dly 32'h0;
	end else if (pre_dav_out_r) begin
		if (en_windowing) begin
			dout_r	<= # dly ^mult_out[71:70] ? 32'h7fff_ffff : mult_out[70:39];	//the only ovf case would be (-2^35)^2, all the rest is representable on 70 bits
		end else begin
			dout_r	<= # dly din_r;
		end
	end
end

always @(posedge clk) begin
	case(acq_lenght)
		5'h00: phase_inc <= # dly  18'h0_0001;
		5'h01: phase_inc <= # dly  18'h0_0002;
		5'h02: phase_inc <= # dly  18'h0_0004;
		5'h03: phase_inc <= # dly  18'h0_0008;
		5'h04: phase_inc <= # dly  18'h0_0010;
		5'h05: phase_inc <= # dly  18'h0_0020;
		5'h06: phase_inc <= # dly  18'h0_0040;
		5'h07: phase_inc <= # dly  18'h0_0080;
		5'h08: phase_inc <= # dly  18'h0_0100;
		5'h09: phase_inc <= # dly  18'h0_0200;
		5'h0a: phase_inc <= # dly  18'h0_0400;
		5'h0b: phase_inc <= # dly  18'h0_0800;
		5'h0c: phase_inc <= # dly  18'h0_1000;
		5'h0d: phase_inc <= # dly  18'h0_2000;
		5'h0e: phase_inc <= # dly  18'h0_4000;
		5'h0f: phase_inc <= # dly  18'h0_8000;
		default: phase_inc	<= # dly  18'h1_0000;
	endcase
end

always @(posedge clk or negedge clrn) begin
	if (~clrn || ~en_windowing) begin
		phase			<= # dly 19'h0;
		win_calc_start	<= # dly 1'b0;
	end else if (waiting_first_dav && (dav_in_l||dav_in_r)) begin	
		phase			<= # dly phase + {phase_inc, 1'b0};
		win_calc_start	<= # dly 1'b1;
	end else begin
		phase			<= # dly phase;
		win_calc_start	<= # dly 1'b0;
	end	
end	
			
cordic_window_calc win_calculator(
	.clk(clk), 
	.clrn(clrn),
	.vme_cs(vme_cs),
	.vme_wr(vme_wr),
	.vme_rd(vme_rd),
	.vme_addr(vme_addr),
	.vme_datain(vme_datain),
	.vme_dout_dav(vme_dout_dav),
	.vme_dout(vme_dout),
	.start(win_calc_start),
	.phase({phase, 21'h0}),
	.done(win_calc_done),
	.dout(win_calc_out));

endmodule
