`timescale 1 ns/ 1 ns

module interrupt_manager (
	input clk, clrn,
	
	input vme_cs,
	input vme_wr,
	input vme_rd,
	input [3:2] vme_addr,
	input [31:0] vme_data,
	output reg vme_dout_dav,
	output reg [31:0] vme_dout,
	
	input [7:0] interrupt_in,
	output reg interrupt_out,
	output int_in_fifo,
	output reg rora_roak);
	
parameter dly	= 1;

parameter int_reg_addr	= 2'b0;
parameter mask_reg_addr	= 2'b1;
	
reg [7:0] mask_reg, int_masked_old; //if a 1 is in position i than the i_th	interrupt is masked
wire [7:0] int_masked, interrupt;
reg [2:0] int_pointer_r, int_pointer_w;
wire clear_int;
reg [7:0] int_fifo [7:0];
reg fifo_full, fifo_empty;

assign clear_int	= vme_cs && vme_rd && vme_addr==int_reg_addr;

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		{rora_roak, mask_reg}	<= # dly 9'h0ff;
	end else if (vme_cs && vme_wr && vme_addr==mask_reg_addr) begin
		{rora_roak, mask_reg}	<= # dly vme_data[8:0];
	end
end
	
assign int_masked	= interrupt_in	& ~mask_reg;

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		int_masked_old	<= # dly 1'b1;
	end else begin
		int_masked_old	<= # dly int_masked;
	end
end

assign interrupt		= int_masked & ~int_masked_old;
assign new_interrupt	= |interrupt;


always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		int_pointer_r	<= # dly 3'h0;
		int_pointer_w	<= # dly 3'h0;
		interrupt_out	<= # dly 1'b0;		
	end else begin
		if (new_interrupt && ~fifo_full) begin
			int_pointer_w	<= # dly int_pointer_w + 1'b1;
			interrupt_out	<= # dly 1'b1;			
		end else begin
			interrupt_out	<= # dly 1'b0;
		end
		if (clear_int && ~fifo_empty) begin
			int_pointer_r	<= # dly int_pointer_r + 1'b1;
		end
	end
end


always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		fifo_full	<= # dly 1'b0;
		fifo_empty	<= # dly 1'b1;
	end else begin
		if ((int_pointer_w+1'b1)==int_pointer_r && new_interrupt && ~clear_int) begin
			fifo_full	<= # dly 1'b1;
		end else if (int_pointer_w!=int_pointer_r) begin
			fifo_full	<= # dly 1'b0;
		end
		if ((int_pointer_r+1'b1)==int_pointer_w && clear_int && ~new_interrupt) begin
			fifo_empty	<= # dly 1'b1;
		end else if (int_pointer_w!=int_pointer_r) begin
			fifo_empty	<= # dly 1'b0;
		end
	end
end

assign int_in_fifo	= ~fifo_empty;

always @(posedge clk) begin
	if (new_interrupt && ~fifo_full) begin
		int_fifo[int_pointer_w]	<= # dly interrupt;
	end 
end


always @(posedge clk) begin
	vme_dout_dav	<= # dly vme_rd;
	if (vme_rd) begin
		case(vme_addr)
		int_reg_addr:	vme_dout	<= # dly fifo_empty ? 32'h0000_ffff : {24'h0, int_fifo[int_pointer_r]};
		mask_reg_addr:	vme_dout	<= # dly {24'h0, int_fifo[int_pointer_r]}; 	
		default: vme_dout <= # dly 32'hdead_beef;
		endcase
	end
end

endmodule