`timescale 1ns/100ps

module trigger_manager(
	clk, 
	clrn,
	tick,
	cs,
	wr,
	rd,
	addr,
	data,
	dout,
	dav_out,
	start_acq,
	en_acq_ch1,
	en_acq_ch2,
	start_chirp_ch1,
	start_chirp_ch2);

input clk, clrn; 
input tick;
input cs, wr, rd;
input [3:0] addr;
input [31:0] data;
output [31:0] dout;
output dav_out;
output start_acq;
output en_acq_ch1;
output en_acq_ch2;
output start_chirp_ch1;
output start_chirp_ch2;

reg en_acq_ch1;
reg en_acq_ch2;
reg start_chirp_ch1;
reg start_chirp_ch2;
	
reg start_acq;
	
parameter dly = 1;

parameter addr_command_reg		= 4'h1;

reg [11:0] tick2skip_acq;
reg [11:0] tick2skip_chirp;

reg [31:0] command_reg;

wire [7:0] command;

wire force_acq, force_chirp; 
reg tick_enabled;
reg alternate_chirp;

parameter command_tick_enable			= 8'h01;
parameter command_tick_disable			= 8'h02;
parameter command_alternate_mode		= 8'h03;
parameter command_concurrent_mode		= 8'h04;
parameter command_set_tick2skip_acq		= 8'h05;
parameter command_set_tick2skip_chirp	= 8'h06;
parameter command_acq_trig				= 8'h10;
parameter command_chirp_trig			= 8'h11;
parameter command_acq_chirp_trig		= 8'h12;

reg start_chirp;

reg active_channel;

wire [31:0] status_reg;

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		command_reg	<= # dly 32'h0000_0000;
	end else if (cs && wr && addr==addr_command_reg) begin
		command_reg	<= # dly data;
	end else begin
		command_reg[31]	<= # dly 1'b1;
	end
end

assign command	= command_reg[31:24];

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		tick_enabled	<= # dly 1'b0;
	end else if (command==command_tick_enable) begin
		tick_enabled	<= # dly 1'b1;
	end else if (command==command_tick_disable) begin
		tick_enabled	<= # dly 1'b0;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		alternate_chirp	<= # dly 1'b0;
	end else if (command==command_alternate_mode) begin
		alternate_chirp	<= # dly 1'b1;
	end else if (command==command_concurrent_mode) begin
		alternate_chirp	<= # dly 1'b0;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		tick2skip_acq	<= # dly 12'h0;
	end else if (command==command_set_tick2skip_acq) begin
		tick2skip_acq	<= # dly data[11:0];
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		tick2skip_chirp	<= # dly 12'h0;
	end else if (command==command_set_tick2skip_chirp) begin
		tick2skip_chirp	<= # dly data[11:0];
	end
end

reg [11:0] counter_acq;
reg	[11:0] counter_chirp;

always @(posedge clk or negedge clrn) begin
	if (~clrn || (cs && wr) || ~tick_enabled) begin
		counter_acq	<= # dly 12'h0;
	end else if (tick) begin
		counter_acq	<= # dly (counter_acq==tick2skip_acq) ? 12'h0 : counter_acq + 1'b1;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn || (cs && wr) || ~tick_enabled) begin
		counter_chirp	<= # dly 12'h0;
	end else if (tick) begin
		counter_chirp	<= # dly (counter_chirp==tick2skip_chirp) ? 12'h0 : counter_chirp + 1'b1;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn ||(command==command_tick_enable)) begin
		start_acq		<= # dly 1'b0;
		active_channel	<= # dly 1'b0;
	end else begin
		if (force_acq || (tick_enabled && tick && counter_acq==tick2skip_acq)) begin
			active_channel	<= # dly ~active_channel;
			start_acq		<= # dly 1'b1;
		end else begin
			start_acq	<= # dly 1'b0;
		end
	end
end

always @(posedge clk) begin
	start_chirp	<= # dly force_chirp || (tick_enabled && tick && counter_chirp==tick2skip_chirp);
end

assign force_acq		= (command==command_acq_trig || command==command_acq_chirp_trig);
assign force_chirp		= (command==command_chirp_trig || command==command_acq_chirp_trig);

always @ (posedge clk) begin
	en_acq_ch1		<= # dly ~active_channel;
	en_acq_ch2		<= # dly active_channel;
	start_chirp_ch1	<= # dly alternate_chirp	? (start_chirp && ~active_channel) : start_chirp;
	start_chirp_ch2	<= # dly alternate_chirp	? (start_chirp && active_channel) : start_chirp;
end

assign status_reg[31]		= tick_enabled;
assign status_reg[30]		= alternate_chirp;
assign status_reg[29]		= active_channel;
assign status_reg[28:24]	= 5'h0;
assign status_reg[23:12]	= tick2skip_chirp;
assign status_reg[11:0]		= tick2skip_acq;


assign dout		= addr[0]	? command_reg : status_reg;
assign dav_out	= cs && rd;

endmodule