`timescale 1ns/100ps

module trigger_manager(
	clk, 
	clrn,
	tick,
	cs,
	wr,
	addr,
	data,
	start_acq,
	start_chirp);

input clk, clrn; 
input tick;
input cs, wr;
input [3:0] addr;
input [31:0] data;
output start_acq;
output start_chirp;
	
reg start_acq;
reg start_chirp;
	
parameter dly = 1;

parameter addr_control_reg		= 4'h0;
parameter addr_tick2skeep_acq	= 4'h1;
parameter addr_tick2skeep_chirp	= 4'h1;

reg [31:0] control_reg;
reg [31:0] tick2skeep_acq;
reg [31:0] tick2skeep_chirp;

wire force_acq, force_chirp; 
wire tick_enabled;

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		control_reg	<= # dly 32'h0;
	end else if (cs && wr && addr==addr_control_reg) begin
		control_reg	<= # dly data;
	end else begin
		control_reg[1:0]	<= # dly 2'h0;
	end
end

assign force_acq	= control_reg[0];
assign force_chirp	= control_reg[1];
assign tick_enabled	= control_reg[31];
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		tick2skeep_acq	<= # dly 32'h0;
	end else if (cs && wr && addr==addr_tick2skeep_acq) begin
		tick2skeep_acq	<= # dly data;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		tick2skeep_chirp	<= # dly 32'h0;
	end else if (cs && wr && addr==addr_tick2skeep_chirp) begin
		tick2skeep_chirp	<= # dly data;
	end
end

reg [31:0] counter_acq;
reg	[31:0] counter_chirp;

always @(posedge clk or negedge clrn) begin
	if (~clrn || (cs && wr)) begin
		counter_acq	<= # dly 32'h0;
	end else if (tick) begin
		counter_acq	<= # dly (counter_acq==addr_tick2skeep_acq) ? 32'h0 : counter_acq + 1'b1;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn || (cs && wr)) begin
		counter_chirp	<= # dly 32'h0;
	end else if (tick) begin
		counter_chirp	<= # dly (counter_chirp==addr_tick2skeep_chirp) ? 32'h0 : counter_chirp + 1'b1;
	end
end

always @(posedge clk) begin
	start_acq	<= # dly force_acq || (tick_enabled && tick && counter_acq==addr_tick2skeep_acq);
end

always @(posedge clk) begin
	start_chirp	<= # dly force_chirp || (tick_enabled && tick && counter_chirp==addr_tick2skeep_chirp);
end

endmodule