// DO NOT CHANGE THE TIMESCALE
// MAKE SURE YOUR SIMULATOR USE "PS" RESOLUTION
`timescale 1ns / 10ps

//`define FREQ166

module test_pl;

    parameter                       addr_bits =    19;      // 19 bits
    parameter                       data_bits =    32;      // 32 bits
    
   `ifdef FREQ133
    parameter                       tCYC      =    7.5;   // Hi-Z
    
   `else
  // `ifdef FREQ166
    parameter                       tCYC      =    6.0;   // Hi-Z
    
  //`endif
    `endif

    reg       [(data_bits - 1) : 0] dq;
    reg       [(addr_bits - 1) : 0] addr;
    reg                             lbo_n;
    reg                             adv_n;
    reg                             clk;
    reg                             adsc_n;
    reg                             adsp_n;
    reg                             bwa_n;
    reg                             bwb_n;
    reg                             bwc_n;
    reg                             bwd_n;
    reg                             bwe_n;
    reg                             gw_n;
    reg                             ce0_n;
    reg                             ce1;
    reg                             ce2_n;
    reg                             oe_n;
    reg                             zz;

    wire      [(data_bits - 1) : 0] DQ = dq;

    as7c33512pfs32 ssram0 (DQ, addr, lbo_n, adv_n, clk, adsc_n, adsp_n, bwa_n, bwb_n, bwc_n, bwd_n, bwe_n, gw_n, ce0_n, ce1, ce2_n, oe_n, zz);

    initial
      begin
       $dumpvars(0,test_pl);
       $timeformat (-9, 1, " ns", 10); 
       $dumpfile("./test_pl.vcd"); 
      end
    
    initial begin
        clk = 1'b0;
    end

  
   `ifdef FREQ133
    always #3.75 clk = ~clk;
    
   `else
   //`ifdef FREQ166
    always #3 clk = ~clk;
    
    //`endif
    `endif

    always @ (posedge clk) begin
        $strobe   ("at time %t CE#=%b ADSP#=%b ADSC#=%b ADV#=%b WRITE#=%b OE#=%b CLK=%b ADDR=%d DQ=%d",
                   $time, {ce0_n, ce2_n, ce1}, adsp_n, adsc_n, adv_n, {gw_n, bwe_n, bwd_n, bwc_n, bwb_n, bwa_n}, oe_n, clk, addr, DQ);
    end

    // deselect
    task deselect;
        begin
            // First clock
            ce0_n   = 1'b1;
            ce2_n  = 1'b0;
            ce1    = 1'b1;
            zz     = 1'b0;
            adsp_n = 1'b1;
            adsc_n = 1'b0;
            adv_n  = 1'b0;
            lbo_n  = 1'b0;
            gw_n   = 1'b1;
            bwe_n  = 1'b0;
            bwa_n  = 1'b0;
            bwb_n  = 1'b0;
            bwc_n  = 1'b0;
            bwd_n  = 1'b0;
            oe_n   = 1'b1;
            addr   = {addr_bits{1'b0}};
            dq     = {data_bits{1'bz}};
            #tCYC;
        end
    endtask

    // Single Write with ADSC#
    task write_adsc_burst_1;
        input [(addr_bits - 1) : 0] addr_in;
        input [(data_bits - 1) : 0] data_in;
        begin
            // First clock (latch address and data)
            ce0_n   = 1'b0;
            ce2_n  = 1'b0;
            ce1    = 1'b1;
            zz     = 1'b0;
            adsp_n = 1'b1;
            adsc_n = 1'b0;
            adv_n  = 1'b0;
            lbo_n  = 1'b0;
            gw_n   = 1'b1;
            bwe_n  = 1'b0;
            bwa_n  = 1'b0;
            bwb_n  = 1'b0;
            bwc_n  = 1'b0;
            bwd_n  = 1'b0;
            oe_n   = 1'b1;
            addr   = addr_in;
            dq     = data_in;
            #tCYC;
            // Second clock (deselect)
            ce0_n   = 1'b1;
            dq     = {data_bits{1'bz}};
            #tCYC;
        end
    endtask

    // Single Write with ADSP#
    task write_adsp_burst_1;
        input [(addr_bits - 1) : 0] addr_in;
        input [(data_bits - 1) : 0] data_in;
        begin
            // First clock (latch address)
            ce0_n   = 1'b0;
            ce2_n  = 1'b0;
            ce1    = 1'b1;
            zz     = 1'b0;
            adsp_n = 1'b0;
            adsc_n = 1'b1;
            adv_n  = 1'b0;
            lbo_n  = 1'b0;
            gw_n   = 1'b1;
            bwe_n  = 1'b0;
            bwa_n  = 1'b0;
            bwb_n  = 1'b0;
            bwc_n  = 1'b0;
            bwd_n  = 1'b0;
            oe_n   = 1'b1;
            addr   = addr_in;
            dq     = {data_bits{1'bz}};
            #tCYC;
            // Second clock (latch data)
            // ADV# must be HIGH to permit a WRITE to loaded address
            ce0_n   = 1'b1;
            adsp_n = 1'b1;
            adv_n  = 1'b1;
            dq     = data_in;
            #tCYC;
            // Third clock (deselect)
            adsc_n = 1'b0;
            dq     = {data_bits{1'bz}};
            #tCYC;
        end
    endtask

    // Back-to-back Write (Write with ADSC# two times)
    task write_adsc_burst_2;
        input [(addr_bits - 1) : 0] addr0_in;
        input [(data_bits - 1) : 0] data0_in;
        input [(addr_bits - 1) : 0] addr1_in;
        input [(data_bits - 1) : 0] data1_in;
        begin
            // First clock (latch first address and data)
            ce0_n   = 1'b0;
            ce2_n  = 1'b0;
            ce1    = 1'b1;
            zz     = 1'b0;
            adsp_n = 1'b1;
            adsc_n = 1'b0;
            adv_n  = 1'b0;
            lbo_n  = 1'b0;
            gw_n   = 1'b1;
            bwe_n  = 1'b0;
            bwa_n  = 1'b0;
            bwb_n  = 1'b0;
            bwc_n  = 1'b0;
            bwd_n  = 1'b0;
            oe_n   = 1'b1;
            addr   = addr0_in;
            dq     = data0_in;
            #tCYC;
            // Second clock (latch second address and data)
            addr   = addr1_in;
            dq     = data1_in;
            #tCYC;
            // Third clock (deselect)
            ce0_n   = 1'b1;
            dq     = {data_bits{1'bz}};
            #tCYC;
        end
    endtask

    // Write burst with ADSC# (4 data)
    task write_adsc_burst_4;
        input [(addr_bits - 1) : 0] addr_in;
        input [(data_bits - 1) : 0] data0_in;
        input [(data_bits - 1) : 0] data1_in;
        input [(data_bits - 1) : 0] data2_in;
        input [(data_bits - 1) : 0] data3_in;
        begin
            // First clock (latch first address and data)
            ce0_n   = 1'b0;
            ce2_n  = 1'b0;
            ce1    = 1'b1;
            zz     = 1'b0;
            adsp_n = 1'b1;
            adsc_n = 1'b0;
            adv_n  = 1'b0;
            lbo_n  = 1'b0;
            gw_n   = 1'b1;
            bwe_n  = 1'b0;
            bwa_n  = 1'b0;
            bwb_n  = 1'b0;
            bwc_n  = 1'b0;
            bwd_n  = 1'b0;
            oe_n   = 1'b1;
            addr   = addr_in;
            dq     = data0_in;
            #tCYC;
            // Second clock (latch second address and data)
            ce0_n   = 1'b1;
            adsc_n = 1'b1;
            dq     = data1_in;
            #tCYC;
            // Third clock (latch third address and data)
            dq     = data2_in;
            #tCYC;
            // Forth clock (latch forth address and data)
            dq     = data3_in;
            #tCYC;
            // Third clock (deselect)
            adsc_n = 1'b0;
            dq     = {data_bits{1'bz}};
            #tCYC;
        end
    endtask

    // Write burst with ADSP# (4 data)
    task write_adsp_burst_4;
        input [(addr_bits - 1) : 0] addr_in;
        input [(data_bits - 1) : 0] data0_in;
        input [(data_bits - 1) : 0] data1_in;
        input [(data_bits - 1) : 0] data2_in;
        input [(data_bits - 1) : 0] data3_in;
        begin
            // First clock (latch address)
            ce0_n   = 1'b0;
            ce2_n  = 1'b0;
            ce1    = 1'b1;
            zz     = 1'b0;
            adsp_n = 1'b0;
            adsc_n = 1'b1;
            adv_n  = 1'b0;
            lbo_n  = 1'b0;
            gw_n   = 1'b1;
            bwe_n  = 1'b0;
            bwa_n  = 1'b0;
            bwb_n  = 1'b0;
            bwc_n  = 1'b0;
            bwd_n  = 1'b0;
            oe_n   = 1'b1;
            addr   = addr_in;
            dq     = {data_bits{1'bz}};
            #tCYC;
            // Second clock (latch first data)
            // ADV# must be HIGH to permit a WRITE to loaded address
            ce0_n   = 1'b1;
            adsp_n = 1'b1;
            adv_n  = 1'b1;
            dq     = data0_in;
            #tCYC;
            // Third clock (latch second data)
            adv_n  = 1'b0;
            dq     = data1_in;
            #tCYC;
            // Forth clock (latch third data)
            dq     = data2_in;
            #tCYC;
            // Fifth clock (latch forth data)
            dq     = data3_in;
            #tCYC;
            // Sixth clock (deselect)
            adsc_n = 1'b0;
            dq     = {data_bits{1'bz}};
            #tCYC;
        end
    endtask

    // Single Read with ADSC#
    task read_adsc_burst_1;
        input [(addr_bits - 1) : 0] addr_in;
        begin
            // First clock (latch address)
            ce0_n   = 1'b0;
            ce2_n  = 1'b0;
            ce1    = 1'b1;
            zz     = 1'b0;
            adsp_n = 1'b1;
            adsc_n = 1'b0;
            adv_n  = 1'b0;
            lbo_n  = 1'b0;
            gw_n   = 1'b1;
            bwe_n  = 1'b0;
            bwa_n  = 1'b1;
            bwb_n  = 1'b1;
            bwc_n  = 1'b1;
            bwd_n  = 1'b1;
            oe_n   = 1'b0;
            addr   = addr_in;
            dq     = {data_bits{1'bz}};
            #tCYC;
            #tCYC;
            // Deselect
            ce0_n   = 1'b1;
            #tCYC;
            #tCYC;
        end
    endtask

    // Single Read with ADSP#
    task read_adsp_burst_1;
        input [(addr_bits - 1) : 0] addr_in;
        begin
            // First clock (latch address)
            ce0_n   = 1'b0;
            ce2_n  = 1'b0;
            ce1    = 1'b1;
            zz     = 1'b0;
            adsp_n = 1'b0;
            adsc_n = 1'b1;
            adv_n  = 1'b0;
            lbo_n  = 1'b0;
            gw_n   = 1'b1;
            bwe_n  = 1'b0;
            bwa_n  = 1'b1;
            bwb_n  = 1'b1;
            bwc_n  = 1'b1;
            bwd_n  = 1'b1;
            oe_n   = 1'b0;
            addr   = addr_in;
            dq     = {data_bits{1'bz}};
            #tCYC;
            #tCYC;
            // Deselect
            ce0_n   = 1'b1;
            adsc_n = 1'b0;
            #tCYC;
            #tCYC;
        end
    endtask

    // Back-to-back Read with ADSC# (Read with ADSC# two times)
    task read_adsc_burst_2;
        input [(addr_bits - 1) : 0] addr0_in;
        input [(addr_bits - 1) : 0] addr1_in;
        begin
            // First clock (latch address)
            ce0_n   = 1'b0;
            ce2_n  = 1'b0;
            ce1    = 1'b1;
            zz     = 1'b0;
            adsp_n = 1'b1;
            adsc_n = 1'b0;
            adv_n  = 1'b0;
            lbo_n  = 1'b0;
            gw_n   = 1'b1;
            bwe_n  = 1'b0;
            bwa_n  = 1'b1;
            bwb_n  = 1'b1;
            bwc_n  = 1'b1;
            bwd_n  = 1'b1;
            oe_n   = 1'b0;
            addr   = addr0_in;
            dq     = {data_bits{1'bz}};
            #tCYC;
            // Second clock (latch second address)
            addr   = addr1_in;
            #tCYC;
            #tCYC;
            // Deselect
            ce0_n   = 1'b1;
            #tCYC;
            #tCYC;
        end
    endtask

    // Read burst with ADSC# (4 data)
    task read_adsc_burst_4;
        input [(addr_bits - 1) : 0] addr_in;
        begin
            // First clock (latch address)
            ce0_n   = 1'b0;
            ce2_n  = 1'b0;
            ce1    = 1'b1;
            zz     = 1'b0;
            adsp_n = 1'b1;
            adsc_n = 1'b0;
            adv_n  = 1'b0;
            lbo_n  = 1'b0;
            gw_n   = 1'b1;
            bwe_n  = 1'b0;
            bwa_n  = 1'b1;
            bwb_n  = 1'b1;
            bwc_n  = 1'b1;
            bwd_n  = 1'b1;
            oe_n   = 1'b0;
            addr   = addr_in;
            dq     = {data_bits{1'bz}};
            #tCYC;
            // Second clock
            ce0_n   = 1'b1;
            adsc_n = 1'b1;
            #tCYC;
            // Third clock (first data valid)
            #tCYC;
            // Forth clock (second data valid)
            #tCYC;
            // Fifth clock (third data valid)
            #tCYC;
            // Sixth clock (deselect - forth data valid)
            adsc_n = 1'b0;
            #tCYC;
            #tCYC;
        end
    endtask

    // Read burst with ADSP# (4 data)
    task read_adsp_burst_4;
        input [(addr_bits - 1) : 0] addr_in;
        begin
            // First clock (latch address)
            ce0_n   = 1'b0;
            ce2_n  = 1'b0;
            ce1    = 1'b1;
            zz     = 1'b0;
            adsp_n = 1'b0;
            adsc_n = 1'b1;
            adv_n  = 1'b0;
            lbo_n  = 1'b0;
            gw_n   = 1'b1;
            bwe_n  = 1'b0;
            bwa_n  = 1'b1;
            bwb_n  = 1'b1;
            bwc_n  = 1'b1;
            bwd_n  = 1'b1;
            oe_n   = 1'b0;
            addr   = addr_in;
            dq     = {data_bits{1'bz}};
            #tCYC;
            // Second clock
            ce0_n   = 1'b1;
            adsp_n = 1'b1;
            #tCYC;
            // Third clock (first data valid)
            #tCYC;
            // Forth clock (second data valid)
            #tCYC;
            // Fifth clock (third data valid)
            #tCYC;
            adsc_n = 1'b0;
            #tCYC;
            #tCYC;
        end
    endtask

    initial begin
        begin
            deselect;
            write_adsc_burst_1 (1, 1);
            write_adsp_burst_1 (2, 2);
            write_adsc_burst_2 (4, 4, 8, 8);
            write_adsc_burst_4 (16, 16, 17, 18, 19);
            write_adsp_burst_4 (32, 32, 33, 34, 35);
            read_adsc_burst_1  (1);
            read_adsp_burst_1  (2);
            read_adsc_burst_2  (4, 8);
            read_adsc_burst_4  (16);
            read_adsp_burst_4  (32);
            deselect;
        end
     $finish;
    end
endmodule
