`timescale 1ns/100ps

module cs4272 (rst,
	mclk,
	lrck,
	sclk,
	sdout,
	sdin,
	cs,
	cclk,
	cdin);
	
input rst;
input mclk;
output lrck;
output sclk;
output sdout;
input sdin;
input cs;
input cclk;
input cdin;

parameter dly = 1;


//SPI INTERFACE
reg [1:0] spi_state;
reg [2:0] spi_cycle;
reg [6:0] spi_shreg;
reg [6:0] spi_addr;
reg spi_r_w;
reg spi_incr;
reg [3:0] spi_map;
reg [7:0] spi_data;
reg [7:0] reg_mem [8:1];

parameter spi_chip_addr	= 7'b0010000;

parameter s_spi_addr	= 2'b00;
parameter s_spi_map		= 2'b01;
parameter s_spi_data	= 2'b10;

initial begin
	reg_mem[1]	= 8'h00;
	reg_mem[2]	= 8'h80;
	reg_mem[3]	= 8'h29;
	reg_mem[4]	= 8'h00;
	reg_mem[5]	= 8'h00;
	reg_mem[6]	= 8'h00;
	reg_mem[7]	= 8'h00;
	reg_mem[8]	= 8'h00;
end

always @(posedge cclk or posedge cs or negedge rst) begin
	if (~rst || cs) begin
		spi_state	<= # dly s_spi_addr;
		spi_cycle	<= # dly 3'h0;
		spi_shreg	<= # dly 8'h0;
		spi_r_w		<= # dly 1'b1;
		spi_map		<= # dly 7'b0;
		if (rst && (((spi_state==s_spi_data || spi_state==s_spi_addr) && |spi_cycle) || spi_state==s_spi_map)) begin
			$display("ERROR: CS4272 SPI cycle interrupted abnormally @ %t", $time);
		end
	end else begin
		spi_shreg	<= # dly {spi_shreg[5:0], cdin};
		spi_cycle	<= # dly spi_cycle + 1'b1;
		case  (spi_state)
			s_spi_addr: begin
				if (&spi_cycle) begin
					spi_state	<= # dly s_spi_map;
					spi_addr	= spi_shreg;
					spi_r_w		=  cdin;
					if (spi_addr!=spi_chip_addr) begin
						$display("ERROR: CS4272 SPI cycle started with wrong chip addr @ %t", $time);
					end
					if (spi_r_w!=1'b0) begin
						$display("ERROR: CS4272 unsupported SPI read cycle started @ %t", $time);
					end
				end
			end
			s_spi_map: begin
				if (&spi_cycle) begin
					spi_state	<= # dly s_spi_data;
					spi_map		= {spi_shreg[2:0], cdin};
					spi_incr	= spi_shreg[6];
					if (|spi_shreg[5:3]) begin
						$display("WORNING: CS4272 SPI cycle with MAP[6:4] = %h @ %t", spi_shreg[5:3], $time);
					end 
				end
			end
			s_spi_data: begin
				if (&spi_cycle) begin
					spi_data	= {spi_shreg, cdin};
					if (spi_addr==spi_chip_addr && ~spi_r_w) begin
						if (spi_map!=4'h0 && spi_map<4'h08)	begin
							reg_mem[spi_map]	<= # dly spi_data;
							$display("CS4275 register %h set to %h @ %t", spi_map, spi_data, $time);
						end else begin
							$display("ERROR: Attempt to write %h in non existing CS4275's register %h  @ %t", spi_data, spi_map,  $time);
						end
					end
					if (spi_incr) begin
						spi_map	<= # dly spi_map + 1'b1;
					end
				end
			end
		endcase
	end
end
									
//ADC & DAC SERIAL INTERFACE (SIMPLIFIED)

reg sclk;
reg [5:0] serial_cycle;
reg [63:0] adc_shreg, dac_shreg;

reg [23:0] dac_left, dac_right;
reg [23:0] adc_left, adc_right;

always begin
	sclk	= 1'b1;
	#40.7;
	sclk	= 1'b0;
	#40.7;
end					  

initial begin
	adc_shreg		= 32'h0;
	serial_cycle	= 5'h3f;
end
						
always @(negedge sclk) begin
	serial_cycle	<= # dly serial_cycle + 1'b1;
end

assign lrck	= serial_cycle[5];


always @(negedge sclk) begin
	adc_shreg	= {adc_shreg[62:0], adc_shreg[63]}; 
	if (serial_cycle==5'h3f) begin
		adc_shreg	= {-dac_left, 8'h0, -dac_right, 8'h0};
	end
end

assign sdout	= adc_shreg[63];

always @(posedge sclk) begin
	dac_shreg	= {dac_shreg[63:0], sdin}; 
end

always @(posedge lrck) begin
	dac_left	<= # dly dac_shreg[63:40];
	dac_right	<= # dly dac_shreg[31:8];
end
	
endmodule					