`timescale 1ns/1ns

module codec_control(	input clk, clrn,
	input cs,
	input wr,
	input rd,
	input [9:2] addr, 
	input [31:0] data,
	output reg dout_dav,
	output reg [31:0] dout,
	
	input rf_clk,
	
	input lrck, sclk, sdout, 
	output sdin,
	output spi_clk, spi_cs, spi_din,
	output reg rst,
	
	output reg [23:0] out_l, out_r,
	output adc_dav_l, adc_dav_r,
	
	input [31:0] fir_out_r, fir_out_l,
	
	output [35:0] mod_ampl_l, mod_ampl_r,
	
	output reg pre_mclk,
	input chirp_start_r, chirp_start_l,
	
	input hw_cycle_start);
	
parameter dly = 1;

parameter addr_pll_r	= 2'd0;
parameter addr_pll_l	= 2'd1;
parameter addr_conf_reg	= 2'd2;	

parameter addr_status_reg	= 6'h0;
parameter addr_command_reg	= 6'h1;
parameter addr_mclk_div		= 6'h2;
parameter addr_f0_reg_r		= 6'h3;
parameter addr_f1_reg_r		= 6'h4;
parameter addr_f2_reg_r		= 6'h5;
parameter addr_df_reg_r		= 6'h6;
parameter addr_f0_reg_l		= 6'h7;
parameter addr_f1_reg_l		= 6'h8;
parameter addr_f2_reg_l		= 6'h9;
parameter addr_df_reg_l		= 6'ha;
parameter addr_spi_reg		= 6'h10;
parameter addr_amplitude_r	= 6'h20;
parameter addr_amplitude_l	= 6'h21;
parameter addr_adc_r_value	= 6'h22;
parameter addr_adc_l_value	= 6'h23;

parameter command_codec_rst_off		= 8'h0;
parameter command_codec_rst_on		= 8'h1;
parameter command_set_loopback_mode	= 8'h2;
parameter command_set_normal_mode	= 8'h3;
parameter command_enable_chirp_r	= 8'h4;
parameter command_disable_chirp_r	= 8'h5;
parameter command_enable_chirp_l	= 8'h6;
parameter command_disable_chirp_l	= 8'h7;
parameter command_sin_mode_r		= 8'h8;
parameter command_cos_mode_r		= 8'h9;
parameter command_sin_mode_l		= 8'ha;
parameter command_cos_mode_l		= 8'hb;

reg [39:0] phase_l;
reg [31:0] phase_adv_left, f0_reg_l;
reg [35:0] dout_left;
wire wr_left_ph_adv;

reg [39:0] phase_r;
reg [31:0] phase_adv_right, f0_reg_r;
reg [35:0] dout_right;
wire wr_right_ph_adv;

wire lword_req;		
reg lword_req_ff, load_lword;	

wire rword_req;		
reg rword_req_ff, load_rword;	

reg [23:0] new_word;

wire l_dav, r_dav;
reg l_dav_req, r_dav_req, adc_dav_l, adc_dav_r, adc_dav_l_int, adc_dav_r_int;
wire [23:0] adc_out_l, adc_out_r;

wire wr_spi;

reg [15:0] mclk_low_lenght, mclk_high_lenght, mclk_counter;

wire wr_df_reg_r, wr_f1_reg_r, wr_f2_reg_r;
reg [31:0] f1_reg_r, f2_reg_r;
reg [15:0] df_reg_r; 

wire wr_df_reg_l, wr_f1_reg_l, wr_f2_reg_l;
reg [31:0] f1_reg_l, f2_reg_l;
reg [15:0] df_reg_l;

reg loopback_mode;

reg [9:0] loopback_counter;
wire lrck_int, sclk_int, sdout_int, sdin_int;

reg chirping_r, chirping_l;
reg enable_chirp_r, enable_chirp_l;

wire wr_amplitude_l, wr_amplitude_r;
reg [31:0] amplitude_l, amplitude_r;

wire spi_done;

reg [31:0] command_reg;
wire [7:0] command;

wire cs_pll_r, cs_pll_l;	
wire pll_r_dout_dav, pll_l_dout_dav;
wire [31:0] pll_r_dout, pll_l_dout;
wire [35:0] cos_out_r, sin_out_r, cos_se1_out_r, sin_se1_out_r, cos_se2_out_r, sin_se2_out_r, cos_out_l, sin_out_l, cos_se1_out_l, sin_se1_out_l, cos_se2_out_l, sin_se2_out_l;
reg sin_mode_r, sin_mode_l;

//COMMAND REGISTER

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		command_reg	<= # dly 32'h0000_0000;
	end else if (cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_command_reg) begin
		command_reg	<= # dly data;
	end else begin
		command_reg[31]	<= # dly 1'b1;
	end
end

assign command	= command_reg[31:24];


//LOOPBACK CONTROL

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		loopback_mode	<= # dly 1'b0;
	end else if (command==command_set_loopback_mode) begin
		loopback_mode	<= # dly 1'b1;
	end else if (command==command_set_normal_mode) begin
		loopback_mode	<= # dly 1'b0;
	end
end	

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		loopback_counter	<= # dly 10'b0;
	end else begin
		loopback_counter	<= # dly loopback_counter + 1'b1;
	end
end

assign lrck_int		= loopback_mode	? loopback_counter[9]	: lrck;
assign sclk_int		= loopback_mode	? loopback_counter[3]	: sclk;
assign sdout_int	= loopback_mode	? sdin_int				: sdout;
assign sdin			= sdin_int;

//RIGHT FREQUENCY SYNTHESIZER
		
assign cs_pll_r			= (addr[9:8]==addr_pll_r && cs);	
assign wr_right_ph_adv	= cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_f0_reg_r;
assign wr_df_reg_r		= cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_df_reg_r;
assign wr_f1_reg_r		= cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_f1_reg_r;
assign wr_f2_reg_r		= cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_f2_reg_r;
assign wr_amplitude_r	= cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_amplitude_r;

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		enable_chirp_r	<= # dly 1'b0;
	end else if (command==command_enable_chirp_r) begin
		enable_chirp_r	<= # dly 1'b1;
	end else if (command==command_disable_chirp_r) begin
		enable_chirp_r	<= # dly 1'b0;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		sin_mode_r	<= # dly 1'b0;
	end else if (command==command_sin_mode_r) begin
		sin_mode_r	<= # dly 1'b1;
	end else if (command==command_cos_mode_r) begin
		sin_mode_r	<= # dly 1'b0;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		df_reg_r	<= # dly 16'b0;
	end else if (wr_df_reg_r) begin
		df_reg_r	<= # dly data[15:0];
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		chirping_r	<= # dly 1'b0;
	end else if (enable_chirp_r && chirp_start_r) begin
		chirping_r	<= # dly 1'b1;
	end else if (phase_adv_right>=f2_reg_r) begin
		chirping_r	<= # dly 1'b0;
	end
end	

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		f1_reg_r	<= # dly 32'b0;
	end else if (wr_f1_reg_r) begin
		f1_reg_r	<= # dly data;
	end 
end
		
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		f2_reg_r	<= # dly 32'b0;
	end else if (wr_f2_reg_r) begin
		f2_reg_r	<= # dly data;
	end 
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		f0_reg_r	<= # dly 32'h0800_0000;
	end else if (wr_right_ph_adv) begin
		f0_reg_r	<= # dly data;
	end
end
		
always @(posedge clk) begin
	if (enable_chirp_r && chirp_start_r && ~chirping_r) begin
		phase_adv_right	<= # dly f1_reg_r;
	end else if (chirping_r) begin
		phase_adv_right <= # dly phase_adv_right + df_reg_r[15:0];
	end	else begin
		phase_adv_right	<= # dly f0_reg_r;
	end			
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phase_r	<= # dly 40'h0;
	end else begin
		phase_r	<= # dly phase_r + phase_adv_right;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		amplitude_r	<= # dly 32'h9b74_ec00; //maximum amplitude
	end else if (wr_amplitude_r) begin
		amplitude_r	<= # dly data;
	end
end

	
q_pll pll_r (.clk(clk),
	.clrn(clrn),
	
	.ext_amplitude(amplitude_r),
	.ext_phase(phase_r),
	
	.start(load_rword),
	.ampl_i(fir_out_r),
	.ampl_q(fir_out_l),
//	output	done,
	.cos_pll(cos_out_r),
	.sin_pll(sin_out_r),
	.cos_se1(cos_se1_out_r),
	.sin_se1(sin_se1_out_r),
	.cos_se2(cos_se2_out_r),
	.sin_se2(sin_se2_out_r),
	.hw_cycle_start(hw_cycle_start),
	
	.mod_ampl_i(mod_ampl_r),
	
	.vme_cs(cs_pll_r),
	.vme_wr(wr),
	.vme_rd(rd),
	.vme_addr(addr[7:2]), 
	.vme_din(data),
	.vme_dav(pll_r_dout_dav),
	.vme_dout(pll_r_dout));	

always @(posedge clk) begin
	dout_right	<= # dly	sin_mode_r ? 	(sin_out_r + sin_se1_out_r + sin_se2_out_r) : (cos_out_r + cos_se1_out_r + cos_se2_out_r);			
end	
	
//LEFT FREQUENCY SYNTHESIZER

assign cs_pll_l			= (addr[9:8]==addr_pll_l && cs);	
assign wr_left_ph_adv	= cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_f0_reg_l;
assign wr_df_reg_l		= cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_df_reg_l;
assign wr_f1_reg_l		= cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_f1_reg_l;
assign wr_f2_reg_l		= cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_f2_reg_l;
assign wr_amplitude_l	= cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_amplitude_l;

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		enable_chirp_l	<= # dly 1'b0;
	end else if (command==command_enable_chirp_l) begin
		enable_chirp_l	<= # dly 1'b1;
	end else if (command==command_disable_chirp_l) begin
		enable_chirp_l	<= # dly 1'b0;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		sin_mode_l	<= # dly 1'b0;
	end else if (command==command_sin_mode_l) begin
		sin_mode_l	<= # dly 1'b1;
	end else if (command==command_cos_mode_l) begin
		sin_mode_l	<= # dly 1'b0;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		df_reg_l	<= # dly 16'b0;
	end else if (wr_df_reg_l) begin
		df_reg_l	<= # dly data[15:0];
	end 
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		chirping_l	<= # dly 1'b0;
	end else if (enable_chirp_l && chirp_start_l) begin
		chirping_l	<= # dly 1'b1;
	end else if (phase_adv_left>=f2_reg_l) begin
		chirping_l	<= # dly 1'b0;
	end
end	


always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		f1_reg_l	<= # dly 32'b0;
	end else if (wr_f1_reg_l) begin
		f1_reg_l	<= # dly data;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		f2_reg_l	<= # dly 32'b0;
	end else if (wr_f2_reg_l) begin
		f2_reg_l	<= # dly data;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		f0_reg_l	<= # dly 32'h0800_0000;
	end else if (wr_left_ph_adv) begin
		f0_reg_l	<= # dly data;
	end
end
		
always @(posedge clk) begin
	if (enable_chirp_l && chirp_start_l && ~chirping_l) begin
		phase_adv_left	<= # dly f1_reg_l;
	end else if (chirping_l) begin
		phase_adv_left <= # dly phase_adv_left + df_reg_l[15:0];
	end	else begin
		phase_adv_left	<= # dly f0_reg_l;
	end			
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phase_l	<= # dly 40'h0;
	end else begin
		phase_l	<= # dly phase_l + phase_adv_left;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		amplitude_l	<= # dly 32'h9b74_ec00; //maximum amplitude
	end else if (wr_amplitude_l) begin
		amplitude_l	<= # dly data;
	end
end

	
q_pll pll_l (.clk(clk),
	.clrn(clrn),
	
	.ext_amplitude(amplitude_l),
	.ext_phase(phase_l),
	
	.start(load_lword),
	.ampl_i(fir_out_l),
	.ampl_q(fir_out_r),
//	output	done,
	.cos_pll(cos_out_l),
	.sin_pll(sin_out_l),	
	.cos_se1(cos_se1_out_l),
	.sin_se1(sin_se1_out_l),
	.cos_se2(cos_se2_out_l),
	.sin_se2(sin_se2_out_l),
	
	.hw_cycle_start(hw_cycle_start),
	
	.mod_ampl_i(mod_ampl_l),
		
	.vme_cs(cs_pll_l),
	.vme_wr(wr),
	.vme_rd(rd),
	.vme_addr(addr[7:2]), 
	.vme_din(data),
	.vme_dav(pll_l_dout_dav),
	.vme_dout(pll_l_dout));	

always @(posedge clk) begin
	dout_left	<= # dly sin_mode_l ? 	(sin_out_l + sin_se1_out_l + sin_se2_out_l) : (cos_out_l + cos_se1_out_l + cos_se2_out_l);		
end

//DAC INTERFACE 

always @(posedge lword_req or posedge load_lword) begin
	if (load_lword) begin
		lword_req_ff	<= # dly 1'b0;
	end else begin
		lword_req_ff	<= # dly 1'b1;
	end
end

always @(posedge clk) begin
	load_lword	<= # dly lword_req_ff;
end

always @(posedge rword_req or posedge load_rword) begin
	if (load_rword) begin
		rword_req_ff	<= # dly 1'b0;
	end else begin
		rword_req_ff	<= # dly 1'b1;
	end
end

always @(posedge clk) begin
	load_rword	<= # dly rword_req_ff;
end

always @(posedge clk) begin
	if (load_lword) begin
		new_word	<= # dly (dout_left[11]&& ~(~dout_left[35] && &dout_left[34:12])) ? dout_left[35:12] + 1'b1 : dout_left[35:12];
	end else if (load_rword) begin
		new_word	<= # dly (dout_right[11]&& ~(~dout_right[35] && &dout_right[34:12])) ? dout_right[35:12] + 1'b1 : dout_right[35:12];
	end
end		
			
cs4272_dacs cs4272_dacs_i(
	.clrn(clrn), 
	.lrck(lrck_int), 
	.sclk(sclk_int), 
	.sdin(sdin_int),
	.lword_req(lword_req), 
	.rword_req(rword_req),
	.word(new_word));
	
//ADC INTERFACE

always @(posedge l_dav or posedge adc_dav_l_int) begin
	if (adc_dav_l_int) begin
		l_dav_req	<= # dly 1'b0;
	end else begin	
		l_dav_req	<= # dly 1'b1;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		adc_dav_l_int	<= # dly 1'b0;
	end else begin
		adc_dav_l_int	<= # dly l_dav_req;
	end
end

always @(posedge r_dav or posedge adc_dav_r_int) begin
	if (adc_dav_r_int) begin
		r_dav_req	<= # dly 1'b0;
	end else begin	
		r_dav_req	<= # dly 1'b1;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		adc_dav_r_int	<= # dly 1'b0;
	end else begin
		adc_dav_r_int	<= # dly r_dav_req;
	end
end

always @(posedge clk) begin
	adc_dav_l	<= # dly adc_dav_l_int;
	adc_dav_r	<= # dly adc_dav_r_int;	
	if (adc_dav_l_int) begin
		out_l	<= # dly adc_out_l;
	end
	if (adc_dav_r_int) begin
		out_r	<= # dly adc_out_r;
	end
end

cs4272_adcs cs4272_adcs_i( 
	.clrn(clrn), 
	.lrck(lrck_int), 
	.sclk(sclk_int), 
	.sdout(sdout_int),
	.l_dav(l_dav), 
	.r_dav(r_dav),
	.l_word(adc_out_l), 
	.r_word(adc_out_r));	
		
//SPI INTERFACE

assign wr_spi	= cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_spi_reg;

cs4272_spi_interface spi_interface_i(
	.clk(clk),
	.clrn(rst),
	.start(wr_spi),
	.addr(data[11:8]),
	.data_in(data[7:0]),
	.done(spi_done),
	.cs(spi_cs),
	.sclk(spi_clk),
	.sdi(spi_din));

//PRE MCLK
	
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		mclk_low_lenght		<= # dly 16'd37;
		mclk_high_lenght	<= # dly 16'd38;
	end else if (cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_mclk_div) begin
		mclk_low_lenght		<= # dly data[15:0];
		mclk_high_lenght	<= # dly data[31:16];
	end
end

always @(posedge rf_clk or negedge clrn) begin
	if (~clrn) begin
		mclk_counter	<= # dly 16'h0;
		pre_mclk		<= # dly 1'b0;
	end else begin
		mclk_counter	<= # dly mclk_counter + 1'b1;
		if (pre_mclk && mclk_counter==mclk_high_lenght) begin
			pre_mclk		<= # dly 1'b0;
			mclk_counter	<= # dly 16'h0;
		end else if (~pre_mclk && mclk_counter==mclk_low_lenght) begin
			pre_mclk		<= # dly 1'b1;
			mclk_counter	<= # dly 16'h0;
		end else if (cs && wr && addr[9:8]==addr_conf_reg && addr[7:2]==addr_mclk_div) begin
			mclk_counter	<= # dly 16'h0;
		end			
	end
end


//RST GENERATION


always@(posedge clk or negedge clrn) begin
	if (~clrn) begin
		rst	<= # dly 1'b0;
	end else begin
		if (command==command_codec_rst_off) begin
			rst	<= # dly 1'b1;
		end else if (command==command_codec_rst_on) begin
			rst	<= # dly 1'b0;
		end
	end
end		

//READBACK FUNCTIONALITY

always @* begin
	if (addr[9:8]==addr_conf_reg && cs) begin
		dout_dav	= rd;
		case (addr[7:2])
		   addr_status_reg	: dout	= {25'h0, sin_mode_r, sin_mode_l, enable_chirp_r, enable_chirp_l, loopback_mode ,rst ,spi_done}; 
		   addr_command_reg	: dout	= command_reg;
		   addr_mclk_div	: dout	= {mclk_low_lenght, mclk_high_lenght};	  
		   addr_f0_reg_r	: dout	= f0_reg_r;	  
		   addr_f1_reg_r	: dout	= f1_reg_r;	  
		   addr_f2_reg_r	: dout	= f2_reg_r;	  
		   addr_df_reg_r	: dout	= df_reg_r;	  
		   addr_f0_reg_l	: dout	= f0_reg_l;	  
		   addr_f1_reg_l	: dout	= f1_reg_l;	  
		   addr_f2_reg_l	: dout	= f2_reg_l;	  
		   addr_df_reg_l	: dout	= df_reg_l;	  
		   addr_spi_reg		: dout	= 32'hdead_beef;   
		   addr_amplitude_r	: dout	= amplitude_r;
		   addr_amplitude_l	: dout	= amplitude_l;
		   addr_adc_r_value	: dout	= {out_r, 8'h0};
		   addr_adc_l_value	: dout	= {out_l, 8'h0};
		   default: dout	= 32'hdead_beef;
		endcase
	end else if (addr[9:8]==addr_pll_r && cs) begin
		dout_dav	= pll_r_dout_dav;
		dout		= pll_r_dout;	
	end else if (addr[9:8]==addr_pll_l && cs) begin
		dout_dav	= pll_l_dout_dav;
		dout		= pll_l_dout;	
	end else begin
		dout_dav	= 1'b0;
		dout		= 32'hdead_beef;
	end
end
		
endmodule