`timescale 1ns/1ns

module codec_control(	input clk, clrn,
	input cs,
	input wr,
	input [17:0] addr, 
	input [31:0] data,
	
	input lrck, sclk, sdout, 
	output sdin,
	output spi_clk, spi_cs, spi_din,
	output reg rst,
	
	output reg [23:0] out_l, out_r,
	output adc_dav_l, adc_dav_r,
	
	output reg pre_mclk,
	input chirp_start_r, chirp_start_l);
	
parameter dly = 1;

parameter addr_freq_synth_left	= 2'd0;
parameter addr_freq_synth_right	= 2'd1;
parameter addr_conf_reg			= 2'd2;	
parameter addr_spi_interface	= 2'd3;

parameter addr_r_ph_adv			= 16'h1;
parameter addr_l_ph_adv			= 16'h2;
parameter addr_mclk_div			= 16'h3;
parameter addr_chirp_control0_r	= 16'h4;
parameter addr_chirp_control1_r	= 16'h5;
parameter addr_chirp_control2_r	= 16'h6;
parameter addr_chirp_control0_l	= 16'h7;
parameter addr_chirp_control1_l	= 16'h8;
parameter addr_chirp_control2_l	= 16'h9;
parameter addr_codec_rst_off	= 16'ha;
parameter addr_codec_rst_on		= 16'hb;
parameter addr_start_chirp_r	= 16'hc;
parameter addr_start_chirp_l	= 16'hd;
parameter addr_start_chirp_lr	= 16'he;

reg [31:0] phase_adv_left;
wire wr_left_mem;
wire [18:0] dout_left;
wire wr_left_ph_adv;

reg [31:0] phase_adv_right;
wire wr_right_mem;
wire [18:0] dout_right;
wire wr_right_ph_adv;

wire lword_req;		
reg lword_req_ff, load_lword;	

wire rword_req;		
reg rword_req_ff, load_rword;	

reg [23:0] new_word;

wire l_dav, r_dav;
reg l_dav_req, r_dav_req, adc_dav_l, adc_dav_r, adc_dav_l_int, adc_dav_r_int;
wire [23:0] adc_out_l, adc_out_r;

wire wr_spi;

reg [15:0] mclk_low_lenght, mclk_high_lenght, mclk_counter;

wire wr_chirp_control0_r, wr_chirp_control1_r, wr_chirp_control2_r;
reg [31:0] chirp_control0_r, chirp_control1_r, chirp_control2_r;

wire wr_chirp_control0_l, wr_chirp_control1_l, wr_chirp_control2_l;
reg [31:0] chirp_control0_l, chirp_control1_l, chirp_control2_l;

wire soft_chirp_start_r, soft_chirp_start_l;

//RIGHT FREQUENCY SYNTHESIZER
		
assign wr_right_mem			= cs && wr && addr[17:16]==addr_freq_synth_right;
assign wr_right_ph_adv		= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_r_ph_adv;
assign wr_chirp_control0_r	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_chirp_control0_r;
assign wr_chirp_control1_r	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_chirp_control1_r;
assign wr_chirp_control2_r	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_chirp_control2_r;
assign soft_chirp_start_r	= cs && wr && addr[17:16]==addr_conf_reg && (addr[15:0]==addr_start_chirp_r || addr[15:0]==addr_start_chirp_lr);

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		chirp_control0_r	<= # dly 32'b0;
	end else if (wr_chirp_control0_r) begin
		chirp_control0_r	<= # dly data;
	end else if ((chirp_control0_r[30] && chirp_start_r) || soft_chirp_start_r) begin
		chirp_control0_r[31]	<= # dly 1'b1;
	end else if (phase_adv_right>=chirp_control2_r) begin
		chirp_control0_r[31]	<= # dly 1'b0;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		chirp_control1_r	<= # dly 32'b0;
	end else if (wr_chirp_control1_r) begin
		chirp_control1_r	<= # dly data;
	end 
end
		
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		chirp_control2_r	<= # dly 32'b0;
	end else if (wr_chirp_control2_r) begin
		chirp_control2_r	<= # dly data;
	end 
end
		
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phase_adv_right	<= # dly 32'h0004_0000;
	end else if (wr_right_ph_adv) begin
		phase_adv_right	<= # dly data;
	end else if (chirp_control0_r[30] && (chirp_start_r || soft_chirp_start_r)) begin
		phase_adv_right	<= # dly chirp_control1_r;
	end else if (chirp_control0_r[31]) begin
		phase_adv_right <= # dly phase_adv_right + chirp_control0_r[15:0];
	end			
end
	
freq_synthesizer freq_synthesizer_right_i(  
	.clk(clk), 
	.clrn(clrn),
	.phase_adv(phase_adv_right),
	.wr(wr_right_mem),
	.addr(addr[15:0]),
	.data(data),
	.dout(dout_right));


//LEFT FREQUENCY SYNTHESIZER

assign wr_left_mem			= cs && wr && addr[17:16]==addr_freq_synth_left;
assign wr_left_ph_adv		= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_l_ph_adv;
assign wr_chirp_control0_l	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_chirp_control0_l;
assign wr_chirp_control1_l	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_chirp_control1_l;
assign wr_chirp_control2_l	= cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_chirp_control2_l;
assign soft_chirp_start_l	= cs && wr && addr[17:16]==addr_conf_reg && (addr[15:0]==addr_start_chirp_l || addr[15:0]==addr_start_chirp_lr);

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		chirp_control0_l	<= # dly 32'b0;
	end else if (wr_chirp_control0_l) begin
		chirp_control0_l[30:0]	<= # dly data[30:0];
	end else if ((chirp_control0_l[30] && chirp_start_l) || soft_chirp_start_l) begin
		chirp_control0_l[31]	<= # dly 1'b1;
	end else if (phase_adv_left>=chirp_control2_l) begin
		chirp_control0_l[31]	<= # dly 1'b0;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		chirp_control1_l	<= # dly 32'b0;
	end else if (wr_chirp_control1_l) begin
		chirp_control1_l	<= # dly data;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		chirp_control2_l	<= # dly 32'b0;
	end else if (wr_chirp_control2_l) begin
		chirp_control2_l	<= # dly data;
	end
end
		
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phase_adv_left	<= # dly 32'h0001_0000;
	end else if (wr_left_ph_adv) begin
		phase_adv_left	<= # dly data;
	end else if (chirp_control0_l[30] && (chirp_start_l || soft_chirp_start_l)) begin
		phase_adv_left	<= # dly chirp_control1_l;
	end else if (chirp_control0_l[31]) begin
		phase_adv_left <= # dly phase_adv_left + chirp_control0_l[15:0];
	end			
end
	
freq_synthesizer freq_synthesizer_left_i(  
	.clk(clk), 
	.clrn(clrn),
	.phase_adv(phase_adv_left),
	.wr(wr_left_mem),
	.addr(addr[15:0]),
	.data(data),
	.dout(dout_left));

//DAC INTERFACE 

always @(posedge lword_req or posedge load_lword) begin
	if (load_lword) begin
		lword_req_ff	<= # dly 1'b0;
	end else begin
		lword_req_ff	<= # dly 1'b1;
	end
end

always @(posedge clk) begin
	load_lword	<= # dly lword_req_ff;
end

always @(posedge rword_req or posedge load_rword) begin
	if (load_rword) begin
		rword_req_ff	<= # dly 1'b0;
	end else begin
		rword_req_ff	<= # dly 1'b1;
	end
end

always @(posedge clk) begin
	load_rword	<= # dly rword_req_ff;
end

always @(posedge clk) begin
	if (load_lword) begin
		new_word	<= # dly {dout_left, 5'h0};
	end else if (load_rword) begin
		new_word	<= # dly {dout_right, 5'h0};
	end
end		
			
cs4272_dacs cs4272_dacs_i(
	.clrn(clrn), 
	.lrck(lrck), 
	.sclk(sclk), 
	.sdin(sdin),
	.lword_req(lword_req), 
	.rword_req(rword_req),
	.word(new_word));
	
//ADC INTERFACE

always @(posedge l_dav or posedge adc_dav_l_int) begin
	if (adc_dav_l_int) begin
		l_dav_req	<= # dly 1'b0;
	end else begin	
		l_dav_req	<= # dly 1'b1;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		adc_dav_l_int	<= # dly 1'b0;
	end else begin
		adc_dav_l_int	<= # dly l_dav_req;
	end
end

always @(posedge r_dav or posedge adc_dav_r_int) begin
	if (adc_dav_r_int) begin
		r_dav_req	<= # dly 1'b0;
	end else begin	
		r_dav_req	<= # dly 1'b1;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		adc_dav_r_int	<= # dly 1'b0;
	end else begin
		adc_dav_r_int	<= # dly r_dav_req;
	end
end

always @(posedge clk) begin
	adc_dav_l	<= # dly adc_dav_l_int;
	adc_dav_r	<= # dly adc_dav_r_int;	
	if (adc_dav_l_int) begin
		out_l	<= # dly adc_out_l;
	end
	if (adc_dav_r_int) begin
		out_r	<= # dly adc_out_r;
	end
end

cs4272_adcs cs4272_adcs_i( 
	.clrn(clrn), 
	.lrck(lrck), 
	.sclk(sclk), 
	.sdout(sdout),
	.l_dav(l_dav), 
	.r_dav(r_dav),
	.l_word(adc_out_l), 
	.r_word(adc_out_r));	
		
//SPI INTERFACE

assign wr_spi	= cs && wr && addr[17:16]==addr_spi_interface && addr[15:0]==16'h0;

cs4272_spi_interface spi_interface_i(
	.clk(clk),
	.clrn(rst),
	.start(wr_spi),
	.addr(data[11:8]),
	.data_in(data[7:0]),
	.done(),
	.cs(spi_cs),
	.sclk(spi_clk),
	.sdi(spi_din));

//PRE MCLK
	
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		mclk_low_lenght		<= # dly 16'd63;
		mclk_high_lenght	<= # dly 16'd63;
	end else if (cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_mclk_div) begin
		mclk_low_lenght		<= # dly data[15:0];
		mclk_high_lenght	<= # dly data[31:16];
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		mclk_counter	<= # dly 16'h0;
		pre_mclk		<= # dly 1'b0;
	end else begin
		mclk_counter	<= # dly mclk_counter + 1'b1;
		if (pre_mclk && mclk_counter==mclk_high_lenght) begin
			pre_mclk		<= # dly 1'b0;
			mclk_counter	<= # dly 16'h0;
		end else if (~pre_mclk && mclk_counter==mclk_low_lenght) begin
			pre_mclk		<= # dly 1'b1;
			mclk_counter	<= # dly 16'h0;
		end else if (cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_mclk_div) begin
			mclk_counter	<= # dly 16'h0;
		end			
	end
end

//RST GENERATION


always@(posedge clk or negedge clrn) begin
	if (~clrn) begin
		rst	<= # dly 1'b0;
	end else begin
		if (cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_codec_rst_off) begin
			rst	<= # dly 1'b1;
		end else if (cs && wr && addr[17:16]==addr_conf_reg && addr[15:0]==addr_codec_rst_on) begin
			rst	<= # dly 1'b0;
		end
	end
end		

endmodule