`timescale 1 ns/ 1 ns

//The coeficients are expressed on 32 bits in 2 complement as fixed point. They must all be smaller equal than 1 in modulus.  
//The overflow is checked and saturation implemented.

module cordic_window_calc (
	input	clk, 
	input	clrn,
	
	input	vme_cs,
	input	vme_wr,
	input	vme_rd,
	input	[5:2] vme_addr,
	input	[31:0] vme_datain,
	output	vme_dout_dav,
	output	reg [31:0] vme_dout,
	
	input	start,
	input	[39:0] phase,
	output	reg done,
	output	[35:0] dout);

parameter dly = 1;

reg [31:0] a0, a1, a2, a3, a4;

parameter address_a0	= 4'd0;
parameter address_a1	= 4'd1;
parameter address_a2	= 4'd2;
parameter address_a3	= 4'd3;
parameter address_a4	= 4'd4;

reg [2:0] state;

parameter	s_idle	= 3'h0;
parameter	s_a1	= 3'h1;
parameter	s_a2	= 3'h2;
parameter	s_a3	= 3'h3;
parameter	s_a4	= 3'h4;
parameter	s_round	= 3'h5;
		
reg cordic_start;
reg [31:0] cordic_ampl;	
reg [39:0] cordic_phase;
wire cordic_done;

reg [38:0] window_value;

wire [35:0] cos_out;

//REGISTERS

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		a0	<= # dly 32'h4000_0000;
	end else if (vme_wr && vme_cs && vme_addr==address_a0) begin
		a0	<= # dly vme_datain;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		a1	<= # dly 32'hb245_8a00;
	end else if (vme_wr && vme_cs && vme_addr==address_a1) begin
		a1	<= # dly vme_datain;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		a2	<= # dly 32'h0;
	end else if (vme_wr && vme_cs && vme_addr==address_a2) begin
		a2	<= # dly vme_datain;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		a3	<= # dly 32'h0;
	end else if (vme_wr && vme_cs && vme_addr==address_a3) begin
		a3	<= # dly vme_datain;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		a4	<= # dly 32'h0;
	end else if (vme_wr && vme_cs && vme_addr==address_a4) begin
		a4	<= # dly vme_datain;
	end
end

// STATE MACHINE

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		state			<= # dly s_idle;
		cordic_start	<= # dly 1'b0;
		cordic_ampl		<= # dly 32'h0;
		cordic_phase	<= # dly 40'h0;
		done			<= # dly 1'b1;
		window_value	<= # dly 39'h0;
	end else begin
		case (state)
		s_idle:begin
			if (start) begin
				state			<= # dly s_a1;
				done			<= # dly 1'b0;
				window_value	<= # dly {3'b000, a0, 4'h0};
				cordic_start	<= # dly 1'b1;
				cordic_ampl		<= # dly a1;
				cordic_phase	<= # dly phase;
			end
		end
		s_a1:begin
			if (cordic_done && ~cordic_start) begin
				state			<= # dly s_a2;
				window_value	<= # dly window_value + {{3{cos_out[35]}}, cos_out};
				cordic_start	<= # dly 1'b1;
				cordic_ampl		<= # dly a2;
				cordic_phase	<= # dly cordic_phase + phase;					
			end else begin
				cordic_start	<= # dly 1'b0;
			end
		end		
		s_a2:begin
			if (cordic_done && ~cordic_start) begin
				state			<= # dly s_a3;
				window_value	<= # dly window_value + {{3{cos_out[35]}}, cos_out};
				cordic_start	<= # dly 1'b1;
				cordic_ampl		<= # dly a3;
				cordic_phase	<= # dly cordic_phase + phase;					
			end else begin
				cordic_start	<= # dly 1'b0;
			end
		end	
		s_a3:begin
			if (cordic_done && ~cordic_start) begin
				state			<= # dly s_a4;
				window_value	<= # dly window_value + {{3{cos_out[35]}}, cos_out};
				cordic_start	<= # dly 1'b1;
				cordic_ampl		<= # dly a4;
				cordic_phase	<= # dly cordic_phase + phase;					
			end else begin
				cordic_start	<= # dly 1'b0;
			end		
		end	
		s_a4:begin
			cordic_start	<= # dly 1'b0;
			if (cordic_done && ~cordic_start) begin
				state			<= # dly s_round;
				window_value	<= # dly window_value + {{3{cos_out[35]}}, cos_out};					
			end
		end		
		s_round:begin
			done	<= # dly 1'b1;
			state	<= # dly s_idle;
			if (window_value[38] && ~&window_value[37:35]) begin
				window_value	<= # dly {3'b111, 36'h8_0000_0000};
			end else if (~window_value[38] && |window_value[37:35]) begin
				window_value	<= # dly {3'b000, 36'h7_ffff_ffff};
			end
		end		
		default: begin
			state			<= # dly s_idle;
			cordic_start	<= # dly 1'b0;
			cordic_ampl		<= # dly 32'h0;
			cordic_phase	<= # dly 40'h0;
			done			<= # dly 1'b1;
			window_value	<= # dly 39'h0;
		end
		endcase
	end
end

assign dout	= window_value[35:0];	
							
cordic_rotator_folded cordic_r (.clk(clk),
	.clrn(clrn),
	.phase_in(cordic_phase),
	.start_value({cordic_ampl[31], cordic_ampl, 9'h0}),	
	.start(cordic_start),
	.done(cordic_done),
	.cos_out(cos_out));		

assign vme_dout_dav	= vme_rd;
	
always @* begin
	case (vme_addr) 
	address_a0: vme_dout	<= # dly a0;	
	address_a0: vme_dout	<= # dly a1;	
	address_a0: vme_dout	<= # dly a2;	
	address_a0: vme_dout	<= # dly a3;	
	address_a0: vme_dout	<= # dly a4;	
	default: vme_dout	<= # dly a0;
	endcase
end	
	
endmodule
