`timescale 1ns/1ns

module cs4272_spi_interface(clk,
	clrn,
	start,
	addr,
	data_in,
	done,
	cs,
	sclk,
	sdi);	
input	clk, clrn;
input	start;
input	[3:0] addr;
input	[7:0] data_in;
output	done;
output	cs, sclk, sdi;

parameter half_sclk_cycle = 5'd16; //decremented by 1

reg cs, sclk;
reg done;

parameter dly =1;

reg [4:0] cycle_counter;
reg [4:0] sh_counter;
reg [23:0] sh_reg;
reg [1:0] state;

parameter s_post_reset	= 2'd0;
parameter s_idle		= 2'd1;
parameter s_send		= 2'd2;


assign sdi	= sh_reg[23]; 

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		 state			<= # dly s_post_reset;
		 cycle_counter	<= # dly 5'b0;
		 sh_reg			<= # dly {7'b0010000, 1'b0, 8'h7, 8'h3};
		 cs				<= # dly 1'b1;
		 sclk			<= # dly 1'b1;
		 done			<= # dly 1'b1;
		 sh_counter		<= # dly 5'h0;
	end else begin
		case (state)
			s_post_reset: begin
				if (cycle_counter[4]) begin
					sclk		<= # dly 1'b0;
				end
				cycle_counter	<= # dly cycle_counter + 1'b1;
				if (&cycle_counter) begin
					state	<= # dly s_send;
				end
			end
			s_idle: begin
				sh_reg			<= # dly {7'b0010000, 1'b0, 4'b0, addr, data_in};
				cycle_counter	<= # dly 5'b0;
				sh_counter		<= # dly 5'h0;
				cs				<= # dly 1'b1;				
				if (start) begin
					state	<= # dly s_send;
					sclk	<= # dly 1'b0;
					done	<= # dly 1'b0;
				end else begin
					state	<= # dly s_idle;
					sclk	<= # dly 1'b1; 
					done	<= # dly 1'b1;
				end
			end
			s_send: begin
				cs	<= # dly (sh_counter==5'd24) && ~sclk;
				if (cycle_counter==half_sclk_cycle) begin
					sclk			<= # dly ~sclk;
					cycle_counter	<= # dly 5'h0;					
					if (~sclk) begin //rising edge of sclk
						sh_counter	<= # dly sh_counter + 1'b1;
						if (sh_counter==5'd24) begin
							state	<= # dly s_idle;
						end else begin
							state	<= # dly s_send;
						end
					end else begin //falling edge of sclk
						sh_reg		<= # dly {sh_reg[22:0], 1'b0};
					end 
				end else begin
					cycle_counter	<= # dly cycle_counter	+ 1'b1;
				end
			end
			default: begin
				state	<= # dly s_idle;
			end
		endcase
	end
end

endmodule		