`timescale 1 ns/ 1 ns

module fft_mem_master (
	clk, 
	clrn,
	vme_module_sel,
	vme_wr,
	vme_rd,
	vme_addr,
	vme_datain,
	vme_dout_dav,
	vme_dout,
	vme_retry_n,
	trigger,
	en_trigger,
	fir_dec_dav_l,
	fir_dec_ovf_l,
	fir_dec_dout_l,
	fir_dec_dav_r,
	fir_dec_ovf_r,
	fir_dec_dout_r,
	en_windowing,
	acq_lenght,
	fft_mod_req,
	fft_acc_granted,
	fft_start,
	fft_iteration,
	fft_n_2,
	fft_x_t,
	fft_x_f,
	fft_overflow,
	sram_addr,
	sram_data,
	sram_gwe_n,
	sram_bwe_n,
	sram_oe_n,
	sram_ce1_n,
	sram_clk,
	int_for_raw,
	int_for_fft,
	pll_fft_mode);
	
input	clk; 
input	clrn;

input	vme_module_sel;
input	vme_wr;	
input	vme_rd;
input	[21:2] vme_addr;
input	[31:0] vme_datain;
output	vme_dout_dav;
output	[31:0]vme_dout;
output	vme_retry_n;
	
input	trigger;
input	en_trigger;

input	fir_dec_dav_l;
input	fir_dec_dav_r;
input	fir_dec_ovf_l;
input	fir_dec_ovf_r;
input	[31:0] fir_dec_dout_l;
input	[31:0] fir_dec_dout_r;

output	en_windowing;
output [4:0] acq_lenght;

output	fft_mod_req;
input	fft_acc_granted;	
output	fft_start;
output	[3:0] fft_iteration;
output	[15:0] fft_n_2;
output	[31:0] fft_x_t;
input	[31:0] fft_x_f;
input	fft_overflow;

output	[20:0] sram_addr;
inout	[31:0]sram_data;
output	sram_gwe_n;
output	sram_bwe_n;
output	sram_oe_n;
output	sram_ce1_n;
output	sram_clk;

output	int_for_raw;
output	int_for_fft;

output pll_fft_mode;



reg fft_mod_req;
reg	fft_start;
reg	[3:0] fft_iteration;
reg	[15:0] fft_n_2;
reg	[31:0] vme_dout;
reg vme_dout_dav;
reg vme_retry_n;
reg [4:0] acq_lenght;
reg	int_for_raw;
reg	int_for_fft;
reg pll_fft_mode;

parameter dly = 1;

reg [3:0] fft_lenght;

reg	[4:0] pre_acq_lenght;

reg	first_fir_dec_dav;
wire fir_dec_dav;

reg fir_dec_ovf_l_reg, fir_dec_ovf_r_reg;

wire [7:0] command;

reg mode_normal;
reg trigger_armed;
reg raw_data_required, fft_data_required;

wire [31:0] status_reg, status_reg2;
reg [31:0] command_reg;
reg [31:0] trig_number; //equal to a time stamp if a trigger comes each 20ms


reg [17:0] mem_pointer;

wire sel_reg_space, sel_mem_space;

reg fft_overflow_reg;

reg	[20:0] sram_addr;
reg	sram_oe_n;			
reg	sram_gwe_n;

reg [2:0] state;
reg [31:0] sram_data_int;
reg oe_data_int;

reg [31:0] sram_data_synch;
reg vme_dout_dav_mem;
reg [4:0] counter;
reg adc_l_wr_req, adc_r_wr_req;
reg acquiring;
reg raw_data_read;
reg fft_data_read;
reg [20:0] fft_end_address, acq_end_address;
reg [31:0] time_stamp;
reg [15:0] end_n_2;

reg [17:0] sram_addr_inc;

reg [15:0] fir_dec_dav_counter;


reg window_req;

wire trigger_acq;

parameter s_idle		= 3'h0;
parameter s_vme_rd		= 3'h1;
parameter s_acquiring	= 3'h2;
parameter s_fft_rd		= 3'h3;
parameter s_fft_wr		= 3'h4;
parameter s_fft_done	= 3'h5;
parameter s_0padding	= 3'h6;

parameter addr_status_reg	= 4'h0; 
parameter addr_status_reg2	= 4'h1;
parameter addr_command_reg	= 4'h2;
parameter addr_trig_number	= 4'h3;
parameter addr_time_stamp	= 4'h4;

parameter command_arm_trigger	= 8'h01; //When in single mode it arms the trigger
parameter command_normal_mode	= 8'h02; //Each tigger_acq will trigger the acquisition from the codec 
parameter command_single_mode	= 8'h03; //Only the first 
parameter command_fft_mode		= 8'h05; //only the FFT is required
parameter command_raw_mode		= 8'h06; //only the RAW data are required
parameter command_start_fft		= 8'h07; //forces the start of an FFT conversion of the data stored in the memory
parameter command_set_fft_lenght= 8'h08; //sets the lenght of each fft (4^(9-lenght)): if the acq_lenght is smaller the rest of the array is 0-padded
parameter command_set_acq_lenght= 8'h09; //sets the number of point to acquire for the FFT (2^18-lenght)
parameter command_en_windowing	= 8'h0a; //enables the windowing of the acquired data
parameter command_dis_windowing	= 8'h0b; //disables the windowing of the acquired data
parameter command_both_mode		= 8'h0c; //the FFT can start when the last raw data is transferred
parameter command_pll_fft_mode_on	= 8'h0d; //the data from the FFT should be modulated by the pll
parameter command_pll_fft_mode_off	= 8'h0e; //the data from the FFT should not be modulated by the pll (default)
parameter command_clear_all		= 8'h7f; //clears all the configuration and the acquisitions

reg [20:0] last_read_addr;


//FIXED ASSIGNMENTS

assign sram_bwe_n	= 1'b1;
assign sram_ce1_n	= 1'b0;
assign sram_clk		= clk;

assign sel_mem_space	= vme_addr[21];
assign sel_reg_space	= ~sel_mem_space;

assign fir_dec_dav	= first_fir_dec_dav && (fir_dec_dav_l || fir_dec_dav_r); //valid when both the channels have a windowed data

assign en_windowing	= window_req && acquiring;

assign trigger_acq	= en_trigger && trigger;

//REGISTERS

always @(posedge clk or negedge clrn) begin
	if (~clrn || ~acquiring) begin
		first_fir_dec_dav	<= # dly 1'b0;
	end else begin
		if (fir_dec_dav_l || fir_dec_dav_r) begin
			first_fir_dec_dav <= # dly ~first_fir_dec_dav;
		end
	end
end
			
always @(posedge clk or negedge clrn)begin
	if (~clrn) begin
		command_reg	<= # dly 32'h8000_0000; 
	end else if (vme_module_sel && vme_wr && sel_reg_space && vme_addr[5:2]==addr_command_reg) begin
		command_reg	<= # dly vme_datain;
	end else begin
		command_reg[31]	<= # dly 1'b1;
	end
end

assign command	= command_reg[31:24];

assign status_reg[31]		= 1'b0;
assign status_reg[30]		= pll_fft_mode;
assign status_reg[29]		= fir_dec_ovf_r_reg;
assign status_reg[28]		= fir_dec_ovf_l_reg;
assign status_reg[27]		= fft_overflow_reg;
assign status_reg[26]		= acquiring;
assign status_reg[25]		= ~raw_data_read;
assign status_reg[24]		= state==s_fft_rd || state==s_fft_wr;
assign status_reg[23]		= state==s_fft_done; //equivalent to ~fft_data_read
assign status_reg[22]		= trigger_armed;
assign status_reg[21]		= mode_normal;
assign status_reg[20] 		= raw_data_required;
assign status_reg[19]		= window_req;
assign status_reg[18]		= fft_data_required; 
assign status_reg[17:0]		= mem_pointer;

assign status_reg2[31:20]	= fir_dec_dav_counter[11:0];
assign status_reg2[19:16]	= fft_lenght;
assign status_reg2[11:9]	= state;
assign status_reg2[8]		= first_fir_dec_dav;
assign status_reg2[7]		= adc_r_wr_req;
assign status_reg2[6]		= adc_l_wr_req;
assign status_reg2[5]		= en_windowing;
assign status_reg2[4:0]		= pre_acq_lenght;


always @(posedge clk or negedge clrn)begin
	if (~clrn || command==command_clear_all) begin
		trig_number	<= # dly 32'h0;
	end else if (trigger) begin
		trig_number	<= # dly trig_number + 1'b1;
	end
end

always @(posedge clk or negedge clrn)begin
	if (~clrn || command==command_clear_all) begin
		fir_dec_dav_counter	<= # dly 1'b0;
	end else if (fir_dec_dav) begin
		fir_dec_dav_counter	<= # dly fir_dec_dav_counter + 1'b1;
	end
end		

// CONFIGURATION BITS

always @(posedge clk or negedge clrn)begin
	if (~clrn || command==command_clear_all) begin
		mode_normal	<= # dly 1'b0;
	end else if (command==command_normal_mode) begin
		mode_normal	<= # dly 1'b1;
	end else if (command==command_single_mode) begin
		mode_normal	<= # dly 1'b0;
	end
end
	
always @(posedge clk or negedge clrn)begin
	if (~clrn || command==command_clear_all) begin
		trigger_armed	<= # dly 1'b0;
	end else if (mode_normal || command==command_arm_trigger) begin
		trigger_armed	<= # dly 1'b1;
	end else if (trigger_acq) begin
		trigger_armed	<= # dly 1'b0;		
	end
end

always @(posedge clk or negedge clrn)begin
	if (~clrn || command==command_clear_all) begin
		raw_data_required	<= # dly 1'b1;
	end else if (command==command_fft_mode) begin
		raw_data_required	<= # dly 1'b0;
	end else if (command==command_raw_mode || command==command_both_mode) begin
		raw_data_required	<= # dly 1'b1;
	end
end

always @(posedge clk or negedge clrn)begin
	if (~clrn || command==command_clear_all) begin
		fft_data_required	<= # dly 1'b1;
	end else if (command==command_raw_mode) begin
		fft_data_required	<= # dly 1'b0;
	end else if (command==command_fft_mode || command==command_both_mode) begin
		fft_data_required	<= # dly 1'b1;
	end
end


always @(posedge clk or negedge clrn)begin
	if (~clrn || command==command_clear_all) begin
		window_req	<= # dly 1'b0;
	end else if (command==command_dis_windowing) begin
		window_req	<= # dly 1'b0;
	end else if (command==command_en_windowing) begin
		window_req	<= # dly 1'b1;
	end
end

always @(posedge clk or negedge clrn)begin
	if (~clrn || command==command_clear_all) begin
		pll_fft_mode	<= # dly 1'b0;
	end else if (command==command_pll_fft_mode_off) begin
		pll_fft_mode	<= # dly 1'b0;
	end else if (command==command_pll_fft_mode_on) begin
		pll_fft_mode	<= # dly 1'b1;
	end
end

always @(posedge clk or negedge clrn)begin
	if (~clrn || command==command_clear_all) begin
		fft_lenght	<= # dly 4'h4; //1024 points
	end else if (command==command_set_fft_lenght) begin
		fft_lenght	<= # dly command_reg[3:0];
	end
end

always @(posedge clk or negedge clrn)begin
	if (~clrn || command==command_clear_all) begin
		pre_acq_lenght	<= # dly 5'h0; //256k points :the real acq_lenght is fixed by the fft lenght
	end else if (command==command_set_acq_lenght) begin
		pre_acq_lenght	<= # dly command_reg[4:0];
	end
end

always @(posedge clk) begin
	if (pre_acq_lenght[4:0]<{fft_lenght, 1'b0}) begin
		acq_lenght	<= # dly {fft_lenght, 1'b0};
	end else begin
		acq_lenght	<= # dly pre_acq_lenght;
	end
end

//FIR&DECIMATOR OVERFLOW DURING ACQUISITION REGISTERS

always @(posedge clk or negedge clrn)begin
	if (~clrn || (trigger_armed && trigger_acq)) begin
		fir_dec_ovf_l_reg	<= # dly 1'b0;
	end else if (acquiring && fir_dec_ovf_l) begin
			fir_dec_ovf_l_reg	<= # dly 1'b1;
	end 
end

always @(posedge clk or negedge clrn)begin
	if (~clrn || (trigger_armed && trigger_acq)) begin
		fir_dec_ovf_r_reg	<= # dly 1'b0;
	end else if (acquiring && fir_dec_ovf_r) begin
		fir_dec_ovf_r_reg	<= # dly 1'b1;
	end 
end

//FFT CONNECTIONS

assign fft_x_t	= sram_data_synch;

always @(posedge clk or negedge clrn) begin
	if (~clrn || acquiring) begin
		fft_overflow_reg	<= # dly 1'b0;
	end else if ((state==s_fft_rd || state==s_fft_wr)&& fft_overflow) begin
		fft_overflow_reg	<= # dly 1'b1;
	end
end

// STATE MACHINE

assign sram_data	= oe_data_int ? sram_data_int : 32'hz;

always @(posedge clk) begin
	case(fft_lenght)
		4'h0: fft_end_address <= # dly  {3'h1, 18'h3ffff};
		4'h1: fft_end_address <= # dly  {3'h1, 18'h0ffff};
		4'h2: fft_end_address <= # dly  {3'h1, 18'h03fff};
		4'h3: fft_end_address <= # dly  {3'h1, 18'h00fff};
		4'h4: fft_end_address <= # dly  {3'h1, 18'h003ff};
		4'h5: fft_end_address <= # dly  {3'h1, 18'h000ff};
		4'h6: fft_end_address <= # dly  {3'h1, 18'h0003f};
		4'h7: fft_end_address <= # dly  {3'h1, 18'h0000f};
		default: fft_end_address	<= # dly  {3'h1, 18'h00003};
	endcase
end

always @(posedge clk) begin
	case(acq_lenght)
		5'h00: acq_end_address <= # dly  {3'h1, 18'h3ffff};
		5'h01: acq_end_address <= # dly  {3'h1, 18'h1ffff};
		5'h02: acq_end_address <= # dly  {3'h1, 18'h0ffff};
		5'h03: acq_end_address <= # dly  {3'h1, 18'h07fff};
		5'h04: acq_end_address <= # dly  {3'h1, 18'h03fff};
		5'h05: acq_end_address <= # dly  {3'h1, 18'h01fff};
		5'h06: acq_end_address <= # dly  {3'h1, 18'h00fff};
		5'h07: acq_end_address <= # dly  {3'h1, 18'h007ff};
		5'h08: acq_end_address <= # dly  {3'h1, 18'h003ff};
		5'h09: acq_end_address <= # dly  {3'h1, 18'h001ff};
		5'h0a: acq_end_address <= # dly  {3'h1, 18'h000ff};
		5'h0b: acq_end_address <= # dly  {3'h1, 18'h0007f};
		5'h0c: acq_end_address <= # dly  {3'h1, 18'h0003f};
		5'h0d: acq_end_address <= # dly  {3'h1, 18'h0001f};
		5'h0e: acq_end_address <= # dly  {3'h1, 18'h0000f};
		5'h0f: acq_end_address <= # dly  {3'h1, 18'h00007};
		default: acq_end_address	<= # dly  {3'h1, 18'h00003};
	endcase
end

always @(posedge clk) begin
	case(fft_iteration)
		4'h0: end_n_2 <= # dly  16'hffff;
		4'h1: end_n_2 <= # dly  16'h3fff;
		4'h2: end_n_2 <= # dly  16'h0fff;
		4'h3: end_n_2 <= # dly  16'h03ff;
		4'h4: end_n_2 <= # dly  16'h00ff;
		4'h5: end_n_2 <= # dly  16'h003f;
		4'h6: end_n_2 <= # dly  16'h000f;
		4'h7: end_n_2 <= # dly  16'h0003;
		default: end_n_2	<= # dly  16'h0000;
	endcase
end

always@ (posedge clk) sram_data_synch <= # dly sram_data;

//reg [31:0] addr0, addr1, addr2;


always @(posedge clk or negedge clrn)begin
	if (~clrn ||  command==command_clear_all) begin
		state			<= # dly s_idle;
		sram_data_int	<= # dly 32'h0;
		oe_data_int		<= # dly 1'b0;
		sram_addr		<= # dly 21'b0;
		sram_oe_n		<= # dly 1'b1;
		sram_gwe_n		<= # dly 1'b1;
		vme_dout_dav_mem<= # dly 1'b0;	
		counter			<= # dly 5'h0;
		adc_l_wr_req	<= # dly 1'b0;
		adc_r_wr_req	<= # dly 1'b0;
		acquiring		<= # dly 1'b0;
		raw_data_read	<= # dly 1'b1;
		fft_data_read	<= # dly 1'b1;
		mem_pointer		<= # dly 18'h0;
		time_stamp		<= # dly 32'h0;
		fft_start		<= # dly 1'b0;
		fft_iteration	<= # dly 4'b0;
		fft_n_2			<= # dly 16'h0;
		sram_addr_inc	<= # dly 18'd256;	
		fft_mod_req		<= # dly 1'b0;
		int_for_raw		<= # dly 1'b0;
		int_for_fft		<= # dly 1'b0;
		last_read_addr	<= # dly 21'h0;
	end else begin
		case (state)
		s_idle: begin
			int_for_raw		<= # dly 1'b0;
			int_for_fft		<= # dly 1'b0;
			last_read_addr	<= # dly 21'h0;		
			if (trigger_armed && trigger_acq) begin		
				state			<= # dly s_idle; //we delay the acquisition start of 1 clock cycle to synchronize properely with the window
				sram_data_int	<= # dly 32'h0;
				oe_data_int		<= # dly 1'b0;
				sram_addr		<= # dly 21'b0;
				sram_oe_n		<= # dly 1'b1;
				sram_gwe_n		<= # dly 1'b1;	
				vme_dout_dav_mem<= # dly 1'b0;
				counter			<= # dly 5'h0;						
				adc_l_wr_req	<= # dly 1'b0;
				adc_r_wr_req	<= # dly 1'b0;
				acquiring		<= # dly 1'b1;
				raw_data_read	<= # dly 1'b0;
				fft_data_read	<= # dly 1'b0;
				mem_pointer		<= # dly 18'h0;	
			end else if (acquiring) begin	//we delay the acquisition start of 1 clock cycle to synchronize properely with the window	
				state			<= # dly s_acquiring;
				sram_data_int	<= # dly 32'h0;
				oe_data_int		<= # dly 1'b0;
				sram_addr		<= # dly 21'b0;
				sram_oe_n		<= # dly 1'b1;
				sram_gwe_n		<= # dly 1'b1;	
				vme_dout_dav_mem<= # dly 1'b0;
				counter			<= # dly 5'h0;						
				adc_l_wr_req	<= # dly 1'b0;
				adc_r_wr_req	<= # dly 1'b0;
				acquiring		<= # dly 1'b1;
				raw_data_read	<= # dly 1'b0;
				fft_data_read	<= # dly 1'b0;
				mem_pointer		<= # dly 18'h0;	
				time_stamp		<= # dly trig_number;
			end else if (vme_module_sel && sel_mem_space && vme_wr) begin
				sram_addr		<= # dly {2'b0, vme_addr[20:2]};
				sram_data_int	<= # dly vme_datain;
				oe_data_int		<= # dly 1'b1;
				sram_oe_n		<= # dly 1'b1;
				sram_gwe_n		<= # dly 1'b0;
				counter			<= # dly 5'h0;						
			end else if (vme_module_sel && sel_mem_space && vme_rd) begin
				state			<= # dly s_vme_rd;
				oe_data_int		<= # dly 1'b0;
				sram_addr		<= # dly {2'b0, vme_addr[20:2]};
				sram_oe_n		<= # dly 1'b1;
				sram_gwe_n		<= # dly 1'b1;
				counter			<= # dly 5'h0;											
			end else if (command==command_start_fft) begin
				state			<= # dly s_fft_rd;
				sram_data_int	<= # dly 32'h0;
				oe_data_int		<= # dly 1'b0;
				sram_addr		<= # dly 21'h0;
				sram_oe_n		<= # dly 1'b1;
				sram_gwe_n		<= # dly 1'b1;
				vme_dout_dav_mem<= # dly 1'b0;
				counter			<= # dly 5'h0;
				acquiring		<= # dly 1'b0;
				raw_data_read	<= # dly 1'b1;
				fft_data_read	<= # dly 1'b0;	
				mem_pointer		<= # dly 18'h0;	
				fft_start		<= # dly 1'b0;
				fft_iteration	<= # dly fft_lenght;
				fft_n_2			<= # dly 16'h0;
			end else begin		
				state			<= # dly s_idle;
				sram_data_int	<= # dly 32'h0;
				oe_data_int		<= # dly 1'b0;
				sram_addr		<= # dly 21'b0;
				sram_oe_n		<= # dly 1'b1;
				sram_gwe_n		<= # dly 1'b1;	
				vme_dout_dav_mem<= # dly 1'b0;
				counter			<= # dly 5'h0;
				adc_l_wr_req	<= # dly 1'b0;
				adc_r_wr_req	<= # dly 1'b0;
				acquiring		<= # dly 1'b0;
				raw_data_read	<= # dly 1'b1;
				fft_data_read	<= # dly 1'b1;	
				mem_pointer		<= # dly 18'h0;	
			end
			case (fft_lenght)
				4'h0: sram_addr_inc	<= # dly 18'h1_0000;
				4'h1: sram_addr_inc	<= # dly 18'h4000;
				4'h2: sram_addr_inc	<= # dly 18'h1000;
				4'h3: sram_addr_inc	<= # dly 18'h400;
				4'h4: sram_addr_inc	<= # dly 18'h100;
				4'h5: sram_addr_inc	<= # dly 18'h40;
				4'h6: sram_addr_inc	<= # dly 18'h10;
				4'h7: sram_addr_inc	<= # dly 18'h4;
				default: sram_addr_inc	<= # dly 18'h1;
			endcase		
		end
		s_vme_rd: begin
			if (counter==5'h2) begin
				if (~raw_data_read) begin
					state	<= # dly s_acquiring;
				end else if (~fft_data_read) begin
					state	<= # dly s_fft_done;
				end else begin
					state	<= # dly s_idle;
				end
				sram_oe_n		<= # dly 1'b1;
				vme_dout_dav_mem<= # dly 1'b1;
				counter			<= # dly 5'h0;
				if (sram_addr==acq_end_address) begin
					raw_data_read	<= # dly 1'b1;
				end
				if (sram_addr==fft_end_address && raw_data_read) begin
					fft_data_read	<= # dly 1'b1;
				end
			end else begin
				sram_oe_n		<= # dly 1'b0;
				counter			<= # dly counter + 1'b1;
			end
			if (fir_dec_dav) begin
				adc_l_wr_req	<= # dly 1'b1;
				adc_r_wr_req	<= # dly 1'b1;
			end						
		end	
		s_acquiring: begin
			vme_dout_dav_mem<= # dly 1'b0;
			if (vme_module_sel && sel_mem_space && vme_rd  && ~raw_data_read) begin
				state			<= # dly s_vme_rd;
				oe_data_int		<= # dly 1'b0;
				sram_addr		<= # dly {2'b0, vme_addr[20:2]};
				sram_oe_n		<= # dly 1'b1;
				sram_gwe_n		<= # dly 1'b1;
				counter			<= # dly 5'h0;
				if (fir_dec_dav) begin
					adc_l_wr_req	<= # dly 1'b1;
					adc_r_wr_req	<= # dly 1'b1;
				end	
				if (~acquiring) begin
					last_read_addr	<= # dly {2'b0, vme_addr[20:2]};
				end
			end else if (acquiring && (fir_dec_dav || adc_r_wr_req || adc_l_wr_req)) begin
				oe_data_int		<= # dly 1'b1;
				sram_oe_n		<= # dly 1'b1;
				sram_gwe_n		<= # dly 1'b0;
				if (fir_dec_dav || adc_r_wr_req) begin
					sram_addr		<= # dly {3'b0, mem_pointer};
					sram_data_int	<= # dly fir_dec_dout_r;
					adc_r_wr_req	<= # dly 1'b0;
					adc_l_wr_req	<= # dly 1'b1;
				end else begin
					sram_addr		<= # dly {3'b1, mem_pointer};
					sram_data_int	<= # dly fir_dec_dout_l;
					mem_pointer		<= # dly mem_pointer + 1'b1;
					adc_l_wr_req	<= # dly 1'b0;
					if (mem_pointer==acq_end_address[17:0]) begin
						acquiring		<= # dly 1'b0;
						raw_data_read	<= # dly ~raw_data_required;
						int_for_raw		<= # dly 1'b1;
					end
				end 
			end else begin
				oe_data_int		<= # dly 1'b0;
				sram_oe_n		<= # dly 1'b1;
				sram_gwe_n		<= # dly 1'b1;
				if (~acquiring && raw_data_read) begin
					if (~fft_data_required) begin
						state	<= # dly s_idle;
					end else if (acq_end_address[17:0]==fft_end_address[17:0]) begin
						fft_mod_req		<= # dly 1'b1;
						sram_addr		<= # dly 21'h0;
						counter			<= # dly 5'h0;
						fft_start		<= # dly 1'b0;
						fft_iteration	<= # dly fft_lenght;
						fft_n_2			<= # dly 16'h0;
						if (fft_acc_granted) begin		
							state			<= # dly s_fft_rd;
						end
					end else begin
						sram_addr		<= # dly 21'h0;
						counter			<= # dly 5'h0;
						state			<= # dly s_0padding;
					end					
				end
			end
		end
		s_0padding:begin
			sram_data_int	<= # dly 32'h0;
			if (fft_mod_req) begin
				oe_data_int		<= # dly 1'b0;
				sram_oe_n		<= # dly 1'b1;			
				sram_gwe_n		<= # dly 1'b1;			
				fft_mod_req		<= # dly 1'b1;
				sram_addr		<= # dly 21'h0;
				counter			<= # dly 5'h0;
				fft_start		<= # dly 1'b0;
				fft_iteration	<= # dly fft_lenght;
				fft_n_2			<= # dly 16'h0;
				if (fft_acc_granted) begin
					state			<= # dly s_fft_rd;
				end
			end else begin
				counter			<= # dly counter + 1'b1;
				oe_data_int		<= # dly 1'b1;
				sram_oe_n		<= # dly 1'b1;
				sram_gwe_n		<= # dly 1'b0;
				sram_addr		<= # dly {2'b0, counter[0], mem_pointer};
				if (counter[0]) begin
					mem_pointer		<= # dly mem_pointer + 1'b1;
					if (mem_pointer==fft_end_address[17:0]) begin
						fft_mod_req		<= # dly 1'b1;			
					end
				end	
			end
		end	
		s_fft_rd: begin
			sram_data_int	<= # dly fft_x_f;
			if (counter==5'h0) begin
				mem_pointer	<= # dly sram_addr[17:0];
			end
			if (counter==5'd11) begin
				state				<= # dly s_fft_wr;
				oe_data_int			<= # dly 1'b1;				
				sram_gwe_n			<= # dly 1'b0;
				counter				<= # dly 5'h0;
				sram_addr[20:18]	<= # dly 3'b000;
				sram_addr[17:0]		<= # dly mem_pointer;
				mem_pointer			<= # dly mem_pointer + 1'b1;
			end else begin
				fft_start			<= # dly counter==5'h2;
				sram_oe_n			<= # dly counter>5'h8;				
				counter				<= # dly counter + 1'b1;
				sram_addr[20:19]	<= # dly 2'b00;
				sram_addr[18]		<= # dly ~sram_addr[18];
				sram_addr[17:0]		<= # dly sram_addr[18] ? sram_addr[17:0] + sram_addr_inc : sram_addr[17:0];
			end
		end
		s_fft_wr: begin
			sram_data_int	<= # dly fft_x_f;
			if (counter==5'h7) begin
				fft_iteration	<= # dly &(sram_addr[17:0]|~fft_end_address[17:0]) ? (fft_iteration	+ 1'b1) : fft_iteration;			
				sram_addr_inc	<= # dly &(sram_addr[17:0]|~fft_end_address[17:0]) ? {2'b0, sram_addr_inc[17:2]} : sram_addr_inc;	
				fft_n_2			<= # dly (fft_n_2==end_n_2) ? 16'b0 : fft_n_2 + 1'b1;		
				sram_gwe_n		<= # dly 1'b1;
				if (fft_iteration==4'h8 && &(sram_addr[17:0]|~fft_end_address[17:0])) begin 
					state	<= # dly s_fft_done;		
				end else begin
					state	<= # dly s_fft_rd;
				end
				oe_data_int			<= # dly 1'b0;				
				counter				<= # dly 5'h0;
				sram_addr[20:18]	<= # dly 3'b000;
				if ( &(sram_addr[17:0]|~fft_end_address[17:0])) begin
					sram_addr[17:0]	<= # dly  18'h0;
				end else if (fft_n_2==end_n_2) begin
					sram_addr[17:0]	<= # dly  sram_addr[17:0] + 1'b1;		
				end else begin
					sram_addr[17:0]	<= # dly  mem_pointer;	
				end	
			end else begin
				oe_data_int		<= # dly 1'b1;
				counter			<= # dly counter + 1'b1;
				sram_addr[18]	<= # dly ~sram_addr[18];
				sram_addr[17:0]	<= # dly sram_addr[18] ? sram_addr[17:0] + sram_addr_inc : sram_addr[17:0];
			end		
		end
		s_fft_done: begin
			int_for_fft			<= # dly 1'b1;
			fft_mod_req			<= # dly 1'b0;
			vme_dout_dav_mem	<= # dly 1'b0;
			if (fft_data_read) begin
				state	<= # dly s_idle;
			end else if (vme_module_sel && sel_mem_space && vme_rd) begin
				state			<= # dly s_vme_rd;
				oe_data_int		<= # dly 1'b0;
				sram_oe_n		<= # dly 1'b1;
				sram_gwe_n		<= # dly 1'b1;
				counter			<= # dly 5'h0;
				sram_addr[19:18]<= # dly {1'b0, vme_addr[20]};
				case (fft_lenght) 
					4'h0: sram_addr[17:0] <= # dly {vme_addr[3:2], vme_addr[5:4], vme_addr[7:6], vme_addr[9:8], vme_addr[11:10], vme_addr[13:12], vme_addr[15:14], vme_addr[17:16], vme_addr[19:18]};
					4'h1: sram_addr[17:0] <= # dly {vme_addr[19:18], vme_addr[3:2], vme_addr[5:4], vme_addr[7:6], vme_addr[9:8], vme_addr[11:10], vme_addr[13:12], vme_addr[15:14], vme_addr[17:16]};
					4'h2: sram_addr[17:0] <= # dly {vme_addr[19:16], vme_addr[3:2], vme_addr[5:4], vme_addr[7:6], vme_addr[9:8], vme_addr[11:10], vme_addr[13:12], vme_addr[15:14]};
					4'h3: sram_addr[17:0] <= # dly {vme_addr[19:14], vme_addr[3:2], vme_addr[5:4], vme_addr[7:6], vme_addr[9:8], vme_addr[11:10], vme_addr[13:12]};
					4'h4: sram_addr[17:0] <= # dly {vme_addr[19:12], vme_addr[3:2], vme_addr[5:4], vme_addr[7:6], vme_addr[9:8], vme_addr[11:10]};
					4'h5: sram_addr[17:0] <= # dly {vme_addr[19:10], vme_addr[3:2], vme_addr[5:4], vme_addr[7:6], vme_addr[9:8]};
					4'h6: sram_addr[17:0] <= # dly {vme_addr[19:8], vme_addr[3:2], vme_addr[5:4], vme_addr[7:6]};
					4'h7: sram_addr[17:0] <= # dly {vme_addr[19:6], vme_addr[3:2], vme_addr[5:4]};
					default: sram_addr[17:0]	<= # dly  vme_addr[19:2];
				endcase			
			end	
		end
		default: begin
			state			<= # dly s_idle;
			sram_data_int	<= # dly 32'h0;
			oe_data_int		<= # dly 1'b0;
			sram_addr		<= # dly 21'b0;
			sram_oe_n		<= # dly 1'b1;
			sram_gwe_n		<= # dly 1'b1;
			vme_dout_dav_mem<= # dly 1'b0;	
			counter			<= # dly 5'h0;
			adc_l_wr_req	<= # dly 1'b0;
			adc_r_wr_req	<= # dly 1'b0;
			acquiring		<= # dly 1'b0;
			raw_data_read	<= # dly 1'b1;
			fft_data_read	<= # dly 1'b1;
			mem_pointer		<= # dly 18'h0;
			time_stamp		<= # dly 32'h0;
			fft_start		<= # dly 1'b0;
			fft_iteration	<= # dly 4'b0;
			fft_n_2			<= # dly 16'h0;	
		end
		endcase
	end
end



//VME DOUT & DOUT_DAV

always @(*) begin
	if (sel_mem_space) begin
		vme_dout	= sram_data_synch;
	end else begin
		case (vme_addr[5:2])
		addr_status_reg:	vme_dout = status_reg;
		addr_status_reg2:	vme_dout = status_reg2;
		addr_command_reg:	vme_dout = command_reg;
		addr_trig_number:	vme_dout = trig_number;
		addr_time_stamp:	vme_dout = time_stamp;
		default:			vme_dout = 32'hdeadbeef;
		endcase
	end
end

always @(*) begin 
	if (sel_mem_space) begin
		vme_dout_dav	= vme_dout_dav_mem;
		vme_retry_n		= ~(vme_rd && (state==s_fft_rd || state==s_fft_wr));
	end else begin
		vme_dout_dav	= vme_rd;
		vme_retry_n		= 1'b1;
	end
end		
endmodule				
				