`timescale 1ns/1ns

module pll_fp_det(
	input	clk, 
	input	clrn,
	
	input	ref,
	input	pll_out,
	output	reg err_dav,
	output	reg [15:0] err);

parameter dly = 1;

reg early, late;
reg early_synch, late_synch;
wire earlylate;

assign earlylate	= early_synch && late_synch;

always @(posedge pll_out or posedge earlylate) begin
	if (earlylate) begin
		early	<= # dly 1'b0;
	end else begin
		early	<= # dly 1'b1;
	end
end	

always @(posedge ref or posedge earlylate) begin
	if (earlylate) begin
		late	<= # dly 1'b0;
	end else begin
		late	<= # dly 1'b1;
	end
end	

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		early_synch	<= # dly 1'b0;
		late_synch	<= # dly 1'b0;
	end else begin
		early_synch	<= # dly early;
		late_synch	<= # dly late;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		err_dav	<= # dly 1'b0;
	end else begin
		err_dav	<= # dly early_synch && late_synch;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn || err_dav) begin  
		err	<= # dly 16'h0;
	end else if (late_synch && ~early_synch) begin
		err	<= # dly err + 1'b1;
	end else if (early_synch && ~late_synch) begin
		err	<= # dly err - 1'b1;
	end
end
		
endmodule