`timescale 1ns/1ns

module pll_pid(
	input	clk, 
	input	clrn,
	
	input	vme_cs,
	input	vme_wr,
	input	vme_rd,
	input	[4:2] vme_addr, 
	input	[31:0] vme_din,
	output	reg vme_dav,
	output	reg	[31:0] vme_dout,
	
	input	err_dav,
	input	[35:0] err,
	output	reg df_dav, 
	output	reg [31:0] df); 

//parameter a0_def	= 32'h0001_0000;	
//parameter a1_def	= 32'h0001_0000;	//inverted when assigned
//parameter a2_def	= 32'h0;	
//parameter b1_def	= 32'h0;	

parameter freq_def	= 32'd68_000_000;//about 2.4kHz
	
parameter dly = 1;	
		
reg [31:0] a0, a1, a2, b1;
reg [35:0] err_z0, err_z1, err_z2;
reg [68:0] accumulator; //32+35+2 bits 

reg	[2:0] state;	

parameter s_idle	= 2'd0;	
parameter s_a1		= 2'd1;
parameter s_a2		= 2'd2;
parameter s_b1		= 2'd3;

parameter addr_a0		= 3'h0;
parameter addr_a1		= 3'h1;
parameter addr_a2		= 3'h2;
parameter addr_b1		= 3'h3;

reg	[35:0] mult_data, mult_coef;
wire [71:0] mult_out; 
reg df_dav_int;

always @(posedge clk) begin
	if (vme_cs && vme_wr && vme_addr==addr_a0) begin
		a0	<= # dly vme_din;
	end
end

always @(posedge clk) begin
	if (vme_cs && vme_wr && vme_addr==addr_a1) begin
		a1	<= # dly vme_din;
	end
end

always @(posedge clk) begin
	if (vme_cs && vme_wr && vme_addr==addr_a2) begin
		a2	<= # dly vme_din;
	end
end

always @(posedge clk) begin
	if (vme_cs && vme_wr && vme_addr==addr_b1) begin
		b1	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		err_z0	<= # dly 36'h0;
		err_z1	<= # dly 36'h0;
		err_z2	<= # dly 36'h0;
	end else if (err_dav) begin
		err_z0	<= # dly err;
		err_z1	<= # dly err_z0;
		err_z2	<= # dly err_z1;
	end
end
				
assign mult_out	= $signed(mult_data)*$signed(mult_coef);

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		state		<= # dly s_idle;
		mult_data	<= # dly 36'h0;
		mult_coef	<= # dly 36'h0;
		accumulator	<= # dly 69'h0;
		df_dav_int	<= # dly 1'b0;
	end else begin
		case (state)
		s_idle: begin
			df_dav_int		<= # dly 1'b0;		
			if (err_dav) begin
				state		<= # dly s_a1;
				accumulator	<= # dly mult_out[68:0]; //df_z1*b1 (already calculated)
				mult_data	<= # dly err;
				mult_coef	<= # dly {{4{a0[31]}}, a0};
			end
		end
		s_a1: begin
			state		<= # dly s_a2;
			accumulator	<= # dly accumulator + mult_out[68:0];
			mult_data	<= # dly err_z1;
			mult_coef	<= # dly {{4{a1[31]}}, a1};
			df_dav_int	<= # dly 1'b0;
		end
		s_a2: begin
			state		<= # dly s_b1;
			accumulator	<= # dly accumulator + mult_out[68:0];
			mult_data	<= # dly err_z1;
			mult_coef	<= # dly {{4{a0[31]}}, a2};
			df_dav_int	<= # dly 1'b0;
		end
		s_b1: begin
			state		<= # dly s_idle;
			accumulator	<= # dly accumulator + mult_out[68:0];
			mult_data	<= # dly accumulator[68:33];
			mult_coef	<= # dly {{4{b1[31]}}, b1};
			df_dav_int	<= # dly 1'b1;
		end
		default: begin
			state		<= # dly s_idle;
			mult_data	<= # dly 36'h0;
			mult_coef	<= # dly 36'h0;
			accumulator	<= # dly 69'h0;
			df_dav_int	<= # dly 1'b0;
		end
		endcase
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		df_dav	<= # dly 1'b0;
		df		<= # dly 32'h0;
	end else begin
		df_dav	<= # dly df_dav_int;
		if (df_dav_int) begin 
			df	<= # dly accumulator[68:37];
		end
	end
end

always @* begin
	vme_dav	= vme_rd;
	case (vme_addr)
	addr_a0: vme_dout	= a0;
	addr_a1: vme_dout	= a1;
	addr_a2: vme_dout	= a2;
	addr_b1: vme_dout	= b1;
	default: vme_dout	= 32'h0;
	endcase
end

endmodule