`timescale 1ns/1ns

module q_pll
	(input	clk,
	input 	clrn,
	
	input	[31:0] ext_amplitude,
	input	[39:0] ext_phase,
	
	input 	start,
	input	[31:0] ampl_i,
	input	[31:0] ampl_q,
	output	done,
	output	[35:0] cos_pll,
	output	[35:0] sin_pll,
	output	[35:0] cos_se1,
	output	[35:0] sin_se1,
	output	[35:0] cos_se2,
	output	[35:0] sin_se2,
	
	input	hw_cycle_start,
	
	output	[35:0] mod_ampl_i,	
	
	input	vme_cs,
	input	vme_wr,
	input	vme_rd,
	input	[5:0] vme_addr, 
	input	[31:0] vme_din,
	output	reg vme_dav,
	output	reg	[31:0] vme_dout);

parameter dly = 1;

parameter reg_space_base_addr	= 1'd0;
parameter pid_space_base_addr	= 1'd1;

parameter pid_a_base_addr		= 2'd0;
parameter pid_p_base_addr		= 2'd1;
parameter pid_w_base_addr		= 2'd2;

parameter addr_status		= 5'd0;
parameter addr_ampl_dem_q	= 5'd1;
parameter addr_phy_dem_q	= 5'd2;
parameter addr_ampl_dem_i	= 5'd3;
parameter addr_phy_dem_i	= 5'd4;
parameter addr_des_ampl		= 5'd5;
parameter addr_phy_corr		= 5'd6;
parameter addr_oow_a0_limit	= 5'd7;
parameter addr_oow_a1_limit	= 5'd8;
parameter addr_oow_ampl		= 5'd9;
parameter addr_oow_df		= 5'd10;
parameter addr_oow_f1		= 5'd11;
parameter addr_oow_f2		= 5'd12;
parameter addr_pll_freq		= 5'd13;
parameter addr_pll_ampl		= 5'd14;
parameter addr_dphy_df		= 5'd15;

parameter addr_wpll_mod_ampl	= 5'd16;
parameter addr_wpll_mod_freq	= 5'd17;
parameter addr_des_phy_mod		= 5'd18;
parameter addr_phy_mod_depth	= 5'd19;
parameter addr_se12_ampl		= 5'd20;
parameter addr_se12_dfreq		= 5'd21;
parameter addr_se1_ampl_dem		= 5'd22;
parameter addr_se1_phy_dem		= 5'd23;
parameter addr_se2_ampl_dem		= 5'd24;
parameter addr_se2_phy_dem		= 5'd25;



reg [39:0] phase_pll; 
wire [39:0] phase_exciter;
reg [41:0] ampl_pll; 
wire [41:0] ampl_exciter;  
wire [35:0] cos_pll_ext, sin_pll_ext;
wire [35:0] cos_se1_ext, sin_se1_ext;
wire [35:0] cos_se2_ext, sin_se2_ext;
wire done_iq;
wire [35:0] ampl_dem_i, ampl_dem_q;
wire [39:0] phy_dem_i, phy_dem_q;
reg	[31:0] des_ampl, phy_corr;
wire vme_reg_space_cs, vme_pid_a_cs, vme_pid_b_cs;
wire [35:0] pid_a_input, pid_p_input;
wire [31:0] vme_dout_a_pid, vme_dout_p_pid;
wire vme_dav_a_pid, vme_dav_p_pid;
wire dampl_dav, df_dav;
wire [31:0] dampl, df;
reg [31:0] oow_a0_limit, oow_a1_limit, oow_ampl;
reg oow, oow_old;
reg [15:0] status_reg;
wire disable_oow_ampl, disable_oow_phase, disable_ampl_control;
reg [31:0] freq_pll;
reg [31:0] oow_df, oow_f1, oow_f2;
reg pos_neg_chirp;
wire ext_mode;
reg clrn_ampl_pid;
reg clrn_phase_pid;
wire force_oow;
reg [31:0] dphy_df;
wire enable_hw_cycle_start;

wire disable_w_pll_mod;
wire mcginn_mode;
wire disable_se;

reg [31:0] wpll_mod_ampl, wpll_mod_freq, des_phy_mod;
reg [39:0] phy_wmod_freq;

reg [31:0] se12_ampl, se12_dfreq; 
wire [31:0] se1_ampl_dem, se2_ampl_dem;
wire [39:0] se1_phy_dem, se2_phy_dem;
reg [39:0] phy_se1, phy_se2;
		
reg [39:0] phase_pll_mem [31:0]; 
wire [4:0] phase_pll_dly;
reg [4:0] phase_pll_mem_pntr;
wire [39:0] phase_pll_mem_out;

//*******************************
// BASE ADDRESSES DEFINITION
//*******************************

assign vme_reg_space_cs	= vme_addr[5]==reg_space_base_addr && vme_cs;
assign vme_pid_a_cs		= vme_addr[5]==pid_space_base_addr && vme_addr[4:3]==pid_a_base_addr && vme_cs;
assign vme_pid_p_cs		= vme_addr[5]==pid_space_base_addr && vme_addr[4:3]==pid_p_base_addr && vme_cs;

//*******************************
// external mode controls
//*******************************

assign phase_exciter	= ext_mode	? ext_phase : phase_pll;
assign ampl_exciter		= ext_mode	? {1'b0, ext_amplitude, 9'h0} : ampl_pll;

//********************************
// cordic mixer - demixer
//********************************

wire [35:0] demod_phy, mcginn_phy_out, phy_mod_ampl;

cordic_demodulator demod (
	.clk(clk),
	.clrn(clrn),
	
	.phase_pll(phase_exciter),
	.ampl_pll(ampl_exciter),
	.phase_pll_dly(phase_pll_mem_out),
	.ampl_i({{2{ampl_i[31]}} ,ampl_i , 8'b0}),
	.ampl_q({{2{ampl_q[31]}} ,ampl_q , 8'b0}),
	.phase_mod(phy_wmod_freq),
	.phase_se1(phy_se1),
	.phase_se2(phy_se2),
	.ampl_se12({1'b0, se12_ampl, 9'h0}),
	.start(start),
	
	.done_pll(done),
	.cos_out_pll(cos_pll_ext),
	.sin_out_pll(sin_pll_ext),
	.cos_out_se1(cos_se1_ext),
	.sin_out_se1(sin_se1_ext),
	.cos_out_se2(cos_se2_ext),
	.sin_out_se2(sin_se2_ext),
	
	.done_iq(done_iq),
	.ampl_out_i(ampl_dem_i),
	.phase_out_i(phy_dem_i),
	.ampl_out_q(ampl_dem_q),
	.phase_out_q(phy_dem_q),
	.ampl_out_se1(se1_ampl_dem),
	.phase_out_se1(se1_phy_dem),	
	.ampl_out_se2(se2_ampl_dem),
	.phase_out_se2(se2_phy_dem),

	.demod_phy_out(demod_phy),
	.phy_mod_ampl(phy_mod_ampl),
	.mcginn_phy_out(mcginn_phy_out));
	
assign cos_pll	= cos_pll_ext[35:0]; 
assign sin_pll	= sin_pll_ext[35:0]; 
assign cos_se1	= cos_se1_ext[35:0]; 
assign sin_se1	= sin_se1_ext[35:0]; 
assign cos_se2	= cos_se2_ext[35:0]; 
assign sin_se2	= sin_se2_ext[35:0]; 
assign mod_ampl_i	= mcginn_mode	? mcginn_phy_out	:	{demod_phy -  phy_dem_i[39:4]};

//***********************************
// FEEDBACK LOOPS
//***********************************

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		des_ampl	<= # dly 32'h0040_0000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_des_ampl) begin
		des_ampl	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phy_corr	<= # dly 32'h0000_0000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_phy_corr) begin
		phy_corr	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		dphy_df	<= # dly 32'h0000_0000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_dphy_df) begin
		dphy_df	<= # dly vme_din;
	end
end

reg [71:0] mult_out;
wire [35:0] mult_data, mult_coef;

assign mult_data	= {freq_pll, 4'b0};
assign mult_coef	= {dphy_df, 4'b0};

always @(posedge clk) begin 
	mult_out		<= # dly mult_data*mult_coef;
end

wire [31:0] se1_phy_out, se2_phy_out;
wire [31:0] phy_dem_q_out;

assign pid_a_input	= {1'b0, des_ampl, 3'b0} - ampl_dem_i;
assign pid_p_input	= {phy_corr, 4'b0} - phy_dem_i[39:4] + mult_out[57:22];
assign se1_phy_out	= phy_corr - se1_phy_dem[39:8] + mult_out[57:26];
assign se2_phy_out	= phy_corr - se2_phy_dem[39:8] + mult_out[57:26];
assign phy_dem_q_out	= phy_corr - phy_dem_q[39:8] + mult_out[57:26];

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		clrn_ampl_pid	<= # dly 1'b0;
		clrn_phase_pid	<= # dly 1'b0;
	end else begin
		clrn_ampl_pid	<= # dly  ~(disable_ampl_control || (oow && ~disable_oow_ampl));		
		clrn_phase_pid	<= # dly ~(oow && ~disable_oow_phase);
	end
end	
		
pll_pid amplitude_pid(
	.clk(clk), 
	.clrn(clrn_ampl_pid),
	.vme_cs(vme_pid_a_cs),
	.vme_wr(vme_wr),
	.vme_rd(vme_rd),
	.vme_addr(vme_addr[2:0]), 
	.vme_din(vme_din),
	.vme_dav(vme_dav_a_pid),
	.vme_dout(vme_dout_a_pid),	
	.err_dav(done_iq),
	.err(pid_a_input),
	.df_dav(dampl_dav), 
	.df(dampl));
	
pll_pid phase_pid(
	.clk(clk), 
	.clrn(clrn_phase_pid),
	.vme_cs(vme_pid_p_cs),
	.vme_wr(vme_wr),
	.vme_rd(vme_rd),
	.vme_addr(vme_addr[2:0]), 
	.vme_din(vme_din),
	.vme_dav(vme_dav_p_pid),
	.vme_dout(vme_dout_p_pid),	
	.err_dav(done_iq),
	.err(pid_p_input),
	.df_dav(df_dav), 
	.df(df));

//****************************************
//OUT OF (LOCKING) WINDOW SETTINGS
//****************************************	
		
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		oow_a0_limit	<= # dly 32'd1_000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_oow_a0_limit) begin
		oow_a0_limit	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		oow_a1_limit	<= # dly 32'd360_000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_oow_a1_limit) begin
		oow_a1_limit	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		oow_ampl	<= # dly 32'h4000_0000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_oow_ampl) begin
		oow_ampl	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		oow_f1	<= # dly 32'h0600_0000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_oow_f1) begin
		oow_f1	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		oow_f2	<= # dly 32'h1000_0000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_oow_f2) begin
		oow_f2	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		oow_df	<= # dly 31'h0004_0000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_oow_df) begin
		oow_df	<= # dly vme_din[30:0];
	end
end

/*	
always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		oow_old	<= # dly 1'b0;
		oow		<= # dly 1'b1;
	end else begin
		oow_old	<= # dly oow;
		if (force_oow) begin
			oow	<= # dly 1'b1;
		end else if (ampl_dem_i[34:3] < oow_a0_limit) begin
			oow	<= # dly 1'b1;
		end else if (ampl_dem_i[34:3] > oow_a1_limit) begin
			oow	<= # dly 1'b0;
		end
	end
end
*/

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		oow_old	<= # dly 1'b0;
		oow		<= # dly 1'b1;
	end else begin
		oow_old	<= # dly oow;
		if (force_oow) begin
			oow	<= # dly 1'b1;
		end else begin
			if ((pid_p_input[35:4] > oow_a0_limit) && (pid_p_input[35:4] < {-oow_a0_limit})) begin
				oow	<= # dly 1'b1;
			end else if ((pid_p_input[35:4] < oow_a1_limit) || (pid_p_input[35:4] > {-oow_a1_limit})) begin
				oow	<= # dly 1'b0;
			end		
		end
	end
end


//*********************************************
//PLL AMPLITUDE FREQUENCY AND PHASE
//*********************************************

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		ampl_pll	<= # dly {1'b0, 32'h9b74_ec00, 9'b0};
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_pll_ampl) begin
		ampl_pll	<= # dly {1'b0, vme_din, 9'h0};
	end else if (oow && ~disable_oow_ampl) begin
		ampl_pll	<= # dly {1'b0, oow_ampl, 9'h0};
	end else if (dampl_dav && ~disable_ampl_control) begin
		ampl_pll	<= # dly ampl_pll + {dampl, 10'h0};
	end else if (ampl_pll[41]==1'b1) begin
		ampl_pll	<= # dly 42'h0;
	end else if (ampl_pll>{1'b0, 32'h9b74_ec00, 9'b0}) begin
		ampl_pll	<= # dly {1'b0, 32'h9b74_ec00, 9'b0};
	end		
end
	
reg [31:0] pll_start_freq;

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		freq_pll		<= # dly 32'h0010_0000;
		pos_neg_chirp	<= # dly 1'b1;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_pll_freq) begin
		freq_pll		<= # dly vme_din;
		pll_start_freq	<= # dly vme_din;
	end else if (oow && ~disable_oow_phase) begin		
		if (start) begin
			if (freq_pll>oow_f2) begin
				pos_neg_chirp	<= # dly 1'b0;
				freq_pll		<= # dly freq_pll - {1'b0,oow_df};
			end else if (freq_pll<oow_f1) begin
				pos_neg_chirp	<= # dly 1'b1;
				freq_pll		<= # dly freq_pll + {1'b0,oow_df};
			end else begin
				freq_pll	<= # dly pos_neg_chirp ? freq_pll + {1'b0,oow_df}: freq_pll - {1'b0,oow_df};
			end
		end
	end else if (hw_cycle_start && enable_hw_cycle_start) begin
		freq_pll		<= # dly pll_start_freq;	
	end else if (df_dav) begin
		freq_pll	<= # dly freq_pll + df;
	end else if ({freq_pll + df} < oow_f1) begin
		freq_pll	<= # dly oow_f1;
	end else if  ({freq_pll + df} > oow_f2) begin
		freq_pll	<= # dly oow_f2;
	end 
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phase_pll	<= # dly 40'h0;
	end else begin
		if (phy_wmod_freq[39]) begin
			phase_pll	<= # dly phase_pll + freq_pll - wpll_mod_ampl;		
		end else begin
			phase_pll	<= # dly phase_pll + freq_pll + wpll_mod_ampl;
		end	
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phase_pll_mem_pntr	<= # dly 5'h0;
	end else if (start) begin
		phase_pll_mem_pntr	<= # dly phase_pll_mem_pntr + 1'b1;
	end
end

always @(posedge clk) begin
	if (start) begin
		phase_pll_mem[phase_pll_mem_pntr]	<= # dly phase_pll;
	end
end

assign phase_pll_mem_out	= phase_pll_mem[{phase_pll_mem_pntr - phase_pll_dly}];


//***********************************************
//STATUS/CONTROL REGISTER
//***********************************************

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		status_reg	<= # dly 16'h014f;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_status) begin
		status_reg	<= # dly vme_din[15:0];
	end
end

assign phase_pll_dly		= status_reg[13:9];
assign disable_se			= status_reg[8];
assign mcginn_mode			= status_reg[7];
assign disable_w_pll_mod	= status_reg[6];
assign enable_hw_cycle_start= status_reg[5];
assign force_oow			= status_reg[4];
assign ext_mode				= status_reg[3];	
assign disable_ampl_control	= status_reg[2];
assign disable_oow_ampl		= status_reg[1];
assign disable_oow_phase	= status_reg[0];

//**********************************************
//PLL W MODULATION
//**********************************************

always @(posedge clk or negedge clrn) begin
	if (~clrn || disable_w_pll_mod) begin
		wpll_mod_ampl	<= # dly 32'h0000_0000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_wpll_mod_ampl) begin
		wpll_mod_ampl	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		wpll_mod_freq	<= # dly 32'h0000_0000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_wpll_mod_freq) begin
		wpll_mod_freq	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		des_phy_mod	<= # dly 32'h0000_0000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_des_phy_mod) begin
		des_phy_mod	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phy_wmod_freq	<= # dly 40'h0;
	end else begin
		phy_wmod_freq	<= # dly phy_wmod_freq + wpll_mod_freq;
	end
end


//**********************************************
//SIDE EXCITER CONTROL
//**********************************************

always @(posedge clk or negedge clrn) begin
	if (~clrn || disable_se) begin
		se12_ampl	<= # dly 32'h0000_0000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_se12_ampl) begin
		se12_ampl	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		se12_dfreq	<= # dly 32'h0000_0000;
	end else if (vme_reg_space_cs && vme_wr && vme_addr[4:0]==addr_se12_dfreq) begin
		se12_dfreq	<= # dly vme_din;
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phy_se1	<= # dly 40'h0;
	end else begin
		phy_se1	<= # dly phy_se1 + freq_pll - {8'h0, se12_dfreq};
	end
end

always @(posedge clk or negedge clrn) begin
	if (~clrn) begin
		phy_se2	<= # dly 40'h0;
	end else begin
		phy_se2	<= # dly phy_se2 + freq_pll + {8'h0, se12_dfreq};
	end
end



//**********************************************
//VME READOUT
//**********************************************

always @* begin
	if (vme_reg_space_cs) begin
		vme_dav	= vme_rd;
		case (vme_addr[4:0])
			addr_status			:	vme_dout	= {14'h0, ext_mode, oow, status_reg};
			addr_ampl_dem_q		:	vme_dout	= ampl_dem_q[34:3];  
			addr_phy_dem_q		:	vme_dout	= phy_dem_q_out;
			addr_ampl_dem_i		:	vme_dout	= ampl_dem_i[34:3];  
			addr_phy_dem_i		:	vme_dout	= pid_p_input[35:4];   
			addr_des_ampl		:	vme_dout	= des_ampl;	   
			addr_phy_corr		:	vme_dout	= phy_corr;	   
			addr_oow_a0_limit	:	vme_dout	= oow_a0_limit;
			addr_oow_a1_limit	:   vme_dout	= oow_a1_limit;
			addr_oow_ampl		:   vme_dout	= oow_ampl;
			addr_oow_df		    :   vme_dout	= oow_df;
			addr_oow_f1		    :   vme_dout	= oow_f1;
		    addr_oow_f2		    :   vme_dout	= oow_f2;
		   	addr_pll_freq		:   vme_dout	= freq_pll;
            addr_pll_ampl		:   vme_dout	= ampl_pll[40:9];
            addr_dphy_df		:   vme_dout	= dphy_df;
            addr_wpll_mod_ampl	:   vme_dout	= wpll_mod_ampl;
			addr_wpll_mod_freq	:   vme_dout	= wpll_mod_freq;
			addr_des_phy_mod	:   vme_dout	= des_phy_mod;
			addr_phy_mod_depth	:   vme_dout	= phy_mod_ampl;
			addr_se12_ampl		:   vme_dout	= se12_ampl;
			addr_se12_dfreq		:   vme_dout	= se12_dfreq;
			addr_se1_ampl_dem	:   vme_dout	= se1_ampl_dem;
			addr_se1_phy_dem	:   vme_dout	= se1_phy_out;
			addr_se2_ampl_dem	:   vme_dout	= se2_ampl_dem;
			addr_se2_phy_dem	:   vme_dout	= se2_phy_out;
            default				:   vme_dout	= 32'hdead_beef;            
		endcase
	end else if (vme_pid_a_cs) begin
		vme_dav		= vme_dav_a_pid;
		vme_dout	= vme_dout_a_pid;
	end else if (vme_pid_p_cs) begin
		vme_dav		= vme_dav_a_pid;
		vme_dout	= vme_dout_a_pid;
	end else begin
		vme_dav		= 1'b0;
		vme_dout	= 32'hdead_beef;
	end
end		

endmodule