`timescale 1ns/1ns

module w_generator_34	(
	clk,
	start, n_2, iteration,
	w_r, w_i, //W_i and W_r *2^32 (2's complement)
	w_mem_wr,
	w_mem_addr,
	w_mem_din,
	w_mem_dout); 
	
input clk, start;
input [15:0] n_2;
input [3:0] iteration;
output [33:0] w_r, w_i;
input w_mem_wr;
input [15:0] w_mem_addr;
input [31:0] w_mem_din;
output [31:0] w_mem_dout;

reg [33:0] w_r, w_i;

parameter dly = 1;

supply0 gnd;
reg [17:0] phase, w_phase_increment, cos_phase;
reg gen_sin;
wire [15:0] cos_mem_addr;
reg [33:0] w_abs, w_old; 
wire [33:0] w_value;
reg [17:0] phase_old;
wire [31:0] cos_mem_out;

always @(posedge clk) begin
	if (start) begin
		case (iteration) 
			4'd0: w_phase_increment	<= # dly {2'b0, n_2};            
			4'd1: w_phase_increment	<= # dly {2'b0, n_2, 2'b0};      
			4'd2: w_phase_increment	<= # dly {2'b0, n_2[11:0], 4'b0};
			4'd3: w_phase_increment	<= # dly {2'b0, n_2[9:0], 6'b0}; 
			4'd4: w_phase_increment	<= # dly {2'b0, n_2[7:0], 8'b0}; 
			4'd5: w_phase_increment	<= # dly {2'b0, n_2[5:0], 10'b0};
			4'd6: w_phase_increment	<= # dly {2'b0, n_2[3:0], 12'b0};
			4'd7: w_phase_increment	<= # dly {2'b0, n_2[1:0], 14'b0};
			default: w_phase_increment	<= # dly 18'b0;
		endcase
	end
end

always @(posedge clk) begin
	if (start) begin
		gen_sin	<= # dly 1'b1;
	end else begin
		gen_sin	<= # dly ~gen_sin;
	end
end
		
always @(posedge clk) begin
	if (start) begin
		case (iteration) 
			4'd0: phase	<= # dly {2'b0, n_2};
			4'd1: phase	<= # dly {2'b0, n_2, 2'b0};
			4'd2: phase	<= # dly {2'b0, n_2[11:0], 4'b0};
			4'd3: phase	<= # dly {2'b0, n_2[9:0], 6'b0};
			4'd4: phase	<= # dly {2'b0, n_2[7:0], 8'b0};
			4'd5: phase	<= # dly {2'b0, n_2[5:0], 10'b0};
			4'd6: phase	<= # dly {2'b0, n_2[3:0], 12'b0};
			4'd7: phase	<= # dly {2'b0, n_2[1:0], 14'b0};
			default: phase	<= # dly 18'b0; //last iteration or error
		endcase
	end else if (gen_sin) begin
		phase		<= # dly phase + 18'h30000;
		cos_phase	<= # dly phase + w_phase_increment;
	end else begin
		phase		<= # dly cos_phase;
	end 
end

always @(posedge clk) phase_old	<= # dly phase;

assign cos_mem_addr = phase[16]	? -phase[15:0] : phase[15:0];

always @(*) begin
	if (phase_old[16:0]==17'h10000) begin
		w_abs	<= # dly 34'h0;	
	end else if (cos_mem_out==32'h1) begin	//we code 1*2^32 as 1 (anyway not used in the cos_table) on 32bit
		w_abs	<= # dly 34'h1_0000_0000;
	end else begin
		w_abs	<= # dly {2'b0, cos_mem_out};
	end
end

assign w_value	= (^phase_old[17:16]) ? -w_abs : w_abs;
	
always @(posedge clk) begin
	w_old	<= # dly w_value;
end
	 
always @(posedge clk) begin
	if (gen_sin) begin
		w_r	<= # dly w_old;
		w_i	<= # dly -w_value;
	end
end


ram_dp_32x64k cos_mem_i(
	.data_a(w_mem_din),
	.wren_a(w_mem_wr),
	.address_a(w_mem_addr),
//	.data_b(),
	.address_b(cos_mem_addr),
	.wren_b(gnd),
	.clock(clk),
	.q_a(w_mem_dout),
	.q_b(cos_mem_out));

endmodule