`timescale 1 ns/ 1 ns

//the coeficients are expressed on 32 bits in 2 complement as fixed point. They must all be smaller equal than 1 in modulus. 
//To be able to represent 1 exactly we trow away one bit and 1 is represented as 2^30. 

module window_mem_master (
	clk, 
	clrn,
	vme_module_sel,
	vme_wr,
	vme_rd,
	vme_addr,
	vme_datain,
	vme_dout_dav,
	vme_dout,
	en_window_ch1,
	acq_lenght_ch1,
	en_window_ch2,
	acq_lenght_ch2,
	fir_dec_dav_l,
	fir_dec_dout_l,
	fir_dec_dav_r,
	fir_dec_dout_r,
	dav_out_ch1,
	dout_l_ch1,
	dout_r_ch1,
	dav_out_ch2,	
	dout_l_ch2,
	dout_r_ch2,	
	sram_addr,
	sram_data,
	sram_gwe_n,
	sram_bwe_n,
	sram_oe_n,
	sram_ce1_n,
	sram_clk);
	
input	clk; 
input	clrn;

input	vme_module_sel;
input	vme_wr;	
input	vme_rd;
input	[21:2] vme_addr;
input	[31:0] vme_datain;
output	vme_dout_dav;
output	[31:0]vme_dout;
	
input	fir_dec_dav_l;
input	fir_dec_dav_r;
input	[31:0] fir_dec_dout_l;
input	[31:0] fir_dec_dout_r;
	
input	en_window_ch1;
input	[4:0] acq_lenght_ch1;

input	en_window_ch2;
input	[4:0] acq_lenght_ch2;

output	dav_out_ch1;
output [31:0] dout_l_ch1;
output [31:0] dout_r_ch1;
output	dav_out_ch2;
output [31:0] dout_l_ch2;
output [31:0] dout_r_ch2;

output	[20:0] sram_addr;
inout	[31:0]sram_data;
output	sram_gwe_n;
output	sram_bwe_n;
output	sram_oe_n;
output	sram_ce1_n;
output	sram_clk;


reg	[20:0] sram_addr;
reg sram_oe_n, sram_gwe_n;

reg vme_dout_dav;
reg [31:0] dout_l_ch1;
reg [31:0] dout_r_ch1;
reg [31:0] dout_l_ch2;
reg [31:0] dout_r_ch2;
reg dav_out_ch1, dav_out_ch2;

parameter dly = 1;

reg [31:0] sram_data_synch, sram_data_int;
reg oe_data_int;
reg [17:0]  end_address_ch1, addr_inc_ch1;
reg [17:0]  end_address_ch2, addr_inc_ch2;

wire sel_mem_space, sel_reg_space;

reg vme_mode;

parameter addr_vme_mode_on	= 19'h1;
parameter addr_vme_mode_off	= 19'h2;

reg [2:0] state;

parameter	s_idle			= 3'd0;
parameter 	s_vme_rd		= 3'd1;
parameter	s_addr_ch1		= 3'd2;
parameter	s_addr_ch2		= 3'd3;
parameter	s_prod_l_ch1	= 3'd4;
parameter	s_prod_r_ch1	= 3'd5;
parameter	s_prod_l_ch2	= 3'd6;
parameter	s_prod_r_ch2	= 3'd7;

reg [1:0] counter;
reg oe_sram_data_int;


reg [20:0] ch1_mem_pointer;
reg [20:0] ch2_mem_pointer;

reg en_window_ch1_old;
reg en_window_ch2_old;

reg [35:0] mult_data;
wire [71:0] mult_out;    		


//FIXED ASSIGNMENTS

assign sram_bwe_n	= 1'b1;
assign sram_ce1_n	= 1'b0;
assign sram_clk		= clk;

assign sel_mem_space	= vme_addr[21];
assign sel_reg_space	= ~sel_mem_space;

assign fir_dec_dav	= fir_dec_dav_l; //any of the two would be the same

assign vme_dout	= sram_data_synch;

//REGISTERS

always @(posedge clk or negedge clrn)begin
	if (~clrn) begin
		vme_mode	<= # dly 1'b1; 
	end else if (vme_module_sel && vme_wr && sel_reg_space && vme_addr[20:2]==addr_vme_mode_on) begin
		vme_mode	<= # dly 1'b1; 
	end else if (vme_module_sel && vme_wr && sel_reg_space && vme_addr[20:2]==addr_vme_mode_off) begin
		vme_mode	<= # dly 1'b0; 
	end
end

// STATE MACHINE


always@ (posedge clk) sram_data_synch <= # dly sram_data;

assign sram_data	= oe_data_int ? sram_data_int : 32'hz;

always @(posedge clk) begin
	case(acq_lenght_ch1)
		5'h00: end_address_ch1 <= # dly 18'h3_ffff;
		5'h01: end_address_ch1 <= # dly 18'h3_fffe;
		5'h02: end_address_ch1 <= # dly 18'h3_fffc;
		5'h03: end_address_ch1 <= # dly 18'h3_fff8;
		5'h04: end_address_ch1 <= # dly 18'h3_fff0;
		5'h05: end_address_ch1 <= # dly 18'h3_ffe0;
		5'h06: end_address_ch1 <= # dly 18'h3_ffc0;
		5'h07: end_address_ch1 <= # dly 18'h3_ff80;
		5'h08: end_address_ch1 <= # dly 18'h3_ff00;
		5'h09: end_address_ch1 <= # dly 18'h3_fe00;
		5'h0a: end_address_ch1 <= # dly 18'h3_fc00;
		5'h0b: end_address_ch1 <= # dly 18'h3_f800;
		5'h0c: end_address_ch1 <= # dly 18'h3_f000;
		5'h0d: end_address_ch1 <= # dly 18'h3_e000;
		5'h0e: end_address_ch1 <= # dly 18'h3_c000;
		5'h0f: end_address_ch1 <= # dly 18'h3_8000;
		default: end_address_ch1	<= # dly 18'h3_0000;
	endcase
end

always @(posedge clk) begin
	case(acq_lenght_ch1)
		5'h00: addr_inc_ch1 <= # dly  18'h0_0001;
		5'h01: addr_inc_ch1 <= # dly  18'h0_0002;
		5'h02: addr_inc_ch1 <= # dly  18'h0_0004;
		5'h03: addr_inc_ch1 <= # dly  18'h0_0008;
		5'h04: addr_inc_ch1 <= # dly  18'h0_0010;
		5'h05: addr_inc_ch1 <= # dly  18'h0_0020;
		5'h06: addr_inc_ch1 <= # dly  18'h0_0040;
		5'h07: addr_inc_ch1 <= # dly  18'h0_0080;
		5'h08: addr_inc_ch1 <= # dly  18'h0_0100;
		5'h09: addr_inc_ch1 <= # dly  18'h0_0200;
		5'h0a: addr_inc_ch1 <= # dly  18'h0_0400;
		5'h0b: addr_inc_ch1 <= # dly  18'h0_0800;
		5'h0c: addr_inc_ch1 <= # dly  18'h0_1000;
		5'h0d: addr_inc_ch1 <= # dly  18'h0_2000;
		5'h0e: addr_inc_ch1 <= # dly  18'h0_4000;
		5'h0f: addr_inc_ch1 <= # dly  18'h0_8000;
		default: addr_inc_ch1	<= # dly  18'h1_0000;
	endcase
end

always @(posedge clk) begin
	case(acq_lenght_ch2)
		5'h00: end_address_ch2 <= # dly 18'h3_ffff;
		5'h01: end_address_ch2 <= # dly 18'h3_fffe;
		5'h02: end_address_ch2 <= # dly 18'h3_fffc;
		5'h03: end_address_ch2 <= # dly 18'h3_fff8;
		5'h04: end_address_ch2 <= # dly 18'h3_fff0;
		5'h05: end_address_ch2 <= # dly 18'h3_ffe0;
		5'h06: end_address_ch2 <= # dly 18'h3_ffc0;
		5'h07: end_address_ch2 <= # dly 18'h3_ff80;
		5'h08: end_address_ch2 <= # dly 18'h3_ff00;
		5'h09: end_address_ch2 <= # dly 18'h3_fe00;
		5'h0a: end_address_ch2 <= # dly 18'h3_fc00;
		5'h0b: end_address_ch2 <= # dly 18'h3_f800;
		5'h0c: end_address_ch2 <= # dly 18'h3_f000;
		5'h0d: end_address_ch2 <= # dly 18'h3_e000;
		5'h0e: end_address_ch2 <= # dly 18'h3_c000;
		5'h0f: end_address_ch2 <= # dly 18'h3_8000;
		default: end_address_ch2	<= # dly 18'h3_0000;
	endcase
end

always @(posedge clk) begin
	case(acq_lenght_ch2)
		5'h00: addr_inc_ch2 <= # dly  18'h0_0001;
		5'h01: addr_inc_ch2 <= # dly  18'h0_0002;
		5'h02: addr_inc_ch2 <= # dly  18'h0_0004;
		5'h03: addr_inc_ch2 <= # dly  18'h0_0008;
		5'h04: addr_inc_ch2 <= # dly  18'h0_0010;
		5'h05: addr_inc_ch2 <= # dly  18'h0_0020;
		5'h06: addr_inc_ch2 <= # dly  18'h0_0040;
		5'h07: addr_inc_ch2 <= # dly  18'h0_0080;
		5'h08: addr_inc_ch2 <= # dly  18'h0_0100;
		5'h09: addr_inc_ch2 <= # dly  18'h0_0200;
		5'h0a: addr_inc_ch2 <= # dly  18'h0_0400;
		5'h0b: addr_inc_ch2 <= # dly  18'h0_0800;
		5'h0c: addr_inc_ch2 <= # dly  18'h0_1000;
		5'h0d: addr_inc_ch2 <= # dly  18'h0_2000;
		5'h0e: addr_inc_ch2 <= # dly  18'h0_4000;
		5'h0f: addr_inc_ch2 <= # dly  18'h0_8000;
		default: addr_inc_ch2	<= # dly  18'h1_0000;
	endcase
end


always @(posedge clk or negedge clrn)begin
	if (~clrn) begin
		state			<= # dly s_idle;
		sram_data_int	<= # dly 32'h0;
		oe_data_int		<= # dly 1'b0;
		sram_addr		<= # dly 21'b0;
		sram_oe_n		<= # dly 1'b1;
		sram_gwe_n		<= # dly 1'b1;
		vme_dout_dav	<= # dly 1'b0;	
		counter			<= # dly 2'h0;
		ch1_mem_pointer	<= # dly 18'h0;
		ch2_mem_pointer	<= # dly 18'h0;
		dav_out_ch1		<= # dly 1'b0;
		dav_out_ch2		<= # dly 1'b0;
		dout_l_ch1		<= # dly 32'h0;
		dout_r_ch1		<= # dly 32'h0;
		dout_l_ch2		<= # dly 32'h0;
        dout_r_ch2		<= # dly 32'h0;
        en_window_ch1_old	<= # dly 1'b0;
        en_window_ch2_old	<= # dly 1'b0;        
	end else begin
		case (state)
		s_idle: begin
			ch1_mem_pointer	<= # dly en_window_ch1	?  ch1_mem_pointer : 18'h0;
			ch2_mem_pointer	<= # dly en_window_ch2	?  ch2_mem_pointer : 18'h0;
			if (vme_mode) begin
				dav_out_ch1		<= # dly fir_dec_dav;
				dav_out_ch2		<= # dly fir_dec_dav;
				dout_l_ch1		<= # dly fir_dec_dout_l;
				dout_r_ch1		<= # dly fir_dec_dout_r;
				dout_l_ch2		<= # dly fir_dec_dout_l;
        		dout_r_ch2		<= # dly fir_dec_dout_r;				
				if (vme_module_sel && sel_mem_space && vme_wr) begin
					sram_addr		<= # dly {1'b0, vme_addr[20:2]};
					sram_data_int	<= # dly vme_datain;
					oe_data_int		<= # dly 1'b1;
					sram_oe_n		<= # dly 1'b1;
					sram_gwe_n		<= # dly 1'b0;
					counter			<= # dly 2'h0;						
				end else if (vme_module_sel && sel_mem_space && vme_rd) begin
					state			<= # dly s_vme_rd;
					oe_data_int		<= # dly 1'b0;
					sram_addr		<= # dly {1'b0, vme_addr[20:2]};
					sram_oe_n		<= # dly 1'b1;
					sram_gwe_n		<= # dly 1'b1;
					counter			<= # dly 2'h0;						
				end else begin		
					state			<= # dly s_idle;
					sram_data_int	<= # dly 32'h0;
					oe_data_int		<= # dly 1'b0;
					sram_addr		<= # dly 21'b0;
					sram_oe_n		<= # dly 1'b1;
					sram_gwe_n		<= # dly 1'b1;	
					vme_dout_dav	<= # dly 1'b0;
					counter			<= # dly 2'h0;
				end	
			end else begin
				if (fir_dec_dav) begin
					state			<= # dly s_addr_ch1;
					oe_data_int		<= # dly 1'b0;
					sram_addr		<= # dly {3'h0, ch1_mem_pointer};
					sram_oe_n		<= # dly 1'b1;
					sram_gwe_n		<= # dly 1'b1;
					vme_dout_dav	<= # dly 1'b0;	
					counter			<= # dly 2'h0;
					dav_out_ch1		<= # dly 1'b0;
					dav_out_ch2		<= # dly 1'b0;
					dout_l_ch1		<= # dly dout_l_ch1;
					dout_r_ch1		<= # dly dout_r_ch1;
					dout_l_ch2		<= # dly dout_l_ch2;
        			dout_r_ch2		<= # dly dout_r_ch2;
        		end else begin		
					state			<= # dly s_idle;
					sram_data_int	<= # dly 32'h0;
					oe_data_int		<= # dly 1'b0;
					sram_addr		<= # dly 21'b0;
					sram_oe_n		<= # dly 1'b1;
					sram_gwe_n		<= # dly 1'b1;	
					vme_dout_dav	<= # dly 1'b0;
					counter			<= # dly 2'h0;
					dav_out_ch1		<= # dly 1'b0;
					dav_out_ch2		<= # dly 1'b0;				
				end	
			end			
		end
		s_vme_rd: begin
			ch1_mem_pointer	<= # dly en_window_ch1	?  ch1_mem_pointer : 18'h0;
			ch2_mem_pointer	<= # dly en_window_ch2	?  ch2_mem_pointer : 18'h0;
			dav_out_ch1		<= # dly fir_dec_dav;
			dav_out_ch2		<= # dly fir_dec_dav;
			dout_l_ch1		<= # dly fir_dec_dout_l;
			dout_r_ch1		<= # dly fir_dec_dout_r;
			dout_l_ch2		<= # dly fir_dec_dout_l;
       		dout_r_ch2		<= # dly fir_dec_dout_r;				
			if (counter==2'h2) begin
				state	<= # dly s_idle;
				sram_oe_n		<= # dly 1'b1;
				vme_dout_dav<= # dly 1'b1;
				counter			<= # dly 2'h0;
			end else begin
				sram_oe_n		<= # dly 1'b0;
				counter			<= # dly counter + 1'b1;
			end
		end	
		s_addr_ch1: begin
			sram_oe_n		<= # dly 1'b0;
			if (counter==2'h1) begin
				state		<= # dly s_addr_ch2;
				counter		<= # dly 2'h0;
				sram_addr	<= # dly {3'h0, ch2_mem_pointer};
			end else begin
				counter		<= # dly counter + 1'b1;
			end			
       	end		
		s_addr_ch2: begin
			if (counter==2'h1) begin
				state		<= # dly s_prod_l_ch1;
				counter		<= # dly 2'h0;
			end else begin
				counter		<= # dly counter + 1'b1;
			end			
       	end	
		s_prod_l_ch1: begin
			state				<= # dly s_prod_r_ch1;
			dout_l_ch1			<= # dly (en_window_ch1)	? mult_out[61:30] : fir_dec_dout_l;
			en_window_ch1_old	<= # dly en_window_ch1;
       	end			
		s_prod_r_ch1: begin
			state			<= # dly s_prod_l_ch2;
			dout_r_ch1		<= # dly (en_window_ch1_old)	? mult_out[61:30] : fir_dec_dout_r;
       	end	
		s_prod_l_ch2: begin
			state				<= # dly s_prod_r_ch2;
			sram_oe_n			<= # dly 1'b1;
			dout_l_ch2			<= # dly (en_window_ch2)	? mult_out[61:30] : fir_dec_dout_l;
			en_window_ch2_old	<= # dly en_window_ch2;
       	end	
       	s_prod_r_ch2: begin
			state			<= # dly s_idle;
			dav_out_ch1		<= # dly en_window_ch1_old==en_window_ch1; //to avoid to send the first sample expected windowed unwindowed
			dav_out_ch2		<= # dly en_window_ch2_old==en_window_ch2; //to avoid to send the first sample expected windowed unwindowed
			dout_r_ch2		<= # dly (en_window_ch2_old)	? mult_out[61:30] : fir_dec_dout_r;
			ch1_mem_pointer	<= # dly ((ch1_mem_pointer==end_address_ch1)||(~en_window_ch1_old))	? 18'h0 : ch1_mem_pointer + addr_inc_ch1;
			ch2_mem_pointer	<= # dly ((ch2_mem_pointer==end_address_ch2)||(~en_window_ch2_old))	? 18'h0 : ch2_mem_pointer + addr_inc_ch2;
       	end		
		default: begin
			state			<= # dly s_idle;
			sram_data_int	<= # dly 32'h0;
			oe_data_int		<= # dly 1'b0;
			sram_addr		<= # dly 21'b0;
			sram_oe_n		<= # dly 1'b1;
			sram_gwe_n		<= # dly 1'b1;
			vme_dout_dav	<= # dly 1'b0;	
			counter			<= # dly 2'h0;
			ch1_mem_pointer	<= # dly 18'h0;
			ch2_mem_pointer	<= # dly 18'h0;
			dav_out_ch1		<= # dly 1'b0;
			dav_out_ch2		<= # dly 1'b0;
			dout_l_ch1		<= # dly 32'h0;
			dout_r_ch1		<= # dly 32'h0;
			dout_l_ch2		<= # dly 32'h0;
	        dout_r_ch2		<= # dly 32'h0;
    	end
    	endcase
    end
end	



//MULTIPLIER


always @(*) begin
	case (state) 
		s_prod_l_ch1:	mult_data[31:0]	=  fir_dec_dout_r;
		s_prod_r_ch1:	mult_data[31:0]	=  fir_dec_dout_l;
		s_prod_l_ch2:	mult_data[31:0]	=  fir_dec_dout_r;
		default:		mult_data[31:0]	=  fir_dec_dout_l;
	endcase
	mult_data[35:32]={4{mult_data[31]}};
end
	
sign_mult_36x36 multiplier(
	.clock0(clk),
	.dataa_0(mult_data),
	.datab_0({4'h0, sram_data_synch}),
	.result(mult_out));
	
endmodule
