`timescale 1ns/100ps

module vme_master (
	vme_ga,   	
    vme_gap,        
    vme_asn,        
    vme_ds1n,       
    vme_ds2n,       
    vme_lwordn,     
    vme_wrn,        
    vme_am,   
    vme_addr,
    vme_data,
    vme_dtackn,     
    vme_irqn, 
    vme_iackn,     
    vme_sysclk,     
    vme_sysfailn,   
    vme_berrn,      
    vme_sysresetn,  
    vme_retryn,
    fp_40mhz,
    fp_tclk);     
     	
output	[4:0] vme_ga;
output	vme_gap;
output	vme_asn;
output	vme_ds1n;
output	vme_ds2n;
output	vme_lwordn;
output	vme_wrn;
output	[5:0] vme_am;
output	[31:1] vme_addr;
inout	[31:0] vme_data;
input	vme_dtackn;
input	[7:1] vme_irqn;
output	vme_iackn;
output	vme_sysclk;
inout	vme_sysfailn;
input	vme_berrn;
output	vme_sysresetn;
input	vme_retryn;
output  fp_40mhz;
output  fp_tclk;     


reg	[4:0] vme_ga;
reg	vme_gap;
reg	vme_asn;
reg	vme_ds1n;
reg	vme_ds2n;
reg	vme_lwordn;
reg	vme_wrn;
reg	[5:0] vme_am;
reg	[31:1] vme_addr;
reg	[31:0] vme_data_int;
reg	vme_iackn;
reg	vme_sysclk;
reg	vme_sysresetn;
reg  fp_40mhz;
reg  fp_tclk;     

supply1 vcc;
supply0 gnd;
     
parameter am_a32	= 6'h09;
parameter am_cr_csr	= 6'h2f;

reg [31:0] data;

parameter board_ga = 5'h0a;

//***************************************
// ADDRESSES
//***************************************

parameter board_base_addr	= {3'b0, ~board_ga, 24'h0};

parameter int_manager_base_address		= board_base_addr + 32'h0000_0000;	
parameter debug_reg_base_address		= board_base_addr + 32'h003B_EE80;	
parameter vwbtn_base_address			= board_base_addr + 32'h003B_EEC0;     
parameter trigger_manager_base_address	= board_base_addr + 32'h003B_EF00;
parameter cordic_win_1_base_address		= board_base_addr + 32'h003B_EF40;
parameter cordic_win_2_base_address		= board_base_addr + 32'h003B_EF80; 
parameter cordic_win_3_base_address		= board_base_addr + 32'h003B_EFC0;
parameter codec_base_address			= board_base_addr + 32'h003B_F800;        
parameter fir_dec_base_address			= board_base_addr + 32'h003B_FC00;     
parameter w_mem_base_address			= board_base_addr + 32'h003C_0000;      
parameter vert_sram_master_base_address	= board_base_addr + 32'h0040_0000;
parameter hor_sram_master_base_address	= board_base_addr + 32'h0080_0000;
parameter pm_sram_master_base_address	= board_base_addr + 32'h00C0_0000;

// CODEC CONTROL INTERNAL ADDRESSES

parameter addr_codec_status_reg		= codec_base_address + 32'h0000_0200;
parameter addr_codec_command_reg	= codec_base_address + 32'h0000_0204;
parameter addr_codec_mclk_div		= codec_base_address + 32'h0000_0208;
parameter addr_codec_f0_reg_r		= codec_base_address + 32'h0000_020C;
parameter addr_codec_f1_reg_r		= codec_base_address + 32'h0000_0210;
parameter addr_codec_f2_reg_r		= codec_base_address + 32'h0000_0214;
parameter addr_codec_df_reg_r		= codec_base_address + 32'h0000_0218;
parameter addr_codec_f0_reg_l		= codec_base_address + 32'h0000_021C;
parameter addr_codec_f1_reg_l		= codec_base_address + 32'h0000_0220;
parameter addr_codec_f2_reg_l		= codec_base_address + 32'h0000_0224;
parameter addr_codec_df_reg_l		= codec_base_address + 32'h0000_022C;
parameter addr_codec_spi_reg		= codec_base_address + 32'h0000_0240;
parameter addr_codec_amplitude_r	= codec_base_address + 32'h0000_0280;
parameter addr_codec_amplitude_l	= codec_base_address + 32'h0000_0284;
parameter addr_codec_adc_r_value	= codec_base_address + 32'h0000_0288;
parameter addr_codec_adc_l_value	= codec_base_address + 32'h0000_028C;


//****************************************
//VME bus start-up conditions and reset
//****************************************

assign vme_data	= vme_data_int;

initial begin
	vme_ga			= board_ga;
	vme_gap			= ~^vme_ga;
	vme_asn			= 1'b1;
	vme_ds1n		= 1'b1;
	vme_ds2n		= 1'b1;
	vme_lwordn		= 1'b1;
	vme_wrn			= 1'b1;
	vme_am			= 6'h3f;
	vme_addr		= 31'h0;
	vme_data_int	= 32'hz;
	vme_iackn		= 1'b1;
	vme_sysclk		= 1'b0;
	vme_sysresetn	= 1'b0;
	#100; 
	vme_sysresetn	= 1'b1; 
end

always begin
	#  31.2;
	vme_sysclk	= 1'b1;
	# 31.3;
	vme_sysclk	= 1'b0;
end	

pullup sysfail_pullup (vme_sysfailn);
pullup vme_dtackn_pullup (vme_dtackn);
pullup vme_irqn1_pullup (vme_irqn[1]);
pullup vme_irqn2_pullup (vme_irqn[2]);
pullup vme_irqn3_pullup (vme_irqn[3]);
pullup vme_irqn4_pullup (vme_irqn[4]);
pullup vme_irqn5_pullup (vme_irqn[5]);
pullup vme_irqn6_pullup (vme_irqn[6]);
pullup vme_irqn7_pullup (vme_irqn[7]);


//****************************************
//FRONT PANNEL SIGNAL
//****************************************


always begin
	# 12.5;
	fp_40mhz	= 1'b1;
	# 12.5;
	fp_40mhz	= 1'b0;
end	

always begin
	fp_tclk	= 1'b1;
	# 12000;
	fp_tclk	= 1'b0;
	# 13000;
end	

//**********************************
//VME write cycles : a32 and cr-csr
//**********************************

task vme_write_a32;
	input [31:0] address; 
	input [31:0] data;
	output [31:0] data_read;
integer i;		
begin
	vme_addr 		= address[31:1];
	vme_lwordn		= 1'b0;
	vme_am 			= am_a32;
	vme_data_int 	= data;
	vme_wrn			= 1'b0;
	# 10;
	vme_asn	= 1'b0;
	#10;
	vme_ds1n = 1'b0;
	vme_ds2n = 1'b0;
	i=0;
	while (vme_dtackn && ~(i==1000)) #40 i=i+1;	
	if (vme_dtackn) begin
		$display("No module response to the vme write request: time out");
		$stop;
		vme_asn	= 1'b1;
		vme_ds1n = 1'b1;
		vme_ds2n = 1'b1;
		vme_data_int	= 32'hz;
	end else begin		
		# 10; 
		vme_asn	= 1'b1;
		vme_ds1n = 1'b1;
		vme_ds2n = 1'b1;
		vme_data_int	= 32'hz;
		i=0;
		while (~vme_dtackn && ~(i==1000)) #40 i=i+1;	
		if (~vme_dtackn) begin
			$display("Unable to close the vme write in a usefull time: time out");
			$stop;
		end else begin
			$display ("VME write cycle: d = %h at address %h of A32 address space at %t",data,{vme_addr, 1'b0},$time);
		end
	end
end
endtask

task vme_write_cr_csr;
	input [31:0] address; 
	input [31:0] data;
	output [31:0] data_read;
begin
	vme_addr[18:1] 	= address[18:1];
	vme_addr[23:19]	= ~vme_ga;
	vme_lwordn	= 1'b1;
	vme_am 		= am_cr_csr;
	vme_data_int 	= data;
	vme_wrn		= 1'b0;
	# 10;
	vme_asn	= 1'b0;
	#10;
	vme_ds1n = ~address[0];
	vme_ds2n = address[0];
	wait (vme_dtackn==1'b0);
	# 10; 
	vme_asn = 1'b1;
	vme_ds1n = 1'b1;
	vme_ds2n = 1'b1;
	vme_data_int	= 32'hz;
	wait (vme_dtackn==1'b1);
//	$display ("VME write cycle: d = %h at address %h of CR-CSR address space at %t",data,{vme_addr, 1'b0},$time);
end
endtask

//**********************************
//VME read cycles : a32 and cr_csr
//**********************************

task vme_read_a32;
	input [31:0] address; 
	input [31:0] data;
	output [31:0] data_read;
integer i;	
begin
	vme_addr 	= address[31:1];
	vme_lwordn	= 1'b0;
	vme_am 		= am_a32;
	vme_data_int 	= 32'bz;
	vme_wrn = 1'b1;
	# 10;
	vme_asn	= 1'b0;
	#10;
	vme_ds1n = 1'b0;
	vme_ds2n = 1'b0;
	
	i=0;
	while (vme_dtackn && ~(i==1000)) #40 i=i+1;	
	if (vme_dtackn) begin
		$display("No module response to the vme read request @ 32'h%h : time out", address);
		$stop;
		vme_asn	= 1'b1;
		vme_ds1n = 1'b1;
		vme_ds2n = 1'b1;
		vme_data_int	= 32'hz;
	end else begin		
		data_read = vme_data;
		# 10; 
		vme_asn	= 1'b1;
		vme_ds1n = 1'b1;
		vme_ds2n = 1'b1;
		i=0;
		while (~vme_dtackn && ~(i==1000)) #40 i=i+1;	
		if (~vme_dtackn) begin
			$display("Unable to close the vme write in a usefull time: time out");
			$stop;
		end else begin
			$display ("VME read cycle: d = %h at address %h of A32 address space at %t",data_read ,{vme_addr, 1'b0},$time);
		end
	end
end
endtask

task vme_read_cr_csr;
	input [31:0] address; 
	input [31:0] data;
	output [31:0] data_read;
begin
	vme_addr[18:1] 	= address[18:1];
	vme_addr[23:19]	= ~vme_ga;
	vme_lwordn	= 1'b1;
	vme_am 		= am_cr_csr;
	vme_data_int 	= 32'bz;
	vme_wrn = 1'b1;
	# 10;
	vme_asn	= 1'b0;
	#10;
	vme_ds1n = address[0];
	vme_ds2n = ~address[0];
//	vme_ds1n = 1'b0;
//	vme_ds2n = 1'b0;
	wait (vme_dtackn==1'b0);
	data_read = vme_data;
	# 10;
	vme_asn		= 1'b1; 
	vme_ds1n	= 1'b1;
	vme_ds2n	= 1'b1;
	wait (vme_dtackn==1'b1);
	$display ("VME read cycle: d = %h at address %h of CR-CSR address space at %t",data_read, {vme_addr, 1'b0},$time);
end
endtask

//***************************************
// ADDRESSES
//***************************************
task fill_freq_synth_mem;
	input [31:0] start_addr;
		
	reg [31:0] i;
	reg [31:0] addr;
	reg [17:0] sin256 [255:0];
begin
	$readmemh("sin256.txt", sin256);
	addr	= start_addr;
	i		= 0;
	repeat(256) begin
		vme_write_a32(addr, {14'h0, sin256[i]}, data);
		i		= i + 1;
		addr	= addr+4;
	end
end
endtask	 

//***************************************
//codec2fft_test
//***************************************
task codec2FFT_test;
reg [17:0] i;
reg [31:0] real_part;
integer text_file;
begin
	//Programming the right freq. synthesizer
	vme_write_a32(addr_codec_f0_reg_r, 32'd059500000, data);
	vme_write_a32(addr_codec_f2_reg_r, 32'd536500000, data);
	vme_write_a32(addr_codec_f1_reg_r, 32'd059500000, data);
	vme_write_a32(addr_codec_df_reg_r, {16'h0000, 16'd596}, data);
	vme_write_a32(addr_codec_command_reg, 32'd04000000, data); 
		
	//Programming the left freq. synthesizer 
	vme_write_a32(addr_codec_f0_reg_l, 32'd005950000, data); 
	vme_write_a32(addr_codec_f2_reg_l, 32'd1000000000, data); 
	vme_write_a32(addr_codec_f1_reg_l, 32'd025000000, data);
	vme_write_a32(addr_codec_df_reg_l, {16'h0000, 16'd2500}, data); 
	vme_write_a32(addr_codec_command_reg, 32'd06000000, data); 
	
	//setting the FFT lenght to 4 (1024 points), the mode to both and the trigger type to single and enabling windowing
	vme_write_a32({vert_sram_master_base_address + 32'h8}, {8'h08, 24'h4}, data);	
	vme_write_a32({vert_sram_master_base_address + 32'h8}, {8'h0c, 24'h0}, data);	
	vme_write_a32({vert_sram_master_base_address + 32'h8}, {8'h03, 24'h0}, data);
	vme_write_a32({vert_sram_master_base_address + 32'h8}, {8'h0a, 24'h0}, data);
	vme_write_a32({hor_sram_master_base_address + 32'h8}, {8'h08, 24'h4}, data);	
	vme_write_a32({hor_sram_master_base_address + 32'h8}, {8'h0c, 24'h0}, data);	
	vme_write_a32({hor_sram_master_base_address + 32'h8}, {8'h03, 24'h0}, data);
	vme_write_a32({hor_sram_master_base_address + 32'h8}, {8'h0a, 24'h0}, data);
	#40_000;
	
	//reading the trigger number and the time_stamp
	vme_read_a32({vert_sram_master_base_address + 32'hC}, 32'hz, data);
	$display("The trigger number at the beginning in vsram was %d", data);
	vme_read_a32({vert_sram_master_base_address + 32'h10}, 32'hz, data);	
	$display("The time stamp at the beginning in vsram was %d", data);
	vme_read_a32({hor_sram_master_base_address + 32'hC}, 32'hz, data);
	$display("The trigger number at the beginning in hsram was %d", data);
	vme_read_a32({hor_sram_master_base_address + 32'h10}, 32'hz, data);	
	$display("The time stamp at the beginning in hsram was %d", data);
	
	//arming the trigger
	vme_write_a32({vert_sram_master_base_address + 32'h8}, {8'h01, 24'h0}, data);
	vme_write_a32({hor_sram_master_base_address + 32'h8}, {8'h01, 24'h0}, data);
	
	//sending 2 triggers
	vme_write_a32({trigger_manager_base_address + 32'h4}, 32'h1200_0000, data);		
	#5000_000;	
	vme_write_a32({trigger_manager_base_address + 32'h4}, 32'h1200_0000, data);		
		
	//reading the status register checking when the acquisition is done and than transferring the 1024 data points
	text_file = $fopen("raw_out_v.txt");
	vme_read_a32(vert_sram_master_base_address, 32'hz, data);		
	while (data[17:0]<1023) #3000 vme_read_a32(vert_sram_master_base_address, 32'hz, data);
	for (i=0; i<1024; i=i+1) begin
		vme_read_a32({vert_sram_master_base_address + {10'h0, 1'h0, 1'b1, 1'b0, i, 2'h0}}, 32'hz, data);	
		real_part	= data;
		vme_read_a32({vert_sram_master_base_address + {10'h0, 1'h0, 1'b1, 1'b1, i, 2'h0}}, 32'hz, data);
		$fdisplay(text_file, "%h	%h", real_part, data);	
	end	
	$fclose(text_file);
	
	//reading the status register checking when the FFT data is ready and then transferring the data
	#1_000;
	vme_read_a32(vert_sram_master_base_address, 32'hz, data);		
	while (~data[23]) #50_000 vme_read_a32(vert_sram_master_base_address, 32'hz, data);
	text_file = $fopen("fft_out_v.txt");
	for (i=0; i<1024; i=i+1) begin
		vme_read_a32({vert_sram_master_base_address + {10'h0, 1'h0, 1'b1, 1'b0, i, 2'h0}}, 32'hz, data);	
		real_part	= data;
		vme_read_a32({vert_sram_master_base_address + {10'h0, 1'h0, 1'b1, 1'b1, i, 2'h0}}, 32'hz, data);
		$fdisplay(text_file, "%h	%h", real_part, data);	
	end
	
	//reading the status register checking when the acquisition is done and than transferring the 1024 data points
	text_file = $fopen("raw_out_h.txt");
	vme_read_a32(hor_sram_master_base_address, 32'hz, data);		
	while (data[17:0]<1023) #3000 vme_read_a32(hor_sram_master_base_address, 32'hz, data);
	for (i=0; i<1024; i=i+1) begin
		vme_read_a32({hor_sram_master_base_address + {10'h0, 1'h0, 1'b1, 1'b0, i, 2'h0}}, 32'hz, data);	
		real_part	= data;
		vme_read_a32({hor_sram_master_base_address + {10'h0, 1'h0, 1'b1, 1'b1, i, 2'h0}}, 32'hz, data);
		$fdisplay(text_file, "%h	%h", real_part, data);	
	end	
	$fclose(text_file);
	
	//reading the status register checking when the FFT data is ready and then transferring the data
	#1_000;
	vme_read_a32(hor_sram_master_base_address, 32'hz, data);		
	while (~data[23]) #50_000 vme_read_a32(hor_sram_master_base_address, 32'hz, data);
	text_file = $fopen("fft_out_h.txt");
	for (i=0; i<1024; i=i+1) begin
		vme_read_a32({hor_sram_master_base_address + {10'h0, 1'h0, 1'b1, 1'b0, i, 2'h0}}, 32'hz, data);	
		real_part	= data;
		vme_read_a32({hor_sram_master_base_address + {10'h0, 1'h0, 1'b1, 1'b1, i, 2'h0}}, 32'hz, data);
		$fdisplay(text_file, "%h	%h", real_part, data);	
	end
	
	//reading the trigger number and the time_stamp
	vme_read_a32({vert_sram_master_base_address + 32'hC}, 32'hz, data);
	$display("The trigger number at the end in vsram was %d", data);
	vme_read_a32({vert_sram_master_base_address + 32'h10}, 32'hz, data);	
	$display("The time stamp at the end in vsram was %d", data);
	vme_read_a32({hor_sram_master_base_address + 32'hC}, 32'hz, data);
	$display("The trigger number at the end in hsram was %d", data);
	vme_read_a32({hor_sram_master_base_address + 32'h10}, 32'hz, data);	
	$display("The time stamp at the end in hsram was %d", data);
	
end
endtask

//***************************************
// CONFIGURE INTERRUPT
//***************************************

task configure_interrupt;
	input [2:0] int_level;
	input [7:0] int_vector;
	input [7:0] int_mask; 
begin
	vme_write_cr_csr(32'h7_f002, {16'h0, 1'b1, 4'h0, int_level, 8'h0}, data); 
	$display("Interrupt enabled and level set to %d", int_level);	
	vme_write_cr_csr(32'h7_f003, {16'h0, int_vector, 8'h0}, data); 
	$display("Interrupt vector set to %h", int_vector);		
	vme_write_a32({int_manager_base_address+32'h4}, {24'h0, int_mask}, data);	
	$display("Interrupt mask set to %d", int_mask);
end
endtask

//***************************************
// WAIT FOR INTERRUPT
//***************************************

task wait_for_interrupt;
output [31:0] interrupt_vector;

integer i;
begin
	vme_data_int 	= 32'bz;
	i	= 0;
	while (&vme_irqn && ~(i==100)) #1000 i=i+1;	
	if (&vme_irqn) begin
		$display("No interrupt received: time out");
		interrupt_vector	= 32'hx;
	end else begin
		vme_iackn 		= 1'b0;    
	    vme_addr[3:1]	= ~vme_irqn[7] ? 3'h7 : (~vme_irqn[6] ? 3'h6 : (~vme_irqn[5] ? 3'h5 : (~vme_irqn[4] ? 3'h4 : (~vme_irqn[3] ? 3'h3 : (~vme_irqn[2] ? 3'h2 : ~vme_irqn[1] ? 3'h1 : 3'hx)))));
	    #10 vme_asn = 1'b0;        
//	    $display("Received interrupt reqest of priority level %d", vme_addr[3:1]);
	    vme_ds1n	= 1'b0;      
	    vme_wrn		= 1'b1;
	    i = 0;
		while (vme_dtackn && ~(i==1000)) #1 i=i+1;
		if (vme_dtackn) $display("No module responded to the interrupt acknoledge in a usefull time");
		else begin
			interrupt_vector	=  vme_data;
//			$display("Received interrupt vector: %h", 	interrupt_vector);
	    end
	    #10;
	    vme_asn		= 1'b1;
	    vme_ds1n	= 1'b1;  
		vme_iackn 	= 1'b1;   
		i=0;
		while (~vme_dtackn && ~(i==10)) #1000 i=i+1;	
		if (~vme_dtackn) begin
			$display("Unable to close the vme interrupt acknoledge in a usefull time: time out");
			$stop;
		end
	end
end 
endtask  
	 
//***************************************
// RW COMMAND FROM FILE
//***************************************

task rw_from_file_command;
reg [31:0] wr_addr_data_wait [3:0];

begin
	$readmemh("wr_addr_data_wait.txt", wr_addr_data_wait);
	if (~|wr_addr_data_wait[0]) begin
		vme_read_a32({wr_addr_data_wait[1]+board_base_addr}, 32'hz, data);				
		$display("Read %h @ address %h", data, {wr_addr_data_wait[1]+board_base_addr});
	end else begin
		vme_write_a32({wr_addr_data_wait[1]+board_base_addr}, wr_addr_data_wait[2], data);				
		$display("Wrote %h @ address %h",wr_addr_data_wait[2] , {wr_addr_data_wait[1]+board_base_addr});
	end
	#(wr_addr_data_wait[3]);
end	
endtask

//***************************************
// SIMULATION CONTROL FLOW
//***************************************

initial begin
#1000;
vme_read_a32({int_manager_base_address+32'h4}, 32'hz, data);	
vme_write_a32({int_manager_base_address+32'h4}, 32'h0, data);	
vme_read_a32({int_manager_base_address+32'h4}, 32'hz, data);

#1000000;	
#1000;
$stop;
forever begin
	rw_from_file_command;
	$stop;
end

end

endmodule
