`timescale 1ns/100ps

module vme_master (
	vme_ga,   	
    vme_gap,        
    vme_asn,        
    vme_ds1n,       
    vme_ds2n,       
    vme_lwordn,     
    vme_wrn,        
    vme_am,   
    vme_addr,
    vme_data,
    vme_dtackn,     
    vme_irqn, 
    vme_iackn,      
    vme_sysclk,     
    vme_sysfailn,   
    vme_berrn,      
    vme_sysresetn,  
    vme_retryn);     
     	
output	[4:0] vme_ga;
output	vme_gap;
output	vme_asn;
output	vme_ds1n;
output	vme_ds2n;
output	vme_lwordn;
output	vme_wrn;
output	[5:0] vme_am;
output	[31:1] vme_addr;
inout	[31:0] vme_data;
input	vme_dtackn;
input	[7:1] vme_irqn;
output	vme_iackn;
output	vme_sysclk;
inout	vme_sysfailn;
input	vme_berrn;
output	vme_sysresetn;
input	vme_retryn;

reg	[4:0] vme_ga;
reg	vme_gap;
reg	vme_asn;
reg	vme_ds1n;
reg	vme_ds2n;
reg	vme_lwordn;
reg	vme_wrn;
reg	[5:0] vme_am;
reg	[31:1] vme_addr;
reg	[31:0] vme_data_int;
reg	vme_iackn;
reg	vme_sysclk;
reg	vme_sysresetn;

supply1 vcc;
supply0 gnd;
     
parameter am_a32	= 6'h09;
parameter am_cr_csr	= 6'h2f;

reg [31:0] data;

//***************************************
// ADDRESSES
//***************************************

parameter address_vwbtn_control	= 4'd0; 
parameter address_codec_control	= 4'd1;
parameter address_vsram_master	= 4'd2;
parameter address_w_memory		= 4'd5;
parameter address_trigger_man	= 4'd6;
parameter address_fir_dec		= 4'd7;

// CODEC CONTROL INTERNAL ADDRESSES
parameter addr_freq_synth_left	= 2'd0;
parameter addr_freq_synth_right	= 2'd1;
parameter addr_conf_reg			= 2'd2;	
parameter addr_spi_interface	= 2'd3;

parameter addr_r_ph_adv			= 16'h1;
parameter addr_l_ph_adv			= 16'h2;
parameter addr_mclk_div			= 16'h3;
parameter addr_chirp_control0_r	= 16'h4;
parameter addr_chirp_control1_r	= 16'h5;
parameter addr_chirp_control2_r	= 16'h6;
parameter addr_chirp_control0_l	= 16'h7;
parameter addr_chirp_control1_l	= 16'h8;
parameter addr_chirp_control2_l	= 16'h9;
parameter addr_codec_rst_off	= 16'ha;
parameter addr_codec_rst_on		= 16'hb;
parameter addr_start_chirp_r	= 16'hc;
parameter addr_start_chirp_l	= 16'hd;
parameter addr_start_chirp_lr	= 16'he;



//****************************************
//VME bus start-up conditions and reset
//****************************************

assign vme_data	= vme_data_int;

initial begin
	vme_ga			= 5'h0f;
	vme_gap			= ~^vme_ga;
	vme_asn			= 1'b1;
	vme_ds1n		= 1'b1;
	vme_ds2n		= 1'b1;
	vme_lwordn		= 1'b1;
	vme_wrn			= 1'b1;
	vme_am			= 6'h3f;
	vme_addr		= 31'h0;
	vme_data_int	= 32'hz;
	vme_iackn		= 1'b1;
	vme_sysclk		= 1'b0;
	vme_sysresetn	= 1'b0;
	#100; 
	vme_sysresetn	= 1'b1; 
end

always begin
	#  31.2;
	vme_sysclk	= 1'b1;
	# 31.3;
	vme_sysclk	= 1'b0;
end	

pullup sysfail_pullup (vme_sysfailn);

//**********************************
//VME write cycles : a32 and cr-csr
//**********************************

task vme_write_a32;
	input [31:0] address; 
	input [31:0] data;
	output [31:0] data_read;
begin
	vme_addr 	= {~vme_ga, address[26:1]};
	vme_lwordn	= 1'b0;
	vme_am 		= am_a32;
	vme_data_int 	= data;
	vme_wrn		= 1'b0;
	# 10;
	vme_asn	= 1'b0;
	#10;
	vme_ds1n = 1'b0;
	vme_ds2n = 1'b0;
	wait (vme_dtackn==1'b0);
	# 10; 
	vme_asn	= 1'b1;
	vme_ds1n = 1'b1;
	vme_ds2n = 1'b1;
	vme_data_int	= 32'hz;
	wait (vme_dtackn==1'b1);
//	$display ("VME write cycle: d = %h at address %h of A32 address space at %t",data,{vme_addr, 1'b0},$time);
end
endtask

task vme_write_cr_csr;
	input [31:0] address; 
	input [31:0] data;
	output [31:0] data_read;
begin
	vme_addr[18:1] 	= address[18:1];
	vme_addr[23:19]	= ~vme_ga;
	vme_lwordn	= 1'b1;
	vme_am 		= am_cr_csr;
	vme_data_int 	= data;
	vme_wrn		= 1'b0;
	# 10;
	vme_asn	= 1'b0;
	#10;
	vme_ds1n = 1'b0;
	vme_ds2n = 1'b0;
	wait (vme_dtackn==1'b0);
	# 10; 
	vme_asn = 1'b1;
	vme_ds1n = 1'b1;
	vme_ds2n = 1'b1;
	vme_data_int	= 32'hz;
	wait (vme_dtackn==1'b1);
//	$display ("VME write cycle: d = %h at address %h of CR-CSR address space at %t",data,{vme_addr, 1'b0},$time);
end
endtask

//**********************************
//VME read cycles : a32 and cr_csr
//**********************************

task vme_read_a32;
	input [31:0] address; 
	input [31:0] data;
	output [31:0] data_read;
begin
	vme_addr 	= {~vme_ga, address[26:1]};
	vme_lwordn	= 1'b0;
	vme_am 		= am_a32;
	vme_data_int 	= 32'bz;
	vme_wrn = 1'b1;
	# 10;
	vme_asn	= 1'b0;
	#10;
	vme_ds1n = 1'b0;
	vme_ds2n = 1'b0;
	wait (vme_dtackn==1'b0);
	data_read = vme_data;
	# 10; 
	vme_asn	= 1'b1;
	vme_ds1n = 1'b1;
	vme_ds2n = 1'b1;
	wait (vme_dtackn==1'b1);
//	$display ("VME read cycle: d = %h at address %h of A32 address space at %t",data_read ,{vme_addr, 1'b0},$time);
end
endtask

task vme_read_cr_csr;
	input [31:0] address; 
	input [31:0] data;
	output [31:0] data_read;
begin
	vme_addr[18:1] 	= address[18:1];
	vme_addr[23:19]	= ~vme_ga;
	vme_lwordn	= 1'b1;
	vme_am 		= am_cr_csr;
	vme_data_int 	= 32'bz;
	vme_wrn = 1'b1;
	# 10;
	vme_asn	= 1'b0;
	#10;
	vme_ds1n = address[0];
	vme_ds2n = ~address[0];
	wait (vme_dtackn==1'b0);
	data_read = vme_data;
	# 10;
	vme_asn		= 1'b1; 
	vme_ds1n	= 1'b1;
	vme_ds2n	= 1'b1;
	wait (vme_dtackn==1'b1);
//	$display ("VME read cycle: d = %h at address %h of CR-CSR address space at %t",data_read, {vme_addr, 1'b0},$time);
end
endtask

//***************************************
// ADDRESSES
//***************************************
task fill_freq_synth_mem;
	input [31:0] start_addr;
		
	reg [31:0] i;
	reg [31:0] addr;
	reg [17:0] sin256 [255:0];
begin
	$readmemh("sin256.txt", sin256);
	addr	= start_addr;
	i		= 0;
	repeat(256) begin
		vme_write_a32(addr, {14'h0, sin256[i]}, data);
		i		= i + 1;
		addr	= addr+4;
	end
end
endtask	 

//***************************************
//CHIRP TEST
//***************************************
		
task test_chirp;
begin
	fill_freq_synth_mem({5'h0, address_codec_control, 3'b0, addr_freq_synth_left, 16'h0, 2'b00});	
	fill_freq_synth_mem({5'h0, address_codec_control, 3'b0, addr_freq_synth_right, 16'h0, 2'b00});	
	#10000;
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_codec_rst_off, 2'b00}, 32'h0, data);	
	#10000;	
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_spi_interface, 16'h0, 2'b00}, 32'h275, data);	
	#10000;
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_r_ph_adv, 2'b00}, 32'h3fffffff, data);	
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_l_ph_adv, 2'b00}, 32'h7fffffff, data);	
	#200000;	
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_chirp_control2_r, 2'b00}, 32'd536500000, data);
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_chirp_control1_r, 2'b00}, 32'd59500000, data);
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_chirp_control0_r, 2'b00}, {16'h0000, 16'd596}, data);
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_chirp_control2_l, 2'b00}, 32'h00ffffff, data);
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_chirp_control1_l, 2'b00}, 32'h00000000, data);
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_chirp_control0_l, 2'b00}, 32'h00000fff, data);
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_start_chirp_r, 2'b00}, 32'h0, data);
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_start_chirp_l, 2'b00}, 32'h0, data);
	#23000000;	
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_start_chirp_r, 2'b00}, 32'h0, data);
	#1000000;	
end
endtask

//***************************************
//FFT TEST
//***************************************
		
task test_FFT;
reg [17:0] i;
reg [31:0] real_part;
integer text_file;
reg [31:0] samples_r [1023:0];
reg [31:0] samples_i [1023:0];
begin
	$readmemh("samples_r.txt", samples_r);
	$readmemh("samples_i.txt", samples_i);
	// Loading the 16 x(n)
	for (i=0; i<1024; i=i+1) begin
		vme_write_a32({5'h0, address_vsram_master, 1'h0, 1'b1, 1'b0, i, 2'h0}, samples_r[i], data); 	
		vme_write_a32({5'h0, address_vsram_master, 1'h0, 1'b1, 1'b1, i, 2'h0}, samples_i[i], data);	
	end
	// setting the fft lenght to 4 (1024 points) 
	vme_write_a32({5'h0, address_vsram_master, 21'h1, 2'h0}, {8'h08, 24'h4}, data);	
	//reading the status register	
	vme_read_a32({5'h0, address_vsram_master, 21'h0, 2'h0}, 32'hz, data);		
	// FFT START command 
	vme_write_a32({5'h0, address_vsram_master, 21'h1, 2'h0}, {8'h07, 24'h0}, data);	
	//reading the status register checking when the FFT data is ready
	#500;
	vme_read_a32({5'h0, address_vsram_master, 21'h0, 2'h0}, 32'hz, data);		
	while (~data[8]) #3000 vme_read_a32({5'h0, address_vsram_master, 21'h0, 2'h0}, 32'hz, data);
	//reading the FFT
	text_file = $fopen("fft_out.txt");
	for (i=0; i<1024; i=i+1) begin
		vme_read_a32({5'h0, address_vsram_master, 1'h0, 1'b1, 1'b0, i, 2'h0}, 32'hz, data);	
		real_part	= data;
		vme_read_a32({5'h0, address_vsram_master, 1'h0, 1'b1, 1'b1, i, 2'h0}, 32'hz, data);
		$fdisplay(text_file, "%h	%h", real_part, data);	
	end
	//reading the status register	
	vme_read_a32({5'h0, address_vsram_master, 21'h0, 2'h0}, 32'hz, data);		
	#1000;
	$fclose(text_file);
end
endtask


//***************************************
//codec2fft_test
//***************************************
task codec2FFT_test;
reg [17:0] i;
reg [31:0] real_part;
integer text_file;
begin
	//Programming the FIR left with a[0:4]=[-0.25 -0.5 1 -0.5 -0.25] 	
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h1, 5'd0, 1'b0}, 2'h0}, 32'he000_0000, data);//-0.25
	vme_write_a32({5'h0,address_fir_dec, 13'h0, {2'h1, 5'd0, 1'b1}, 2'h0}, {4'h0, 28'h0}, data);
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h1, 5'd1, 1'b0}, 2'h0}, 32'hc000_0000, data);//-0.5
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h1, 5'd1, 1'b1}, 2'h0}, {4'h0, 28'h0}, data);
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h1, 5'd2, 1'b0}, 2'h0}, 32'h7FFF_FFFF, data);//about 1 (1-2^-35)
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h1, 5'd2, 1'b1}, 2'h0}, {4'hF, 28'h0}, data);
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h1, 5'd3, 1'b0}, 2'h0}, 32'hc000_0000, data);//-0.5
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h1, 5'd3, 1'b1}, 2'h0}, {4'h0, 28'h0}, data);
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h1, 5'd4, 1'b0}, 2'h0}, 32'he000_0000, data);//-0.25
	vme_write_a32({5'h0,address_fir_dec, 13'h0, {2'h1, 5'd4, 1'b1}, 2'h0}, {4'h0, 28'h0}, data);
	//Programming the FIR right with a0=0.25 a1=0.5 a2=~1 a3=0.5 a4=0.25
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h2, 5'd0, 1'b0}, 2'h0}, 32'h1000_0000, data);//0.125
	vme_write_a32({5'h0,address_fir_dec, 13'h0, {2'h2, 5'd0, 1'b1}, 2'h0}, {4'h0, 28'h0}, data);
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h2, 5'd1, 1'b0}, 2'h0}, 32'h2000_0000, data);//0.25
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h2, 5'd1, 1'b1}, 2'h0}, {4'h0, 28'h0}, data);
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h2, 5'd2, 1'b0}, 2'h0}, 32'h4000_0000, data);//0.5
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h2, 5'd2, 1'b1}, 2'h0}, {4'h0, 28'h0}, data);
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h2, 5'd3, 1'b0}, 2'h0}, 32'h2000_0000, data);//0.25
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h2, 5'd3, 1'b1}, 2'h0}, {4'h0, 28'h0}, data);
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h2, 5'd4, 1'b0}, 2'h0}, 32'h1000_0000, data);//0.125
	vme_write_a32({5'h0,address_fir_dec, 13'h0, {2'h2, 5'd4, 1'b1}, 2'h0}, {4'h0, 28'h0}, data);
	//Programming the decimator to discard 1 sample each 2
	vme_write_a32({5'h0,address_fir_dec, 13'd0, {2'h0, 6'd0}, 2'h0}, {28'h0, 4'h1}, data);
	#500_000; //let the filter stabilise
	//Programming the right freq. synthesizer
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_r_ph_adv, 2'b00}, 32'd059500000, data);
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_chirp_control2_r, 2'b00}, 32'd536500000, data);
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_chirp_control1_r, 2'b00}, 32'd059500000, data);
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_chirp_control0_r, 2'b00}, {16'h4000, 16'd596}, data);
	//Programming the left freq. synthesizer 
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_l_ph_adv, 2'b00}, 32'd0, data); 
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_chirp_control2_l, 2'b00}, 32'd1000000000, data); 
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_chirp_control1_l, 2'b00}, 32'd025000000, data);
	vme_write_a32({5'h0, address_codec_control, 3'b0, addr_conf_reg, addr_chirp_control0_l, 2'b00}, {16'h4000, 16'd2500}, data); 
	//setting the FFT lenght to 4 (1024 points), the mode to raw and the trigger type to single
	vme_write_a32({5'h0, address_vsram_master, 21'h1, 2'h0}, {8'h08, 24'h4}, data);	
	vme_write_a32({5'h0, address_vsram_master, 21'h1, 2'h0}, {8'h06, 24'h0}, data);	
	vme_write_a32({5'h0, address_vsram_master, 21'h1, 2'h0}, {8'h03, 24'h0}, data);
	#40_000;
	//reading the trigger number and the time_stamp
	vme_read_a32({5'h0, address_vsram_master, 21'h2, 2'h0}, 32'hz, data);
	$display("The trigger number at the beginning was %d", data);
	vme_read_a32({5'h0, address_vsram_master, 21'h3, 2'h0}, 32'hz, data);	
	$display("The time stamp at the beginning was %d", data);
	//arming the trigger
	vme_write_a32({5'h0, address_vsram_master, 21'h1, 2'h0}, {8'h01, 24'h0}, data);
	//sending a trigger
	vme_write_a32({5'h0, address_trigger_man, 21'h1, 2'h0}, 32'h3, data);			
	//reading the status register checking when the first 512 points have been acquired and then transferring the data
	#500_000;
	vme_read_a32({5'h0, address_vsram_master, 21'h0, 2'h0}, 32'hz, data);		
	while (data[31:14]<511) #500_000 vme_read_a32({5'h0, address_vsram_master, 21'h0, 2'h0}, 32'hz, data);
	text_file = $fopen("raw_out.txt");
	for (i=0; i<512; i=i+1) begin
		vme_read_a32({5'h0, address_vsram_master, 1'h0, 1'b1, 1'b0, i, 2'h0}, 32'hz, data);	
		real_part	= data;
		vme_read_a32({5'h0, address_vsram_master, 1'h0, 1'b1, 1'b1, i, 2'h0}, 32'hz, data);
		$fdisplay(text_file, "%h	%h", real_part, data);	
	end
	//reading the status register checking when the acquisition is done and than transferring the last 512 data points
	vme_read_a32({5'h0, address_vsram_master, 21'h0, 2'h0}, 32'hz, data);		
	while (data[31:14]<1023) #3000 vme_read_a32({5'h0, address_vsram_master, 21'h0, 2'h0}, 32'hz, data);
	for (i=512; i<1024; i=i+1) begin
		vme_read_a32({5'h0, address_vsram_master, 1'h0, 1'b1, 1'b0, i, 2'h0}, 32'hz, data);	
		real_part	= data;
		vme_read_a32({5'h0, address_vsram_master, 1'h0, 1'b1, 1'b1, i, 2'h0}, 32'hz, data);
		$fdisplay(text_file, "%h	%h", real_part, data);	
	end
	$fclose(text_file);
	//reading the status register checking when the FFT data is ready and then transferring the data
	#50_000;
	vme_read_a32({5'h0, address_vsram_master, 21'h0, 2'h0}, 32'hz, data);		
	while (~data[8]) #50_000 vme_read_a32({5'h0, address_vsram_master, 21'h0, 2'h0}, 32'hz, data);
	text_file = $fopen("fft_out.txt");
	for (i=0; i<1024; i=i+1) begin
		vme_read_a32({5'h0, address_vsram_master, 1'h0, 1'b1, 1'b0, i, 2'h0}, 32'hz, data);	
		real_part	= data;
		vme_read_a32({5'h0, address_vsram_master, 1'h0, 1'b1, 1'b1, i, 2'h0}, 32'hz, data);
		$fdisplay(text_file, "%h	%h", real_part, data);	
	end
	//reading the trigger number and the time_stamp
	vme_read_a32({5'h0, address_vsram_master, 21'h2, 2'h0}, 32'hz, data);
	$display("The trigger number at the end was %d", data);
	vme_read_a32({5'h0, address_vsram_master, 21'h3, 2'h0}, 32'hz, data);	
	$display("The time stamp at the end was %d", data);
end
endtask


//***************************************
// SIMULATION CONTROL FLOW
//***************************************

initial	$readmemh("cos_mem.txt", glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.w_generator_i.cos_mem_i.mem);
initial	$readmemh("sin256.txt", glbl.page1_dab64x_i.page3_stratrix_controller.codec_control_i.freq_synthesizer_left_i.mem);
initial	$readmemh("sin256.txt", glbl.page1_dab64x_i.page3_stratrix_controller.codec_control_i.freq_synthesizer_right_i.mem);

initial begin
#1000;

force glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.X1_r = 34'h1_ffff_ffff;
force glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.X1_i = 34'h1_ffff_ffff;
test_FFT;
release glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.X1_r;
release glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.X1_i;
force glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.X1_r = 34'h2_0000_0000;
force glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.X1_i = 34'h2_0000_0000;
test_FFT;
release glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.X1_r;
release glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.X1_i;
force glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.X1_r = 34'h1_ffff_ffff;
force glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.X1_i = 34'h2_0000_0000;
test_FFT;
release glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.X1_r;
release glbl.page1_dab64x_i.page3_stratrix_controller.FFT_module.X1_i;


#1000;	
$stop;
end

endmodule
